/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.app;

import java.awt.event.ActionListener;
import java.io.IOException;
import java.sql.SQLException;
import java.util.HashSet;
import java.util.Set;

import net.java.ao.DBParam;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.database.Database;
import edu.wpi.wpisuite.database.DatabaseConfig;

/**
 * Controls the logic of the database chooser window
 */
public class DatabaseChooserController {
	private DatabaseChooserView view;

	public DatabaseChooserController(String errorMessage) {
		view = new DatabaseChooserView(this);

		view.setUrl(DatabaseConfig.instance().url());
		view.setUsername(DatabaseConfig.instance().username());
		// view.setPassword(DatabaseConfig.instance().password());

		if (errorMessage != null)
			view.error(errorMessage, null);
	}

	public void applySettings() {
		String username = view.getUsername();
		String url = view.getUrl();
		String password = view.getPassword();

		try {
			Database.testConnection(username, password, url);
		} catch (Exception e) {
			view
					.error(
							"Unable to connect to the chosen database. Make sure your input is formatted correctly - check the tooltips for help.",
							e);
			return;
		}

		try {
			DatabaseConfig.instance().setConfig(username, password, url);
		} catch (IOException e) {
			view.error("Unable to set the database configuration file", e);
			return;
		}

		try {
			Database.reinitializeManager();
		} catch (IOException e) {
			view.error("An error occured while initializing the connection", e);
			return;
		}

		try {
			Database.validateDatabase();
		} catch (Exception e) {
			boolean configure = view
					.askBoolean("This database does not appear to be configured for use with WPI Suite. Would you like to configure it now?");

			if (configure)
				configureDatabase();
			else
				return;
		}

		close();
	}

	private void configureDatabase() {
		try {
			Database.migrateTables();
		} catch (Exception e) {
			view.error("An error occured while initializing the database.", e);
		}

		try {
			if (Database.manager().find(User.class, "username = ?", "admin").length > 0) {
				// There already exists an admin user. No need to make a new
				// one.
			} else {
				User admin = Database.manager().create(User.class,
						new DBParam("username", "admin"));
				admin.setAdmin(true);
				admin.setFirstName("Admin");
				admin.setLastName("Admin");
				admin.setPasswordExpired(true);
				admin.setPasswordHash(Database.hashPassword("admin"));
				admin.setEmailAddress("admin@example.com");
				admin.save();

				view
						.showMessage("The database was initialized successfully.\n\n"
								+ "A default administrator has been created with username and password both set to \"admin\"\n\n"
								+ "Please set the system email settings in the System Administration module.");
			}
		} catch (SQLException e) {
			view
					.error(
							"An error occured while creating the default administrator user.",
							e);
		}
	}

	public void cancel() {
		close();
	}

	private void close() {
		view.close();
	}

	private Set<ActionListener> closeListeners = new HashSet<ActionListener>();

	public void addCloseListener(ActionListener actionListener) {
		closeListeners.add(actionListener);
	}

	public void removeCloseListener(ActionListener actionListener) {
		closeListeners.remove(actionListener);
	}

	public void onClose() {
		for (ActionListener l : closeListeners)
			l.actionPerformed(null);
	}
}
