/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.app;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.HashSet;
import java.util.Set;

import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.data.funcs.LoginFunctions;
import edu.wpi.wpisuite.data.funcs.UserManager;
import edu.wpi.wpisuite.database.Database;
import edu.wpi.wpisuite.database.DatabaseConfig;
import edu.wpi.wpisuite.module.ActiveState;

/**
 * Contains the logic behind logging into the application. Can reference out to
 * a DatabaseChooserController if the user wishes to.
 */
public class LoginController {
	private LoginView view;
	private ActiveState state;

	public LoginController(ActiveState state) {
		this.state = state;
		view = new LoginView(this);

		try {
			DatabaseConfig.instance().getConfig();
			DatabaseConfig.instance().validateConfig();
			view.setVisible(true); // only do this if we're okay with config
		} catch (Exception e) {
			settings(e.getMessage());
		}
	}

	private void close() {
		view.close();
	}

	/**
	 * Creates a new user dialog box
	 */
	public void newUser() {
		new NewUserView(view);
	}

	public void settings() {
		settings(null);
	}

	public void settings(String errorMessage) {
		DatabaseChooserController chooser = new DatabaseChooserController(
				errorMessage);
		chooser.addCloseListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				view.setVisible(true);
			}
		});
		view.setVisible(false);
	}

	private Set<ActionListener> closeListeners = new HashSet<ActionListener>();

	public void addCloseListener(ActionListener actionListener) {
		closeListeners.add(actionListener);
	}

	public void removeCloseListener(ActionListener actionListener) {
		closeListeners.remove(actionListener);
	}

	public void onClose() {
		for (ActionListener l : closeListeners)
			l.actionPerformed(null);
	}

	public void forgotPassword() {
		String username = view.getInput("Enter your username");
		if (username != null)
			try {
				User[] possibleUsers = Database.manager().find(User.class,
						"username LIKE ?", username);
				if (possibleUsers.length < 1)
					throw new IOException("No user with username " + username
							+ " exists.");
				User thisUser = possibleUsers[0];
				String tempPW = UserManager.generatePassword();
				UserManager
						.emailPassword(
								thisUser.getEmailAddress(),
								thisUser.getUsername(),
								tempPW,
								thisUser.getFirstName() + " "
										+ thisUser.getLastName(),
								"Password Reset notification for username: "
										+ thisUser.getUsername(),
								"You have chosen to reset your password. Here is your new, temporary password: ");
				thisUser.setPasswordHash(Database.hashPassword(tempPW));
				thisUser.setPasswordExpired(true);
				thisUser.setLocked(false);
				thisUser.save();
				tempPW = "";
				view
						.showMessage("User "
								+ username
								+ "'s password has been reset. The new temporary password has been emailed to the email address on file.");
			}

			catch (Exception e) {
				view.error(e.getMessage());
			}
	}

	/**
	 * Attempts to log in, given the authentication from the login window
	 * 
	 * @param username
	 *            The username to try authenticating with
	 * @param password
	 *            The password to try authenticating with
	 */
	public void login(String username, String password) {
		User newUser = null;
		try {
			newUser = LoginFunctions.Login(username, password);
		} catch (IOException e) {
			view.error(e.getMessage());
			return;
		}
		// Login succeeded.

		// Set state
		state.setState(newUser, newUser.getLastOpenProject());

		if (newUser.isPasswordExpired())
			expirePassword(newUser);
		else
			close();
	}

	private void expirePassword(User newUser) {
		view.setVisible(false);
		PasswordExpiredController expController = new PasswordExpiredController(
				newUser);
		expController.addCloseListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				close();
			}
		});
	}

	public void open() {
		view.setVisible(true);
	}
}
