/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Ian Williams
 *    Alex Kindle
 *    Bryan Crabree
 *    Dan Bjorge
 *    Mihai Tiberiu Luca
 *******************************************************************************/

package edu.wpi.wpisuite.app;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

/**
 * A login window for WPISuiteView. This is a special component of WPISuite that
 * DOES NOT extend Module, since it is built into WPISuite and can't be removed.
 */
public class LoginView extends JFrame {
	private static final long serialVersionUID = 4275667334105580488L;
	private LoginController controller;

	/**
	 * Initializes this object - sets up components and listeners and puts at
	 * center
	 */
	public LoginView(LoginController loginController) {
		this.controller = loginController;

		this.initComponents();
		this.initListeners();

		this.setTitle("Login");
		this.setIconImage(new ImageIcon("resources/WPISuite-128x128.png")
				.getImage());

		// Get the current screen size to center the window
		int sizex = getSize().width;
		int sizey = getSize().height;
		Toolkit toolkit = Toolkit.getDefaultToolkit();
		Dimension scrnsize = toolkit.getScreenSize();
		this.setLocation((scrnsize.width - sizex) / 2,
				(scrnsize.height - sizey) / 2);

		this.setVisible(false);
	}

	public void close() {
		controller.onClose();
		this.setVisible(false);
	}

	public void error(String s) {
		JOptionPane.showMessageDialog(null, s, "Error",
				JOptionPane.ERROR_MESSAGE);
	}

	/**
	 * Listeners that clear out default text in login window.
	 */
	private void initListeners() {

		this.newUserButton.addActionListener(newUserListener);
		this.settingsButton.addActionListener(settingsListener);
		this.forgotPasswordButton.addActionListener(forgotPasswordListener);
		this.loginButton.addActionListener(loginListener);
		this.usernameTextField.addActionListener(loginListener);
		this.passwordField.addActionListener(loginListener);
	}

	private ActionListener settingsListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent evt) {
			controller.settings();
		}
	};

	private ActionListener newUserListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent evt) {
			controller.newUser();
		}
	};

	private ActionListener forgotPasswordListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent evt) {
			controller.forgotPassword();
		}
	};

	private ActionListener loginListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent evt) {
			loginButtonActionPerformed(evt);
		}
	};

	/**
	 * Function called when the "login" button is pushed
	 * 
	 * @param evt
	 */
	protected void loginButtonActionPerformed(java.awt.event.ActionEvent evt) {
		this.loginButton.setEnabled(false);
		this.passwordField.setEnabled(false);

		controller.login(usernameTextField.getText(), new String(passwordField
				.getPassword()));

		this.loginButton.setEnabled(true);
		this.passwordField.setEnabled(true);

		this.usernameTextField.grabFocus();
	}

	public String getInput(String question) {
		return javax.swing.JOptionPane.showInputDialog(this, question, "",
				JOptionPane.QUESTION_MESSAGE);
	}

	public void showMessage(String message) {
		JOptionPane.showMessageDialog(this, message);
	}

	/**
	 * Initializes all components.
	 */
	private void initComponents() {

		usernameLabel = new javax.swing.JLabel();
		passwordLabel = new javax.swing.JLabel();
		usernameTextField = new javax.swing.JTextField();
		passwordField = new javax.swing.JPasswordField();
		loginLabel = new javax.swing.JLabel();
		loginButton = new javax.swing.JButton();
		settingsButton = new javax.swing.JButton();
		newUserButton = new javax.swing.JButton();
		forgotPasswordButton = new javax.swing.JButton();

		setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
		setResizable(false);

		usernameLabel.setText("Username:");

		passwordLabel.setText("Password:");

		loginLabel.setFont(new java.awt.Font("Tahoma", 0, 18)); // NOI18N
		loginLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
		loginLabel.setText("WPI Suite Login");

		loginButton.setText("<html>Log In<br>&nbsp;</html>");
		loginButton
				.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
		loginButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);

		settingsButton.setText("<html>Settings<br>&nbsp;</html>");
		settingsButton
				.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
		settingsButton
				.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);

		newUserButton.setText("<html>New User<br>&nbsp;</html>");
		newUserButton
				.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
		newUserButton
				.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);

		forgotPasswordButton
				.setText("<html><center>Forgot<br>Password</center></html>");
		forgotPasswordButton
				.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
		forgotPasswordButton
				.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(
				getContentPane());
		getContentPane().setLayout(layout);
		layout
				.setHorizontalGroup(layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								layout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																loginLabel,
																javax.swing.GroupLayout.Alignment.TRAILING,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																344,
																Short.MAX_VALUE)
														.addGroup(
																layout
																		.createSequentialGroup()
																		.addGroup(
																				layout
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.LEADING,
																								false)
																						.addComponent(
																								passwordLabel,
																								javax.swing.GroupLayout.DEFAULT_SIZE,
																								javax.swing.GroupLayout.DEFAULT_SIZE,
																								Short.MAX_VALUE)
																						.addComponent(
																								usernameLabel,
																								javax.swing.GroupLayout.DEFAULT_SIZE,
																								javax.swing.GroupLayout.DEFAULT_SIZE,
																								Short.MAX_VALUE))
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addGroup(
																				layout
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.LEADING)
																						.addComponent(
																								passwordField,
																								javax.swing.GroupLayout.DEFAULT_SIZE,
																								288,
																								Short.MAX_VALUE)
																						.addComponent(
																								usernameTextField,
																								javax.swing.GroupLayout.DEFAULT_SIZE,
																								288,
																								Short.MAX_VALUE)))
														.addGroup(
																layout
																		.createSequentialGroup()
																		.addComponent(
																				newUserButton)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addComponent(
																				forgotPasswordButton)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addComponent(
																				settingsButton)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addComponent(
																				loginButton)))
										.addContainerGap()));
		layout
				.setVerticalGroup(layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								layout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(loginLabel)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																usernameLabel)
														.addComponent(
																usernameTextField,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																passwordLabel)
														.addComponent(
																passwordField,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																newUserButton)
														.addComponent(
																forgotPasswordButton)
														.addComponent(
																settingsButton)
														.addComponent(
																loginButton))
										.addContainerGap(
												javax.swing.GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE)));

		loginButton.setIcon(new ImageIcon("resources/LoginIcons/login.png"));
		forgotPasswordButton.setIcon(new ImageIcon(
				"resources/LoginIcons/forgotPassword.png"));
		settingsButton.setIcon(new ImageIcon(
				"resources/LoginIcons/settings.png"));
		newUserButton
				.setIcon(new ImageIcon("resources/LoginIcons/newUser.png"));

		pack();
	}// </editor-fold>

	/**
	 * @param args
	 *            the command line arguments
	 */

	// Variables declaration - do not modify
	private javax.swing.JButton forgotPasswordButton;
	private javax.swing.JButton loginButton;
	private javax.swing.JLabel loginLabel;
	private javax.swing.JButton newUserButton;
	private javax.swing.JPasswordField passwordField;
	private javax.swing.JLabel passwordLabel;
	private javax.swing.JButton settingsButton;
	private javax.swing.JLabel usernameLabel;
	private javax.swing.JTextField usernameTextField;
	// End of variables declaration
}