/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Lucas Scotta - Initial design and implementation
 *    Ian Williams - Initial design and implementation
 *    Dan Bjorge - Additional implementation (modularity, menus)
 *    Bryan Crabtree - Additional implementation (menus)
 *    Mihai Tiberiu Luca
 *    Mihajlo Zeljkovic
 *******************************************************************************/

package edu.wpi.wpisuite.app;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

import javax.swing.UIManager;

import edu.wpi.wpisuite.log.LoggerInitializer;
import edu.wpi.wpisuite.module.ActiveState;
import edu.wpi.wpisuite.module.Module;
import edu.wpi.wpisuite.module.dashboard.Dashboard;


/**
 * The controller for the main application frame.
 * 
 * This is the main entry point for the program.
 */
public class WPISuite {
	// private Logger logger = Logger.getLogger(this.getClass());

	private ActiveState activeState;
	private WPISuiteView mainView;
	private LoginController loginController;

	/**
	 * Constructor for the application controller For now, this uses a
	 * hard-coded user login and state restore
	 */
	public WPISuite() {
		this.activeState = ActiveState.getInstance();
		this.initModules();
		this.mainView = new WPISuiteView(this);
		this.mainView.initializeModules();

		this.mainView.finishInit();

		this.loginController = new LoginController(activeState);
		loginController.addCloseListener(loginCloseListener);
	}

	/**
	 * What is the current state of the application?
	 * 
	 * @return Current state
	 */
	public ActiveState getState() {
		return this.activeState;
	}

	/**
	 * Is there currently a user successfully logged in?
	 * 
	 * Convenience method for existence of getState().getUser().
	 * 
	 * @return Whether there is currently a user logged in.
	 */
	public boolean isLoggedIn() {
		return this.getState().getUser() != null;
	}

	/**
	 * Sets the main that will be representing this application
	 * 
	 * @param view
	 *            The view to set
	 */
	public void setMainView(WPISuiteView view) {
		this.mainView = view;
	}

	/**
	 * @return The main WPISuiteView representing this application
	 */
	public WPISuiteView getMainView() {
		return this.mainView;
	}

	/**
	 * Closes down the application
	 */
	public void close() {
		System.exit(0);
	}

	/**
	 * Pops a login window and waits for a user to try to authenticate
	 */
	public void login() {
		this.getMainView().setVisible(false);
		this.getState().setProject(null);
		this.getState().setUser(null);
		this.loginController.open();
	}

	private ActionListener loginCloseListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent evt) {
			getMainView().setVisible(true);
			getMainView().setFocusable(true);
			getMainView().updateModulesVisibility();
		}
	};

	/** A map of all known modules from their classes */
	private Map<Class<? extends Module>, Module> modules;

	/**
	 * Initializes the module map with all known modules
	 */
	private void initModules() {
		ModuleLoader loader = new ModuleLoader();
		modules = loader.initModuleMap(getState());
	}

	/**
	 * @return A list of the known modules, sorted by name
	 */
	public List<Module> getModules() {
		List<Module> mods = new ArrayList<Module>(modules.values());
		Collections.sort(mods, new Comparator<Module>() {
			@Override
			public int compare(Module a, Module b) {
				return a.getModuleName().compareTo(b.getModuleName());
			}
		});
		return mods;
	}

	/**
	 * @return The module to have selected by default, if possible
	 */
	public Module getDefaultModule() {
	    Module ret = modules.get(Dashboard.class);
	    if (ret != null) {
		return ret;
	    } else {
		System.out.println("Dashboard not exists in modules map, it seems");
	    }
	    return null;
	}
    /**
     * main() replacement.
     *
     */
    public static void startprogram() {
	LoggerInitializer.initializeLoggers();
	try {
	    // Set System L&F
	    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
	} catch (Exception e) {
	    // No System L&F - keep default
	}
	
	new WPISuite();
    }

	/**
	 * Create and run the application
	 * 
	 * @param args
	 *            None accepted
	 */
	public static void main(String[] args) {
	    startprogram();
	}
}
