/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Lucas Scotta - Initial design and implementation
 *    Ian Williams - Initial design and implementation
 *    Dan Bjorge - Initial design and implementation
 *    Sabrina Varanelli
 *    Bryan Crabtree - GUI design
 *    Mihai Tiberiu Luca
 *    Mihajlo Zeljkovic
 *******************************************************************************/

package edu.wpi.wpisuite.app;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.util.HashMap;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;


import edu.wpi.wpisuite.components.DnDTabbedPane;
import edu.wpi.wpisuite.components.EntityIconSet;
import edu.wpi.wpisuite.components.EntityIconSet.IconType;
import edu.wpi.wpisuite.module.ActiveStateEvent;
import edu.wpi.wpisuite.module.ActiveStateListener;
import edu.wpi.wpisuite.module.IActiveModuleController;
import edu.wpi.wpisuite.module.Module;
import edu.wpi.wpisuite.module.ModuleView;
import edu.wpi.wpisuite.module.Status;
import edu.wpi.wpisuite.module.ActiveStateEvent.ActiveStateEventType;

/**
 * The GUI for the main application frame
 * 
 * Right now this interfaces with other modules. Ideally, we don't want this.
 * Eventually we will have an API that connects WPISuite and WPISuiteView with
 * other components in a safe way.
 */
public class WPISuiteView extends JFrame {
	private static final long serialVersionUID = -1170199065937769866L;

	private EntityIconSet icons = new EntityIconSet();
	private WPISuite wpiSuite;
	private JMenuBar mainMenuBar;
	private StatusBar statusBar;
	private JMenu modulesMenu;
	private JMenu addMenu;
	private JMenu fileMenu;
	// private JMenu settingsMenu;
	private JTabbedPane tabbedPane;
	private Map<ModuleView, ItemListener> moduleTabAdders;
	private Map<ModuleView, JCheckBoxMenuItem> moduleViewMenuItemAssociation;
	private Map<ModuleView, JMenuItem[]> moduleViewAddMenuItemsAssociation; // TODO
																			// go
																			// through
																			// code
																			// and
																			// use
																			// this
																			// convenience
																			// map

	/**
	 * Initializes the components, centers the window, sizes to default
	 * 
	 * @param wpiSuite
	 *            The WPISuite controller object associated with this WPISuite
	 */
	public WPISuiteView(WPISuite wpiSuite) {

		super("WPI Suite");
		this.wpiSuite = wpiSuite;
		this.setLayout(new BorderLayout());
		this.addControls();

		this.setIconImage(new ImageIcon("resources/WPISuite-128x128.png")
				.getImage());
		this.setDefaultCloseOperation(EXIT_ON_CLOSE);

		this.wpiSuite.getState().registerListener(new ActiveStateListener() {
			@Override
			public void stateChange(ActiveStateEvent evt) {
				if (evt.getType().equals(ActiveStateEventType.PROJECT)) {
					String title = "WPI Suite";
					if (evt.getSource().getProject() != null)
						title += " - " + evt.getSource().getProject().getName();
					setTitle(title);
					updateModulesVisibility();
				}
			}
		});

		// Get the current screen size to center the window
		int sizex = 1024;
		int sizey = 768;

		Toolkit toolkit = Toolkit.getDefaultToolkit();
		Dimension scrnsize = toolkit.getScreenSize();
		this.setSize(sizex, sizey);
		this.setLocation((scrnsize.width - sizex) / 2,
				(scrnsize.height - sizey) / 2);

	}

	/**
	 * Initializes the Modules that are "installed" into this WPISuite. Note: As
	 * of right now, we are hard coding in the modules. Later, there will be
	 * some mechanism for dynamically determining what modules are presents and
	 * dealing with them.
	 */
	public void initializeModules() {
		// Add modules' Adders to the Add menu
		// Also add modules to the Modules menu
		this.moduleTabAdders = new HashMap<ModuleView, ItemListener>();
		this.moduleViewMenuItemAssociation = new HashMap<ModuleView, JCheckBoxMenuItem>();
		this.moduleViewAddMenuItemsAssociation = new HashMap<ModuleView, JMenuItem[]>();

		for (final Module module : getWpiSuite().getModules()) {
			final ModuleView moduleView = module.getView();
			final JCheckBoxMenuItem moduleMenuItem = new JCheckBoxMenuItem(
					module.getModuleName());
			moduleMenuItem.setMnemonic(module.getModuleKeyShortcut());

			moduleMenuItem.getAccessibleContext().setAccessibleDescription(
					module.getMenuDescription());

			this.modulesMenu.add(moduleMenuItem);
			this.moduleViewMenuItemAssociation.put(moduleView, moduleMenuItem);

			ItemListener moduleTabAdder = new ItemListener() {

				@Override
				public void itemStateChanged(ItemEvent evt) {

					// Hide
					if (evt.getStateChange() == ItemEvent.DESELECTED) {

						int tabIndex = WPISuiteView.this.tabbedPane
								.indexOfComponent(moduleView);

						if (tabIndex != -1) {
							WPISuiteView.this.tabbedPane.remove(tabIndex);
							for (JMenuItem addMenuItem : moduleView
									.getAddMenuItems())
								WPISuiteView.this.addMenu.remove(addMenuItem);
						}
					}

					// Show
					else {

						int tabIndex = WPISuiteView.this.tabbedPane
								.indexOfComponent(moduleView);

						if (tabIndex == -1) {

							if (moduleView.getIcon() == null)
								WPISuiteView.this.tabbedPane.addTab(module
										.getModuleName(), moduleView);
							else
								WPISuiteView.this.tabbedPane.addTab(module
										.getModuleName(), moduleView.getIcon(),
										moduleView);
							WPISuiteView.this.tabbedPane
									.setSelectedComponent(moduleView);
							moduleView
									.setPreferredSize(WPISuiteView.this.tabbedPane
											.getSize());
							moduleView.onAddToTabPane();

							JMenuItem[] addMenuItems = new JMenuItem[moduleView
									.getAddMenuItems().size()];
							int i = 0;
							for (JMenuItem addMenuItem : moduleView
									.getAddMenuItems()) {
								addMenuItem
										.addActionListener(new ActionListener() {
											@Override
											public void actionPerformed(
													ActionEvent e) {
												WPISuiteView.this.tabbedPane
														.setSelectedComponent(moduleView);
											}
										});
								addMenuItems[i++] = addMenuItem;
								WPISuiteView.this.addMenu.add(addMenuItem);
							}
							WPISuiteView.this.moduleViewAddMenuItemsAssociation
									.put(moduleView, addMenuItems);
						} else
							WPISuiteView.this.tabbedPane
									.setSelectedComponent(moduleView);
					}

				}
			};

			this.moduleTabAdders.put(moduleView, moduleTabAdder);

			moduleMenuItem.addItemListener(moduleTabAdder);

			moduleMenuItem.doClick();
		}

		// Set default tab
		WPISuite amanda = getWpiSuite();
		if (amanda != null) {
		    Module molly = amanda.getDefaultModule();
		    if (molly != null) {
			tabbedPane.setSelectedComponent(molly.getView());
		    } else {
			System.out.println("molly is null. fuck.");
		    }
		} else {
		    System.out.println("amanda is null. fuck.");
		}
	}

	/**
	 * Adds active state listener to update all modules upon a change in the
	 * active state
	 */
	public void finishInit() {
		this.tabbedPane.addChangeListener(new ChangeListener() {
			// TODO I think I can do a little better than this...
			@Override
			public void stateChanged(ChangeEvent arg0) {
				Component c = WPISuiteView.this.tabbedPane
						.getSelectedComponent();
				if (c instanceof ModuleView) {
					((ModuleView) c).updateAllComponents();
				}
			}
		});
		this.wpiSuite.getState().setModuleController(new IActiveModuleController() {
			public Module getActiveModule() {
				return ((ModuleView) WPISuiteView.this.tabbedPane.getSelectedComponent()).getModule();
			}
		});
	}

	/**
	 * @return The main application object
	 */
	public WPISuite getWpiSuite() {
		return this.wpiSuite;
	}

	public void logoutItemActionPerformed(ActionEvent evt) {
		this.getWpiSuite().login();
	}

	public void editProfileItemActionPerformed(ActionEvent evt) {
		EditProfileView view = new EditProfileView(this.wpiSuite);
		view.setVisible(true);
		view.setFocusable(true);
	}

	public void quitItemActionPerformed(ActionEvent evt) {
		System.exit(0);
	}

	/**
	 * Looks at the currently logged in user's system administrator status, and
	 * updates which modules are shown accordingly.
	 */
	public void updateModulesVisibility() {
		for (Module m : getWpiSuite().getModules())
			if (m.shouldBeVisible()) {
				showModule(m.getView());
				setModuleEnabled(m.getView(), m.shouldBeEnabled());
			} else
				hideModule(m.getView());

		/*
		 * User u = wpiSuite.getState().getUser();
		 * 
		 * if (u == null) {System.out.println(
		 * "Critical Error: check WPISuiteView:updateModulesVisibility()");
		 * return; // This should never happen... otherwise no user is logged //
		 * in }
		 * 
		 * if (lastUserWasAdmin != u.isAdmin()) { if (u.isAdmin()) { for (Module
		 * m : getWpiSuite().getModules()) { showModule(m.getView()); } } else {
		 * for (Module m : getWpiSuite().getModules()) { if
		 * (m.requriedSysAdmin()) { hideModule(m.getView()); } else {
		 * showModule(m.getView()); } } } } lastUserWasAdmin = u.isAdmin();
		 */
	}

	/**
	 * Sets whether a given module view should be enabled for use
	 * 
	 * @param moduleView
	 *            The view for the module in question
	 * @param enabled
	 *            Whether that module should be enabled
	 */
	private void setModuleEnabled(ModuleView moduleView, boolean enabled) {
		for (int i = 0; i < tabbedPane.getTabCount(); i++)
			if (tabbedPane.getComponentAt(i) == moduleView)
				tabbedPane.setEnabledAt(i, enabled);

		for (JMenuItem addMenuItem : moduleView.getAddMenuItems())
			addMenuItem.setEnabled(enabled);
	}

	/**
	 * Hides the given ModuleView. The menu item is hidden and the tab is
	 * removed for WPISuite.
	 */
	private void hideModule(ModuleView moduleView) {
		Component[] tabs = tabbedPane.getComponents();

		// Make menu item invisible
		moduleViewMenuItemAssociation.get(moduleView).setVisible(false);

		// Remove tab if it is there
		for (Component c : tabs)
			if (c == moduleView)
				tabbedPane.remove(moduleView);

		/*
		 * Remove the add menu items associated with this module TODO this is
		 * bad... what if another module wants this item there? But this problem
		 * is much bigger than the scope of this function anyway Would require
		 * WPISuite to set more standard for adding functions... in fact,
		 * WPISuite would probably know about the code that handles it in any
		 * case, this will be better after the CRUD functions are moved
		 */
		for (JMenuItem it : moduleViewAddMenuItemsAssociation.get(moduleView))
			addMenu.remove(it);
	}

	/**
	 * Shows the given ModuleView. The menu item is made visible and a tab is
	 * added.
	 * 
	 * @param moduleView
	 */
	private void showModule(ModuleView moduleView) {

		Component[] tabs = tabbedPane.getComponents();

		// Make module menu item visible
		moduleViewMenuItemAssociation.get(moduleView).setVisible(true);

		// Add tab if not already there
		boolean alreadyThere = false;
		for (Component c : tabs)
			if (c == moduleView) {
				alreadyThere = true;
				break;
			}
		if (!alreadyThere)
			if (moduleView.getIcon() == null)
				WPISuiteView.this.tabbedPane.addTab(moduleView.getModule()
						.getModuleName(), moduleView);
			else
				WPISuiteView.this.tabbedPane.addTab(moduleView.getModule()
						.getModuleName(), moduleView.getIcon(), moduleView);

		// Show the add menu items associated with this module if not already
		// there
		for (JMenuItem it : moduleViewAddMenuItemsAssociation.get(moduleView))
			addMenu.add(it);
	}

	/**
	 * Initializes all components. Newly added components should be initialized
	 * here as well.
	 */
	private void addControls() {
		Container contentPane = this.getContentPane();

		// Create the menu bar.
		this.mainMenuBar = new JMenuBar();

		// Build the file menu
		this.fileMenu = new JMenu("File");
		this.fileMenu.setMnemonic(KeyEvent.VK_F);
		this.fileMenu.getAccessibleContext().setAccessibleDescription("File");
		JMenuItem profileItem = new JMenuItem("Profile");
		profileItem.setIcon(icons.getUserIcon(IconType.DEFAULT));
		JMenuItem logoutItem = new JMenuItem("Logout");
		logoutItem.setIcon(icons.getLogoutIcon());
		JMenuItem quitItem = new JMenuItem("Quit");
		quitItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_Q,
				java.awt.event.InputEvent.CTRL_MASK));
		this.fileMenu.add(profileItem);
		this.fileMenu.add(logoutItem);
		this.fileMenu.addSeparator();
		this.fileMenu.add(quitItem);

		// Build the add menu
		this.addMenu = new JMenu("Add");
		this.addMenu.setMnemonic(KeyEvent.VK_A);
		this.addMenu.getAccessibleContext().setAccessibleDescription("Add");

		// Build the modules menu.
		this.modulesMenu = new JMenu("View");
		this.modulesMenu.setMnemonic(KeyEvent.VK_V);
		this.modulesMenu.getAccessibleContext().setAccessibleDescription(
				"Modules");

		// Add everything to the main menu
		this.mainMenuBar.add(this.fileMenu);
		this.mainMenuBar.add(this.addMenu);
		this.mainMenuBar.add(this.modulesMenu);

		// Add the main menu to the application
		this.setJMenuBar(this.mainMenuBar);

		logoutItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				WPISuiteView.this.logoutItemActionPerformed(evt);
			}
		});

		quitItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				WPISuiteView.this.quitItemActionPerformed(evt);
			}
		});

		profileItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				WPISuiteView.this.editProfileItemActionPerformed(evt);
			}
		});

		// Create the tabbed pane
		this.tabbedPane = new DnDTabbedPane();

		contentPane.add(this.tabbedPane, BorderLayout.CENTER);

		this.statusBar = StatusBar.getBar();
		this.statusBar.statusChanged(new Status("Welcome to WPI Suite!"));
		contentPane.add(this.statusBar, BorderLayout.SOUTH);

	}
}