/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components;

import java.util.ArrayList;
import java.util.List;

import javax.swing.DefaultComboBoxModel;

import net.java.ao.Entity;
import edu.wpi.wpisuite.data.funcs.Artifact;

/**
 * The data model for a combo box of Entities. Stores the entities separately
 * from their string representations, which allows the combo box to display a
 * useful string despite the entities not having useful toString methods, yet
 * still maintain a persistent model of the entities themselves rather than just
 * strings representing them.
 * 
 * Usually used in conjunction with an EntityComboBox
 * 
 * @param <T>
 *            The type of entity to store
 */
@SuppressWarnings("serial")
public class EntityComboBoxModel<T extends Entity> extends DefaultComboBoxModel {
	/** The entities stored by this model */
	protected List<T> ents = new ArrayList<T>();

	/**
	 * Constructs a new model based on the given entities
	 * 
	 * @param entities
	 *            The entities to initially store in this model
	 */
	public EntityComboBoxModel(T[] entities) {
		this();
		reset(entities);
	}

	/**
	 * Clears this model of entities
	 */
	public void clear() {
		ents.clear();
		super.removeAllElements();
	}

	/**
	 * Clears this model of entities, then sets it to use the given array of new
	 * ones
	 * 
	 * @param entities
	 *            The new entities to set the model to use
	 */
	public void reset(T[] entities) {
		this.clear();
		if (entities != null)
			for (int i = 0; i < entities.length; i++)
				this.addElement(entities[i]);
	}

	/**
	 * Initializes a blank data model
	 */
	public EntityComboBoxModel() {
		super();
	}

	/**
	 * Adds a new entity to the model, including adding a string representation
	 * thereof to the model used by the combo box.
	 * 
	 * @param ent
	 *            The entity to add
	 * @see Artifact#getSimpleName()
	 */
	public void addElement(T ent) {
		if (ent != null) {
			String listElement = (new Artifact(ent)).getSimpleName();
			this.ents.add(ent);
			this.addStringElement(listElement);
		} else
			this.addNullElement();

	}

	/**
	 * Adds a string to the underlying DefaultComboBoxModel for display. Should
	 * generally NOT be used without concurrently adding to ents
	 * 
	 * @param elem
	 *            The string to add to the underlying model
	 */
	protected void addStringElement(String elem) {
		super.addElement(elem);
	}

	/**
	 * Adds a null element, including an appropriate string representation in
	 * the underlying model
	 */
	protected void addNullElement() {
		this.ents.add(null);
		this.addStringElement("-None-");
	}

	/**
	 * @param index
	 *            The index of entity to get
	 * @return The Entity referenced by that index, or null if that index
	 *         references null OR the given index is out of bounds
	 */
	public T getEntity(int index) {
		try {
			return this.ents.get(index);
		} catch (IndexOutOfBoundsException e) {
			return null;
		}
	}

	/**
	 * @param ent
	 *            The entity to find
	 * @return The index of the entity in the model, or -1 if it isn't in the
	 *         model.
	 * @see List#indexOf(Object)
	 */
	public int getEntityIndex(T ent) {
		return this.ents.indexOf(ent);
	}
}
