/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *    Ravi Singh - added color to cell renderer
 *******************************************************************************/

package edu.wpi.wpisuite.components;

import java.awt.Color;
import java.util.List;

import net.java.ao.Entity;

/**
 * A JList customized to contain Entities of a specific type. This is useful
 * both because it allows for a useful string representation of an Entity (which
 * has no useful toString method) and because it allows for type safety when
 * dealing with the list.
 * 
 * Also uses a customized renderer to enable formatting of individual entities.
 * 
 * @param <T>
 *            The type of entity to list
 */
@SuppressWarnings("serial")
public class EntityList<T extends Entity> extends javax.swing.JList {
	/** The custom renderer for the display of entities in this list */
	private FontStyleListCellRenderer cellRenderer;

	/**
	 * Initializes a new, blank list of entities with an appropriate model and
	 * renderer
	 */
	public EntityList() {
		super();
		this.setDefaultModel();
		cellRenderer = new FontStyleListCellRenderer();
		this.setCellRenderer(cellRenderer);
	}

	/**
	 * Initializes this list's data model to a new, blank EntityListModel of the
	 * appropriate type
	 */
	private void setDefaultModel() {
		this.setModel(new EntityListModel<T>());
	}

	/**
	 * Sets this list's data model to the given one
	 * 
	 * @param model
	 *            The new data model to use
	 */
	public void setModel(EntityListModel<T> model) {
		super.setModel(model);
	}

	/**
	 * This function should not be used - only EntityListModels are appropriate
	 * for use with this class. This function is here purely to stop the
	 * superclass's version from allowing arbitrary model specification - if it
	 * is used, it will IGNORE the given model and instead use a newly
	 * instantiated EntityListModel of the appropriate type.
	 * 
	 * @deprecated Use {@link #setModel(EntityListModel)}
	 * @param model
	 *            Ignored
	 */
	@Override
	@Deprecated
	public void setModel(javax.swing.ListModel model) {
		setDefaultModel();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.JList#getModel()
	 */
	@Override
	@SuppressWarnings("unchecked")
	public EntityListModel<T> getModel() {
		try {
			return (EntityListModel<T>) super.getModel();
		} catch (Throwable t) {
			return null;
		}
	}

	/**
	 * @return The currently selected entity, or null if there is none
	 */
	public T getSelectedEntity() {
		int index = this.getSelectedIndex();
		return this.getModel().getEntity(index);
	}

	/**
	 * Sets the list selection to the given entity, or unselects if the entity
	 * is not in the list model
	 * 
	 * @param ent
	 *            The entity to select
	 */
	public void setSelectedEntity(T ent) {
		int index = this.getModel().getEntityIndex(ent);
		this.setSelectedIndex(index);
	}

	/**
	 * @return A list of all selected entities (an empty list if there are none)
	 */
	public List<T> getSelectedEntities() {
		return this.getModel().getEntities(this.getSelectedIndices());
	}

	/**
	 * Sets the given entity to appear in bold text in the list
	 * 
	 * @param ent
	 *            The entity to make bold
	 */
	public void setEntityBold(T ent) {
		cellRenderer.setBold(getModel().getEntityIndex(ent));
	}

	public void setEntityColor(T ent, Color c) {
		cellRenderer.setColor(getModel().getEntityIndex(ent), c);
	}

	/**
	 * Resets all entities in the list to the default font settings
	 */
	public void clearFontStyles() {
		cellRenderer.clearFontStyleMap();
	}

	public void clearColorSyles() {
		cellRenderer.clearColorMap();
	}

	/**
	 * Resets the list to contain no entities and no special font information
	 */
	public void clear() {
		getModel().clear();
		clearFontStyles();
		clearColorSyles();
	}
}
