/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components;

import java.util.ArrayList;
import java.util.List;

import javax.swing.DefaultListModel;

import net.java.ao.Entity;
import edu.wpi.wpisuite.data.funcs.Artifact;

/**
 * Models a list which is represented by and stores Entities, but actually
 * displays the Entity names. Generally used in conjunction with an EntityList
 * 
 * @param <T>
 *            The type of entity to store
 */
@SuppressWarnings("serial")
public class EntityListModel<T extends Entity> extends DefaultListModel {
	/** The list of stored entities */
	protected List<T> ents = new ArrayList<T>();

	/**
	 * Constructs a new list model with the given list of entities.
	 * 
	 * @param entities
	 *            The entities to initialize the model with
	 */
	public EntityListModel(T[] entities) {
		this();
		for (int i = 0; i < entities.length; i++)
			this.addElement(entities[i]);
	}

	/**
	 * Constructs a new list model containing no entities.
	 */
	public EntityListModel() {
		super();
	}

	/**
	 * Adds a new entity to the model, including adding a string representation
	 * of the entity to the underlying model
	 * 
	 * @param ent
	 *            The entity to add
	 * @see Artifact#getSimpleName()
	 */
	public void addElement(T ent) {
		if (ent != null) {
			this.ents.add(ent);
			this.addStringElement(format(ent));
		} else
			this.addNullElement();

	}

	/**
	 * Defines the function which converts an entity to a string. May be
	 * frequently overriden in subclasses.
	 * 
	 * @param ent
	 *            The entity to format
	 * @return The string resulting from the given entity
	 */
	public String format(T ent) {
		return new Artifact(ent).getSimpleName();
	}

	/**
	 * Adds a string element to the underlying list model. Should not be used
	 * unless the ents list is concurrently added to
	 * 
	 * @param elem
	 *            The string to add to the underlying model
	 */
	protected void addStringElement(String elem) {
		super.addElement(elem);
	}

	/**
	 * Adds a null element to the model, and an appropriate string to the
	 * underlying model
	 */
	protected void addNullElement() {
		this.ents.add(null);
		this.addStringElement("-None-");
	}

	/**
	 * Gets the entity at the given index, or null if the index is out of range
	 * OR if the given index contains a null entry
	 * 
	 * @param index
	 *            The index of the desired entity
	 * @return The entity at that index
	 */
	public T getEntity(int index) {
		try {
			return this.ents.get(index);
		} catch (IndexOutOfBoundsException e) {
			return null;
		}
	}

	/**
	 * Maps {@link #getEntity(int)} over the list of indices.
	 * 
	 * @param indices
	 *            The indices to get entities for
	 * @return A list of entities at the given indices
	 */
	public List<T> getEntities(int[] indices) {
		List<T> selectedEnts = new ArrayList<T>();
		for (int index : indices)
			selectedEnts.add(this.getEntity(index));
		return selectedEnts;
	}

	/**
	 * @return A list of all of the entities contained in this list model
	 */
	public List<T> getAllEntities() {
		return new ArrayList<T>(ents);
	}

	/**
	 * @param ent
	 *            The entity to get the index of
	 * @return The index of that entity in the model, or -1 if the entity is not
	 *         in the model
	 * @see List#indexOf(Object)
	 */
	public int getEntityIndex(T ent) {
		return this.ents.indexOf(ent);
	}

	/**
	 * Removes all entities from this model and the underlying string model
	 */
	@Override
	public void clear() {
		super.clear();
		ents.clear();
	}
}
