/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components;

import javax.swing.tree.DefaultMutableTreeNode;

import net.java.ao.Entity;
import edu.wpi.wpisuite.data.funcs.Artifact;

/**
 * A TreeNode (mostly for use with JTrees) which represents and stores an Entity
 * internally but displays as a useful string, despite Entities not actually
 * having useful toString methods. Also allows for type safety in tree nodes
 * 
 * @param <T>
 *            The type of Entity which this node will store
 */
@SuppressWarnings("serial")
public class EntityTreeNode<T extends Entity> extends DefaultMutableTreeNode {
	/** The entity this node represents */
	T ent;

	/**
	 * Constructs a new treenode based on the given entity
	 * 
	 * @param ent
	 *            The entity to base this node on
	 */
	public EntityTreeNode(T ent) {
		super();
		setUserObject(ent);
	}

	/**
	 * Sets this node to be represented by the given entity
	 * 
	 * @param ent
	 *            The entity to set this node to use
	 */
	public void setUserObject(T ent) {
		super.setUserObject(getDisplayObject(ent));
		this.ent = ent;
	}

	/**
	 * @return The entity representing this node
	 */
	@Override
	public T getUserObject() {
		return this.ent;
	}

	/**
	 * The converter between the entity and its representation as displayed for
	 * the user. Usually will return a String.
	 * 
	 * @param ent
	 *            The entity to convert for display.
	 * @return The object which this node will display as.
	 */
	protected Object getDisplayObject(T ent) {
		if (ent == null)
			return getNullDisplayObject();
		return new Artifact(ent).getSimpleName();
	}

	/**
	 * Determines what a null entity will display as. Usually will return a
	 * string.
	 * 
	 * @return The object which this null node will display as
	 */
	protected Object getNullDisplayObject() {
		return "null";
	}
}
