/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *    Ravi Singh - added methods for coloring
 *******************************************************************************/

package edu.wpi.wpisuite.components;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JLabel;
import javax.swing.JList;

/**
 * A list cell renderer which is capable of assigning different font styles to
 * different elements of a list. Maintains a map from list indices to font
 * styles, since different renderers cannot be used for the same list
 */
@SuppressWarnings("serial")
public class FontStyleListCellRenderer extends EntityListCellRenderer {
	/** Maps from list indexes to font style constants */
	private Map<Integer, Integer> fontStyleMap;
	private Map<Integer, Color> colorMap;

	/**
	 * Constructs a new renderer with no font style information set
	 */
	public FontStyleListCellRenderer() {
		super();
		this.fontStyleMap = new HashMap<Integer, Integer>();
		this.colorMap = new HashMap<Integer, Color>();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * javax.swing.DefaultListCellRenderer#getListCellRendererComponent(javax
	 * .swing.JList, java.lang.Object, int, boolean, boolean)
	 */
	@Override
	public Component getListCellRendererComponent(JList list, Object value,
			int index, boolean isSelected, boolean hasFocus) {
		JLabel comp = (JLabel) super.getListCellRendererComponent(list, value,
				index, isSelected, hasFocus);

		if (fontStyleMap.containsKey(index))
			comp.setFont(comp.getFont().deriveFont(fontStyleMap.get(index)));
		if (colorMap.containsKey(index))
			comp.setForeground(colorMap.get(index));

		return comp;
	}

	/**
	 * Resets all list entries to the default font style
	 */
	public void clearFontStyleMap() {
		this.fontStyleMap.clear();
	}

	public void clearColorMap() {
		this.colorMap.clear();
	}

	public void setColor(int index, Color c) {
		colorMap.put(index, c);
	}

	/**
	 * Sets the list entry at the given index to be displayed as bold
	 * 
	 * @param index
	 *            The index whose entry should be bold
	 */
	public void setBold(int index) {
		fontStyleMap.put(index, Font.BOLD);
	}

	/**
	 * Sets the list entry at the given index to be displayed in italics
	 * 
	 * @param index
	 *            The index whose entry should be italic
	 */
	public void setItalic(int index) {
		fontStyleMap.put(index, Font.ITALIC);
	}

	/**
	 * Resets the list entry at the given index to be displayed normally
	 * 
	 * @param index
	 *            The index whose entry should be returned to normal font
	 */
	public void setPlain(int index) {
		fontStyleMap.put(index, Font.PLAIN);
	}
}