/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components;

import java.util.ArrayList;
import java.util.List;

import javax.swing.DefaultComboBoxModel;

/**
 * Models a data model that maps the objects its storing to specific, predefined
 * string values instead of relying on the objects' toString() methods
 * 
 * @param <T>
 *            The type of object to map from in this model
 */
@SuppressWarnings("serial")
public class MappedComboBoxModel<T> extends DefaultComboBoxModel {
	/** Defines a mapping from an object to a string to display representing it */
	public static class Mapping<M> {
		/** The object to map from */
		private M key;

		/** The string to map to */
		private String value;

		/** Initializes a new mapping from the given key to the given value */
		public Mapping(M key, String value) {
			this.key = key;
			this.value = value;
		}

		/**
		 * @return The object being mapped from
		 */
		public M getKey() {
			return key;
		}

		/**
		 * @return The display string associated with the key object
		 */
		public String getValue() {
			return value;
		}
	}

	/** The mappings contained in this model */
	protected List<Mapping<T>> objs = new ArrayList<Mapping<T>>();

	/**
	 * Constructs a new data model containing the given mappings
	 * 
	 * @param mappings
	 *            The mappings to initialize this model with
	 */
	public MappedComboBoxModel(List<Mapping<T>> mappings) {
		this();
		for (Mapping<T> obj : mappings)
			this.addElement(obj);
	}

	/**
	 * Constructs a new, blank data model
	 */
	public MappedComboBoxModel() {
		super();
	}

	/**
	 * Adds a new object to the model for this object and the corresponding
	 * string to the the underlying ComboBoxModel
	 * 
	 * @param obj
	 *            The mapping to add
	 */
	public void addElement(Mapping<T> obj) {
		this.objs.add(obj);
		super.addElement(obj.getValue());
	}

	/**
	 * Gets the object (not its mapping) with the given index
	 * 
	 * @param index
	 *            The index of the desired object
	 * @return The object at the given index
	 */
	public T get(int index) {
		try {
			return this.objs.get(index).getKey();
		} catch (IndexOutOfBoundsException e) {
			return null;
		}
	}

	/**
	 * Returns the index of the first occurance of the given object, or -1 if
	 * the object is not in the model
	 * 
	 * @param obj
	 *            The object to search for
	 * @return The first index of the given object, or -1 if not found
	 */
	public int getIndex(T obj) {
		for (int i = 0; i < objs.size(); i++) {
			T key = objs.get(i).getKey();
			if (key == null ? obj == null : key.equals(obj))
				return i;
		}
		return -1;
	}
}
