/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components.checktree;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.JCheckBox;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import net.java.ao.Entity;

import edu.wpi.wpisuite.components.EntityTreeCellRenderer;
import edu.wpi.wpisuite.components.EntityTreeNode;

/**
 * A JTree which contains a checkbox at each node. The selection status of these
 * checkboxes is defined by {@link #getCheckSelectionModel()}.
 * 
 * This class is mostly a standard JTree with an additional TreeSelectionModel
 * for the checkbox selection.
 */
@SuppressWarnings("serial")
public class CheckTree extends JTree implements TreeSelectionListener {
	private CheckTreeSelectionModel checkSelectionModel;
	private DefaultTreeModel model;
	private DefaultMutableTreeNode root;

	public CheckTree() {
		this(new DefaultMutableTreeNode("All"));
	}

	public CheckTree(DefaultMutableTreeNode root) {
		super(root);

		checkSelectionModel = new CheckTreeSelectionModel(this);
		setCellRenderer(new CheckTreeCellRenderer(new EntityTreeCellRenderer(),
				checkSelectionModel));

		checkSelectionModel.addTreeSelectionListener(this);

		addMouseListener(checkBoxMouseListener);

		this.root = root;
		model = new DefaultTreeModel(root);

		this.setRootVisible(true);
		this.setModel(model);
	}

	@Override
	public void valueChanged(TreeSelectionEvent e) {
		treeDidChange();
	}

	private MouseListener checkBoxMouseListener = new MouseAdapter() {
		private int hotspot = new JCheckBox().getPreferredSize().width;

		@Override
		public void mouseClicked(MouseEvent me) {
			TreePath path = getPathForLocation(me.getX(), me.getY());
			if (path == null)
				return;
			if (me.getX() > getPathBounds(path).x + hotspot)
				return;

			boolean selected = checkSelectionModel.isPathSelected(path, true);
			checkSelectionModel.removeTreeSelectionListener(CheckTree.this);

			try {
				if (selected)
					checkSelectionModel.removeSelectionPath(path);
				else
					checkSelectionModel.addSelectionPath(path);
			} finally {
				checkSelectionModel.addTreeSelectionListener(CheckTree.this);
				treeDidChange();
			}
		}
	};

	/**
	 * @return The selection model for the checkboxes, as opposed to the actual
	 *         selections
	 */
	public CheckTreeSelectionModel getCheckingModel() {
		return checkSelectionModel;
	}

	/**
	 * Sets the given entities to be the top-level children (ie, direct children
	 * of the root node). Does not set up any children for these entities - to
	 * do that, use {@link #setRootChildren(List)} on nodes that have children
	 * defined already.
	 * 
	 * @param <T>
	 *            The type of entities being input
	 * @param entities
	 *            The entities to set as top-level root children
	 */
	public <T extends Entity> void setRootEntities(List<T> entities) {
		List<EntityTreeNode<T>> nodes = new ArrayList<EntityTreeNode<T>>();
		for (T ent : entities)
			nodes.add(new EntityTreeNode<T>(ent));
		setRootChildren(nodes);
	}

	@Override
	public void setModel(TreeModel model) {
		super.setModel(this.model);
	}

	@Override
	public DefaultTreeModel getModel() {
		return model;
	}

	/**
	 * Sets the given nodes to be the top-level children (ie, direct children of
	 * the root node), and then updates the tree view accordingly.
	 * 
	 * @param rootChildren
	 *            The nodes to use as top-level root children.
	 */
	public void setRootChildren(List<? extends MutableTreeNode> rootChildren) {
		root.removeAllChildren();

		for (MutableTreeNode node : rootChildren)
			root.add(node);

		model.nodeStructureChanged(root);

		for (int i = 0; i < getRowCount(); i++)
			expandRow(i);
	}

	/**
	 * Returns a set of every object in this tree whose node is currently
	 * checked of the given class.
	 * 
	 * @param <E>
	 *            The type of object to be finding
	 * @param clazz
	 *            The class of the objects to be finding
	 * @return The set of checked objects
	 */
	public <E> Set<E> getChecked(Class<E> clazz) {
		Set<E> objs = new HashSet<E>();

		TreePath[] paths = this.getCheckingModel().getSelectionPaths();

		if (paths != null)
			for (TreePath path : paths) {
				TreeNode node = (TreeNode) path.getLastPathComponent();

				objs.addAll(getDescendants(node, clazz));
			}

		return objs;
	}

	/**
	 * Returns a list of all of the objects of the given class which are
	 * descendants of the given node, including the one defined by the node
	 * itself if it fits that description.
	 * 
	 * @param <E>
	 *            The type of the objects to filter for
	 * @param node
	 *            The node to start the search from
	 * @param clazz
	 *            The class of the objects to filter for
	 * @return A list of all found objects of the given class
	 */
	@SuppressWarnings("unchecked")
	public <E> List<E> getDescendants(TreeNode node, Class<E> clazz) {
		List<E> ents = new ArrayList<E>();

		if (node instanceof DefaultMutableTreeNode) {
			Object ent = ((DefaultMutableTreeNode) node).getUserObject();
			if (clazz.isInstance(ent))
				ents.add((E) ent);
		}

		for (Object child : Collections.list(node.children()))
			if (child instanceof TreeNode)
				ents.addAll(getDescendants((TreeNode) child, clazz));

		return ents;
	}

	public void checkPathsTo(List<?> axisItems) {

		if (axisItems.isEmpty())
			return;

		Enumeration<?> allNodes = root.preorderEnumeration();

		while (allNodes.hasMoreElements()) {
			DefaultMutableTreeNode node = (DefaultMutableTreeNode) allNodes
					.nextElement();

			if (axisItems.contains(node.getUserObject()))
				getCheckingModel().addSelectionPath(
						new TreePath(node.getPath()));
		}

		model.nodeStructureChanged(root);
	}
}