/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components.editor;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.text.JTextComponent;

import net.java.ao.Entity;

import com.toedter.calendar.JDateChooser;

import edu.wpi.wpisuite.components.EntityComboBox;
import edu.wpi.wpisuite.components.MappedComboBox;
import edu.wpi.wpisuite.components.editor.Editor.Mode;

/**
 * Defines almost all of the detailed functionality required to make a panel
 * view interface with its editor. Primarily, sets up the standard action,
 * delete, and cancel buttons to work appropriately and defines a heavily
 * overloaded <code> registerComponent </code> function which takes care of
 * associating listeners for the editor on property changes and setting up this
 * view so that the editor can set properties, as well as enable and disable
 * components. Also provides convenience methods to easily set a panel as read
 * only.
 * 
 * Most EditorPanelViews extending this class should have to do no more than
 * initializing the layout of their components and registering the ones that
 * deal with properties
 */
@SuppressWarnings("serial")
public abstract class DefaultEntityEditorPanelView extends EditorPanelView {
	/** A map from property names to their associated labels */
	Map<String, JLabel> propertyLabels;

	/** A map from property names to their associated editable fields */
	Map<String, JComponent> propertyFields;

	/** A map from property names to an associated Mutator function wrapper */
	Map<String, Mutator> propertyMutators;

	/**
	 * A map from property names to an associated Mutator function wrapper for
	 * the options allowed over that property
	 */
	Map<String, Mutator> propertyOptionMutators;

	/** A map from property names to an associated Accessor function wrapper */
	Map<String, Accessor> propertyAccessors;

	/**
	 * Constructs a new panel view and registers all the defined components to
	 * interface with the controller
	 * 
	 * @param editor
	 *            The Editor associated with and controlling this view
	 */
	public DefaultEntityEditorPanelView(Editor editor) {
		super(editor);
		propertyLabels = new HashMap<String, JLabel>();
		propertyFields = new HashMap<String, JComponent>();
		propertyMutators = new HashMap<String, Mutator>();
		propertyOptionMutators = new HashMap<String, Mutator>();
		propertyAccessors = new HashMap<String, Accessor>();

		this.initComponents();
		this.registerComponents();
		this.registerDefaultComponents();
	}

	/**
	 * Sets the panel to be read only by disabling and disappearing all of the
	 * buttons and disabling all of the fields.
	 */
	@Override
	public void setReadOnly() {

		actionButton.setEnabled(false);
		actionButton.setVisible(false);
		actionButton.setSize(1, 1);

		deleteButton.setEnabled(false);
		deleteButton.setVisible(false);
		cancelButton.setSize(1, 1);

		cancelButton.setEnabled(false);
		cancelButton.setVisible(false);
		cancelButton.setSize(1, 1);

		setAllPropertiesEnabled(false);
	}

	/**
	 * Override this with the layout initialization for a particular panel.
	 * Usually netbeans will generate this method.
	 */
	protected abstract void initComponents();

	/**
	 * Override this with the component registration for a particular panel.
	 * Will usually just consist of a series of lines of the form:
	 * registerComponent(propertyName, propertyLabel, propertyField);
	 */
	protected abstract void registerComponents();

	/**
	 * Registers the typical buttons all default panels share
	 */
	protected void registerDefaultComponents() {
		this.registerComponent("cancel", this.cancelButton);
		this.registerComponent("delete", this.deleteButton);
		this.registerComponent("action", this.actionButton);
	}

	/**
	 * Registers a text field of some sort to be gettable, settable, and
	 * validatable as a string
	 * 
	 * @param propertyName
	 *            The name of the property this component defines
	 * @param label
	 *            The label for this property - used in validation
	 * @param field
	 *            The field for this property to be edited in
	 */
	protected void registerComponent(String propertyName, JLabel label,
			final JTextComponent field) {
		this.propertyLabels.put(propertyName, label);
		this.propertyFields.put(propertyName, field);
		this.addTextValidator(propertyName, field);
		this.propertyAccessors.put(propertyName, new Accessor() {
			@Override
			public Object get() {
				return field.getText();
			}
		});
		this.propertyMutators.put(propertyName, new Mutator() {
			public void set(Object obj) {
				field.setText(obj == null ? null : obj.toString());
			}
		});
	}

	/**
	 * Registers a text non-mutable label field to be gettable, settable, and
	 * validatable as a string
	 * 
	 * @param propertyName
	 *            The name of the property this component defines
	 * @param label
	 *            The label for this property - used in validation
	 * @param field
	 *            The field for this property to be edited in
	 */
	protected void registerComponent(String propertyName, final JLabel label,
			final JLabel field) {
		this.propertyLabels.put(propertyName, label);
		this.propertyFields.put(propertyName, field);

		this.propertyAccessors.put(propertyName, new Accessor() {
			public Object get() {
				return field.getText();
			}
		});
		this.propertyMutators.put(propertyName, new Mutator() {
			public void set(Object obj) {
				field.setText(obj == null ? null : obj.toString());
			}
		});
	}

	/**
	 * Registers a text label, for example: the Header text on the editor panel
	 * 
	 * @param label
	 *            The JLabel component
	 * @author Lucas Scotta
	 */
	protected void registerComponent(String propertyName, final JLabel label) {
		this.propertyLabels.put(propertyName, label);
		this.propertyFields.put(propertyName, label);

		this.propertyAccessors.put(propertyName, new Accessor() {
			public Object get() {
				return label.getText();
			}
		});
		this.propertyMutators.put(propertyName, new Mutator() {
			public void set(Object obj) {
				label.setText(obj == null ? null : obj.toString());
			}
		});
	}

	/**
	 * Registers a check box field of some sort to be gettable, settable, and
	 * validatable as a boolean
	 * 
	 * @param propertyName
	 *            The name of the property this component defines
	 * @param label
	 *            The label for this property - used in validation
	 * @param field
	 *            The field for this property to be edited in
	 */
	protected void registerComponent(String propertyName, final JLabel label,
			final JCheckBox field) {
		this.propertyLabels.put(propertyName, label);
		this.propertyFields.put(propertyName, field);
		this.addCheckBoxValidator(propertyName, field);
		this.propertyAccessors.put(propertyName, new Accessor() {
			public Object get() {
				return field.getModel().isSelected();
			}
		});
		this.propertyMutators.put(propertyName, new Mutator() {
			public void set(Object obj) {
				field.setSelected((Boolean) obj);
			}
		});
	}

	/**
	 * Registers an EntityComboBox of some sort to be gettable, settable, and
	 * validatable as an Entity of the associated type.
	 * 
	 * Also sets up a property option mutator to allow the setting of the
	 * options this combo box is allowed to be set to.
	 * 
	 * @param propertyName
	 *            The name of the property this component defines
	 * @param label
	 *            The label for this property - used in validation
	 * @param field
	 *            The field for this property to be edited in
	 */
	protected <E extends Entity> void registerComponent(String propertyName,
			JLabel label, final EntityComboBox<E> field) {
		this.propertyLabels.put(propertyName, label);
		this.propertyFields.put(propertyName, field);
		this.addComboBoxValidator(propertyName, field);
		this.propertyAccessors.put(propertyName, new Accessor() {
			@Override
			public Object get() {
				return field.getSelectedEntity();
			}
		});
		this.propertyMutators.put(propertyName, new Mutator() {
			@SuppressWarnings("unchecked")
			@Override
			public void set(Object obj) {
				field.setSelectedEntity((E) obj);
			}
		});
		this.propertyOptionMutators.put(propertyName, new Mutator() {
			@SuppressWarnings("unchecked")
			@Override
			public void set(Object obj) {
				field.getModel().reset((E[]) obj);
			}
		});
	}

	/**
	 * Registers a MappedComboBox of some sort to be gettable, settable, and
	 * validatable as the type of the combo box
	 * 
	 * @param propertyName
	 *            The name of the property this component defines
	 * @param label
	 *            The label for this property - used in validation
	 * @param field
	 *            The field for this property to be edited in
	 */
	protected <T> void registerComponent(String propertyName, JLabel label,
			final MappedComboBox<T> field) {
		this.propertyLabels.put(propertyName, label);
		this.propertyFields.put(propertyName, field);
		this.addComboBoxValidator(propertyName, field);
		this.propertyAccessors.put(propertyName, new Accessor() {
			@Override
			public Object get() {
				return field.getSelected();
			}
		});
		this.propertyMutators.put(propertyName, new Mutator() {
			@SuppressWarnings("unchecked")
			@Override
			public void set(Object obj) {
				field.setSelected((T) obj);
			}
		});
	}

	/**
	 * Registers a date field to be gettable, settable, and validatable
	 * 
	 * @param propertyName
	 *            The name of the property this component defines
	 * @param label
	 *            The label for this property - used in validation
	 * @param field
	 *            The field for this property to be edited in
	 */
	protected void registerComponent(String propertyName, JLabel label,
			final JDateChooser field) {
		this.propertyLabels.put(propertyName, label);
		this.propertyFields.put(propertyName, field);
		addDateValidator(propertyName, field);
		this.propertyAccessors.put(propertyName, new Accessor() {
			@Override
			public Object get() {
				return field.getDate();
			}
		});
		this.propertyMutators.put(propertyName, new Mutator() {
			@Override
			public void set(Object obj) {
				field.setDate((Date) obj);
			}
		});
	}

	/**
	 * Registers a button to throw commands to the editor on press
	 * 
	 * @param commandType
	 *            The name of the command this component defines
	 * @param button
	 *            The button being associated
	 */
	protected void registerComponent(String commandType, JButton button) {
		this.propertyFields.put(commandType, button);
		this.addButtonListener(commandType, button);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * components.editor.EditorPanelView#setPropertyInvalid(java.lang.String,
	 * components.editor.InvalidInputException)
	 */
	@Override
	public void setPropertyInvalid(String propertyName, InvalidInputException e) {
		this.setFieldInvalid(this.propertyLabels.get(propertyName),
				this.propertyFields.get(propertyName), e);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see components.editor.EditorPanelView#setPropertyValid(java.lang.String)
	 */
	@Override
	public void setPropertyValid(String propertyName) {
		this.setFieldValid(this.propertyLabels.get(propertyName),
				this.propertyFields.get(propertyName));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see components.editor.EditorPanelView#setProperty(java.lang.String,
	 * java.lang.Object)
	 */
	@Override
	public void setProperty(String propertyName, Object property) {
		propertyMutators.get(propertyName).set(property);
		this.fireAttributeChange(propertyName, propertyAccessors.get(
				propertyName).get());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * components.editor.EditorPanelView#setPropertyEnabled(java.lang.String,
	 * boolean)
	 */
	@Override
	public void setPropertyEnabled(String propertyName, boolean enabled) {
		propertyFields.get(propertyName).setEnabled(enabled);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see components.editor.EditorPanelView#setPropertyIcon(java.lang.String,
	 * javax.swing.Icon)
	 */
	@Override
	public void setPropertyIcon(String propertyName, Icon icon) {
		propertyLabels.get(propertyName).setIcon(icon);
	}

	/**
	 * Sets every registered property component field to be disabled (or, in the
	 * case of text components, merely non-editable)
	 * 
	 * @param enabled
	 *            Whether the properties should be enabled or disabled
	 */
	public void setAllPropertiesEnabled(boolean enabled) {
		for (JComponent field : propertyFields.values())
			if (field instanceof JTextComponent)
				((JTextComponent) field).setEditable(false);
			else if (field != null)
				field.setEnabled(enabled);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * components.editor.EditorPanelView#setPropertyOptions(java.lang.String,
	 * java.lang.Object[])
	 */
	@Override
	public void setPropertyOptions(String propertyName, Object[] options) {
		propertyOptionMutators.get(propertyName).set(options);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see components.editor.EditorPanelView#getProperty(java.lang.String)
	 */
	@Override
	public Object getProperty(String propertyName) {
		return propertyAccessors.get(propertyName).get();
	}

	/**
	 * Sets the buttons' statuses and texts according to the given mode the
	 * editor is in
	 */
	@Override
	public void changeMode(Mode mode) {
		switch (mode) {
		case CREATE:
			this.actionButton.setText("Create");
			this.deleteButton.setEnabled(false);
			this.cancelButton.setEnabled(true);
			break;
		case EDIT:
			this.actionButton.setText("Save");
			this.deleteButton.setEnabled(true);
			this.cancelButton.setEnabled(true);
			break;
		case SHOW:
			this.actionButton.setText("Save");
			this.actionButton.setEnabled(false);
			this.deleteButton.setEnabled(true);
			this.cancelButton.setEnabled(false);
		}
	}

	/**
	 * Sets the buttons' enabled statuses according to whether the currently
	 * entered information is valid for saving
	 */
	@Override
	public void changeValid(boolean isValid) {
		this.actionButton.setEnabled(isValid);
	}

	// common components
	protected JButton actionButton;
	protected JButton deleteButton;
	protected JButton cancelButton;
}

/**
 * A simple interface defining a standard setter method for any object
 */
interface Mutator {
	public void set(Object obj);
}

/**
 * A simple interface defining a standard getter method for any object
 */
interface Accessor {
	public Object get();
}
