/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components.editor;

import java.util.HashSet;
import java.util.Set;


import edu.wpi.wpisuite.components.EntityIconSet;
import edu.wpi.wpisuite.module.ActiveStateEvent.ActiveStateEventType;

/**
 * Represents the controller for an editor of some sort. Will typically provide
 * means to perform CRUD for a specific type of object. Encapsulates an
 * appropriate view.
 */
public abstract class Editor implements CommandListener, PropertyValidator {
	/** Defines what an Editor is currently doing to its object */
	public static enum Mode {
		CREATE, EDIT, SHOW
	};

	/** Defines how a user could answer a confirmation request */
	public static enum ConfirmResult {
		YES, NO, CANCEL
	};

	/** The containing controller */
	protected EditorContainer container;

	/** The view for this editor controller */
	protected EditorPanelView view;

	/**
	 * The current mode - what this editor is currently doing to its contained
	 * object
	 */
	private Mode mode;

	/**
	 * The set of all properties this editor might be editing which are
	 * currently invalid. Some things should not work unless this set is empty.
	 */
	private Set<String> invalidProperties;

	/**
	 * Set of entity icons to be used in the editor panel
	 */
	protected EntityIconSet entityIcons;

	/**
	 * Changes the mode of this editor and notifies the view that the mode has
	 * changed.
	 * 
	 * @param mode
	 *            The new mode to be set.
	 */
	protected void setMode(Mode mode) {
		this.mode = mode;
		this.view.changeMode(mode);
	}

	/**
	 * @return The current editing mode for this editor.
	 */
	protected Mode getMode() {
		return this.mode;
	}

	/**
	 * Sets a property as containing valid data and notifies the view of the
	 * change if it was previously invalid.
	 * 
	 * @param property
	 *            The name of the property to set as valid.
	 */
	protected void setValid(String property) {
		if (this.invalidProperties.remove(property))
			// This only changed if we actually just removed something
			this.view.changeValid(this.isValid());
	}

	/**
	 * Sets a property as containing invalid data and notifies the view of the
	 * change if it was previously valid.
	 * 
	 * @param property
	 *            The name of the property to set as invalid.
	 */
	protected void setInvalid(String property) {
		if (this.invalidProperties.add(property))
			// This only changed if we actually just added something
			this.view.changeValid(this.isValid());
	}

	/**
	 * @return Whether any of the properties being edited are currently invalid
	 */
	protected boolean isValid() {
		return this.invalidProperties.isEmpty();
	}

	/**
	 * Informs the containing controller and the application as a whole that the
	 * state of this editor has changed.
	 */
	@SuppressWarnings("deprecation")
	public void fireStateChange() {
		this.container.getState().fireStateChange(ActiveStateEventType.PROJECT);
		this.container.onEditorSave();
	}

	/**
	 * Constructs a new Editor for the given supercontroller and the given class
	 * of view, including initializing both this controller and a view of the
	 * given class.
	 * 
	 * @param container
	 *            The containing controller
	 * @param viewClass
	 *            The class of view to initialize. Uses reflection to construct
	 *            a new view based on this editor instance. This view is
	 *            required to have a constructor which can take the editor being
	 *            constructed as its only parameter.
	 */
	public Editor(EditorContainer container,
			Class<? extends EditorPanelView> viewClass) {
		this.container = container;
		try {
			this.view = viewClass.getConstructor(this.getClass()).newInstance(
					this);
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException("Instantiation of "
					+ this.getClass().getCanonicalName() + " failed.");
		}
		this.setMode(Mode.CREATE);
		this.invalidProperties = new HashSet<String>();
		this.entityIcons = new EntityIconSet();
	}

	/**
	 * @return The view for this controller
	 */
	public EditorPanelView getView() {
		return this.view;
	}

	/**
	 * Informs the supercontroller that this editor is no longer applicable to
	 * anything and should be closed down.
	 */
	public void close() {
		this.container.closeEditor();
	}

	/**
	 * Sets this editor to not allow the editing of its entity
	 */
	public void setReadOnly() {
		this.getView().setReadOnly();
	}

}
