/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components.editor;

import net.java.ao.Entity;


import edu.wpi.wpisuite.components.editor.entity.FeatureEditor;
import edu.wpi.wpisuite.components.editor.entity.IterationEditor;
import edu.wpi.wpisuite.components.editor.entity.ReleaseEditor;
import edu.wpi.wpisuite.components.editor.entity.UserEditor;
import edu.wpi.wpisuite.components.editor.entity.UserStoryEditor;
import edu.wpi.wpisuite.data.Feature;
import edu.wpi.wpisuite.data.Iteration;
import edu.wpi.wpisuite.data.Release;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.data.UserStory;

/**
 * This class serves as a way to generate a specific Editor instance based on
 * the type of Entity you wish to edit. It can generate Editor instances based
 * on either the class you wish to edit or a specific object you wish to edit.
 */
public class EditorFactory {
	EditorContainer rm;

	/**
	 * Initializes this editor factory to the given Requirements Manager so that
	 * any Editors it creates can similarly be initialized (notably, so that
	 * they can draw from the application State)
	 * 
	 * @param rm
	 */
	public EditorFactory(EditorContainer rm) {
		this.rm = rm;
	}

	/**
	 * Initializes a new, blank Editor based on the given class.
	 * 
	 * @param editorType
	 *            The type of the Entity you wish to create/edit
	 * @return A new Editor for creating an object of type editorType
	 * @throws IllegalArgumentException
	 *             If the given class does not have an associated Editor type.
	 */
	public Editor getEditor(Class<?> editorType)
			throws IllegalArgumentException {
		if (editorType == null)
			return null;

		if (editorType.equals(Feature.class))
			return new FeatureEditor(this.rm);
		else if (editorType.equals(UserStory.class))
			return new UserStoryEditor(this.rm);
		else if (editorType.equals(Iteration.class))
			return new IterationEditor(this.rm);
		else if (editorType.equals(Release.class))
			return new ReleaseEditor(this.rm);
		else if (editorType.equals(User.class))
			return new UserEditor(this.rm);
		throw new IllegalArgumentException(
				"No known editor applicable to type "
						+ editorType.getCanonicalName());
	}

	/**
	 * Initializes a new Editor based on the type of the given Object, then
	 * initializes the Editor to the attributes of the Object.
	 * 
	 * @param editorObject
	 *            The object to initialize the Editor with.
	 * @return A new Editor instance for editing the given object
	 * @throws IllegalArgumentException
	 *             If the given object's class does not have an associated
	 *             Editor type
	 */
	public Editor getEditor(Entity editorObject)
			throws IllegalArgumentException {
		if (editorObject == null)
			return null;

		if (editorObject instanceof Feature)
			return new FeatureEditor(this.rm, (Feature) editorObject);
		else if (editorObject instanceof UserStory)
			return new UserStoryEditor(this.rm, (UserStory) editorObject);
		else if (editorObject instanceof Iteration)
			return new IterationEditor(this.rm, (Iteration) editorObject);
		else if (editorObject instanceof Release)
			return new ReleaseEditor(this.rm, (Release) editorObject);
		else if (editorObject instanceof User)
			return new UserEditor(this.rm, (User) editorObject);
		throw new IllegalArgumentException(
				"No known editor applicable to object " + editorObject);
	}
}
