/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components.editor;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.JTextComponent;

import org.apache.log4j.Logger;

import com.toedter.calendar.JDateChooser;

import edu.wpi.wpisuite.components.EntityComboBox;
import edu.wpi.wpisuite.components.MappedComboBox;
import edu.wpi.wpisuite.components.editor.Editor.ConfirmResult;

/**
 * Represents a JPanel which acts as a view for an Editor controller. Contains
 * all functionality that involves reading and writing the UI fields to be
 * synchronized with the modeled editor object, but abstracts anything that
 * needs to be done to that object to an Editor
 */
public abstract class EditorPanelView extends JPanel implements ModeListener,
		ValidListener {

	private Logger logger = Logger.getLogger(this.getClass());

	/** The controller for this view */
	Editor editor;

	/**
	 * Initializes a new view according to the given editor
	 * 
	 * @param editor
	 *            The editor to control this view
	 */
	public EditorPanelView(Editor editor) {
		this.editor = editor;
	}

	/**
	 * Asks a user to confirm some action by popping a confirmation dialog box.
	 * 
	 * @param confirmMessage
	 *            The message explaining what is being confirmed
	 * @return Whether or not the user confirmed the action
	 */
	public boolean confirmAction(String confirmMessage) {
		int confirmResult = JOptionPane.showConfirmDialog(null, confirmMessage,
				"Confirm", JOptionPane.OK_CANCEL_OPTION);
		return confirmResult == JOptionPane.YES_OPTION;
	}

	/**
	 * Asks a user to confirm some action by popping a confirmation dialog box.
	 * Allows options for Yes, No, and Cancel.
	 * 
	 * @param message
	 *            The message explaining what is being confirmed. It should be
	 *            clear about what each of Yes, No, and Cancel would mean and
	 *            do.
	 * @return The user's response to the confirmation request
	 */
	public ConfirmResult askYesNoCancelQuestion(String title, String message) {
		int result = JOptionPane.showConfirmDialog(null, message, title,
				JOptionPane.YES_NO_CANCEL_OPTION);
		switch (result) {
		case JOptionPane.NO_OPTION:
			return ConfirmResult.NO;
		case JOptionPane.YES_OPTION:
			return ConfirmResult.YES;
		default:
			return ConfirmResult.CANCEL;
		}
	}

	/**
	 * Reports that some property in this view has changed, and reports such to
	 * the editor for validation. Then reports to the user whether or not the
	 * new value is valid for this property.
	 * 
	 * @param propertyName
	 *            The name of the property to report a change in
	 * @param newProperty
	 *            The new value for the changed property
	 */
	public void fireAttributeChange(String propertyName, Object newProperty) {
		try {
			this.editor.validateProperty(propertyName, newProperty);
			this.setPropertyValid(propertyName);
		} catch (InvalidInputException e) {
			this.setPropertyInvalid(propertyName, e);
		}
	}

	/**
	 * Informs the controlling editor that the user has requested that some
	 * action be performed. This might take the form of a user hitting a button
	 * for example.
	 * 
	 * @param commandType
	 *            A string describing the action the user would like to take
	 */
	public void fireCommand(String commandType) {
		ActionEvent evt = new ActionEvent(this, 0, commandType);
		try {
			this.editor.commandPerformed(evt);
		} catch (IOException e) {
			this.failCommand(commandType, e);
		}
	}

	/**
	 * Adds a listener on a JCheckBox that fires an appropriate property change
	 * to the controlling editor whenever the checkbox selection status changes.
	 * 
	 * @param propertyName
	 *            The name of the property to reference this property as.
	 * @param component
	 *            The JCheckBox which acts as a view on this property.
	 */
	protected void addCheckBoxValidator(final String propertyName,
			final JCheckBox component) {
		// Tells the component to fire an appropriate attribute change whenever
		// the contained document model is updated in any way
		component.getModel().addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent arg0) {
				EditorPanelView.this.fireAttributeChange(propertyName,
						component.getModel().isSelected());
			}
		});
	}

	/**
	 * Adds a listener on a JTextComponent that fires an appropriate property
	 * change to the controlling editor whenever the JTextComponent reports any
	 * changes.
	 * 
	 * @param propertyName
	 *            The name of the property to reference this property as.
	 * @param component
	 *            The JTextComponent which acts as a view on this property.
	 */
	protected void addTextValidator(final String propertyName,
			final JTextComponent component) {
		// Tells the component to fire an appropriate attribute change whenever
		// the contained document model is updated in any way
		component.getDocument().addDocumentListener(new DocumentListener() {
			@Override
			public void changedUpdate(DocumentEvent arg0) {
				EditorPanelView.this.fireAttributeChange(propertyName,
						component.getText());
			}

			@Override
			public void insertUpdate(DocumentEvent arg0) {
				EditorPanelView.this.fireAttributeChange(propertyName,
						component.getText());
			}

			@Override
			public void removeUpdate(DocumentEvent arg0) {
				EditorPanelView.this.fireAttributeChange(propertyName,
						component.getText());
			}

		});
	}

	/**
	 * Adds a listener on an EntityComboBox component that fires an appropriate
	 * property change to the controlling editor whenever the combobox selection
	 * changes.
	 * 
	 * @param propertyName
	 *            The name of the property to reference this object as
	 * @param combobox
	 *            The component acting as a view on this property.
	 */
	protected void addComboBoxValidator(final String propertyName,
			final EntityComboBox<?> combobox) {
		combobox.addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent arg0) {
				EditorPanelView.this.fireAttributeChange(propertyName, combobox
						.getSelectedEntity());
			}
		});
	}

	/**
	 * Adds a listener on an EntityComboBox component that fires an appropriate
	 * property change to the controlling editor whenever the combobox selection
	 * changes.
	 * 
	 * @param propertyName
	 *            The name of the property to reference this object as
	 * @param combobox
	 *            The component acting as a view on this property.
	 */
	protected void addComboBoxValidator(final String propertyName,
			final MappedComboBox<?> combobox) {
		combobox.addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent arg0) {
				EditorPanelView.this.fireAttributeChange(propertyName, combobox
						.getSelected());
			}
		});
	}

	/**
	 * Adds a listener on a JDateChooser component that fires an appropriate
	 * property change to the controlling editor whenever the date selection
	 * changes.
	 * 
	 * @param propertyName
	 *            The name of the property to reference this object as
	 * @param combobox
	 *            The component acting as a view on this property.
	 */
	protected void addDateValidator(final String propertyName,
			final JDateChooser dateChooser) {
		dateChooser.getDateEditor().addPropertyChangeListener("date",
				new PropertyChangeListener() {
					@Override
					public void propertyChange(PropertyChangeEvent arg0) {

						EditorPanelView.this.fireAttributeChange(propertyName,
								dateChooser.getDate());
						/*
						 * try {
						 * EditorPanelView.this.editor.validateProperty(propertyName
						 * , dateChooser.getDate()); } catch
						 * (InvalidInputException e) {
						 * JOptionPane.showMessageDialog(null, e.getMessage());
						 * 
						 * }
						 */
					}
				});
	}

	/**
	 * Adds a listener on a JButton that fires an appropriate command to the
	 * editor controller when the button is pressed.
	 * 
	 * @param commandType
	 *            The name of the command to pass
	 * @param component
	 *            The button to register a listener on
	 */
	protected void addButtonListener(final String commandType,
			final JButton component) {

		if (component == null) {
			logger.error("component is null");
			return;
		}

		component.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				EditorPanelView.this.fireCommand(commandType);
			}
		});
	}

	/**
	 * Shows an error message when the editor informs this view that a command
	 * has somehow failed
	 * 
	 * @param commandType
	 *            The type of ocmmand which has failed
	 * @param e
	 *            An exception containing more details on how the command
	 *            failed.
	 */
	protected void failCommand(String commandType, IOException e) {
		JOptionPane.showMessageDialog(null, e.getMessage());
	}

	// Defines some error validation colors
	protected final Color errorLabelColor = new Color(255, 0, 0);
	protected final Color errorBackgroundColor = new Color(255, 122, 122);
	protected final Color normalLabelColor = new Color(0, 0, 0);
	protected final Color normalBackgroundColor = new Color(255, 255, 255);

	/**
	 * Sets a property viewed by the given label and field component as invalid,
	 * showing up as red to the user and throwing tooltips explaining what is
	 * wrong.
	 * 
	 * @param label
	 *            The label component for this property
	 * @param field
	 *            The field component for this property
	 * @param e
	 *            The exception describing what specifically is invalid about
	 *            the current setting
	 */
	protected void setFieldInvalid(JComponent label, JComponent field,
			InvalidInputException e) {
		label.setToolTipText(e.getMessage());
		label.setForeground(this.errorLabelColor);
		field.setToolTipText(e.getMessage());
		// field.setBackground(errorBackgroundColor);
	}

	/**
	 * Sets a property viewed by the given label and field component as valid,
	 * restoring normal coloration and tooltips
	 * 
	 * @param label
	 *            The label component for this property
	 * @param field
	 *            The field component for this property
	 */
	protected void setFieldValid(JComponent label, JComponent field) {
		label.setToolTipText(null);
		label.setForeground(this.normalLabelColor);
		field.setToolTipText(null);
		// field.setBackground(normalBackgroundColor);
	}

	/**
	 * Sets a particular property as invalid. Intended to be overriden to apply
	 * to the specific labels and fields referencing this property
	 * 
	 * @param propertyName
	 *            The name of the property to mark as invalid
	 * @param e
	 *            The exception explaining the reason for invalidity
	 */
	public abstract void setPropertyInvalid(String propertyName,
			InvalidInputException e);

	/**
	 * Sets a particular property as valid. Intended to be overriden to apply to
	 * the specific labels and fields referencing this property
	 * 
	 * @param propertyName
	 *            The name of the property to mark as valid
	 */
	public abstract void setPropertyValid(String propertyName);

	/**
	 * Gets the value referenced by a specific property name, as currently
	 * represented in the view
	 * 
	 * @param propertyName
	 *            The name of the property to get the value of
	 */
	public abstract Object getProperty(String propertyName);

	/**
	 * Sets the property in the view to the given value. Intended to be
	 * overriden and casted appropriately for all necessary properties.
	 * 
	 * @param propertyName
	 *            The name of the property to set the value of
	 * @param property
	 *            The new value to set the property to
	 */
	public abstract void setProperty(String propertyName, Object property);

	/**
	 * Sets whether or not the property in the view is allowed to be editted.
	 * Intended to be overriden to set the appropriate property component
	 * according to the passed name.
	 * 
	 * @param propertyName
	 *            The name of the property to (en/dis)able
	 * @param enabled
	 *            Whether to enable or disable the property
	 */
	public abstract void setPropertyEnabled(String propertyName, boolean enabled);

	/**
	 * Sets the available choices in a property where the user can select only
	 * from some discrete choices, like in a list, combobox, or option group.
	 * 
	 * @param propertyName
	 *            The name of the property to set options for
	 * @param options
	 *            The options to set available.
	 */
	public abstract void setPropertyOptions(String propertyName,
			Object[] options);

	/**
	 * Sets the panel view to not display editting buttons
	 */
	public abstract void setReadOnly();

	/**
	 * Sets the icon for the label of a specific property.
	 * 
	 * @param propertyName
	 *            The name of the property to set the icon for
	 * @param icon
	 *            The icon to set
	 */
	public abstract void setPropertyIcon(String propertyName, Icon icon);

	public void error(InvalidInputException e) {
		JOptionPane.showMessageDialog(null, e.getMessage(), "Error",
				JOptionPane.ERROR_MESSAGE);
	}

}
