/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components.editor.entity;

import java.awt.event.ActionEvent;
import java.io.IOException;
import java.sql.SQLException;

import net.java.ao.DBParam;


import edu.wpi.wpisuite.components.EntityIconSet.IconType;
import edu.wpi.wpisuite.components.editor.Editor;
import edu.wpi.wpisuite.components.editor.EditorContainer;
import edu.wpi.wpisuite.components.editor.InvalidInputException;
import edu.wpi.wpisuite.data.Feature;
import edu.wpi.wpisuite.data.Release;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.data.funcs.Artifact;
import edu.wpi.wpisuite.database.Database;

/**
 * Defines the idea of a Feature editing interface without actually specifying
 * an implementation thereof. Interfaces between a View which provides that
 * implementation and the Requirements Manager itself.
 */
public class FeatureEditor extends Editor {
	private Feature feature;

	public FeatureEditor(EditorContainer rm) {
		super(rm, FeatureEditorPanelView.class);
		this.setFeature(null);
	}

	public FeatureEditor(EditorContainer rm, Feature feature) {
		super(rm, FeatureEditorPanelView.class);
		this.setFeature(feature);
	}

	public void setFeature(Feature f) {
		this.feature = f;

		String header = "";
		if (f != null) {
			header = "Feature - " + new Artifact(f).toString();
			this.view.setPropertyIcon("header", entityIcons
					.getFeatureIcon(IconType.DEFAULT));
		} else {
			header = "New Feature";
			this.view.setPropertyIcon("header", entityIcons
					.getFeatureIcon(IconType.CREATE));
		}

		this.view.setProperty("header", header);

		String title = "";
		if (f != null)
			title = f.getTitle();

		this.view.setProperty("title", title);

		String desc = "";
		if (f != null)
			desc = f.getDescription();
		this.view.setProperty("description", desc);

		if (f == null)
			this.setOwnerProperties(null);
		else
			this.setOwnerProperties(f.getOwner());

		setReleaseOptions();

		Release rel = null;
		if (f != null)
			rel = f.getRelease();
		view.setProperty("release", rel);

		if (f == null)
			this.setMode(Mode.CREATE);
		else
			this.setMode(Mode.SHOW);
	}

	public void setReleaseOptions() {
		Release[] rels = this.container.getState().getProject().getReleases();
		Release[] relsWithNull = new Release[1 + rels.length];
		relsWithNull[0] = null;
		for (int i = 0; i < rels.length; i++)
			relsWithNull[i + 1] = rels[i];

		this.view.setPropertyOptions("release", relsWithNull);
	}

	public void setOwnerProperties(User newOwner) {
		User[] members = this.container.getState().getProject().getMembers();
		User[] membersWithNull = new User[1 + members.length];
		membersWithNull[0] = null;
		for (int i = 0; i < members.length; i++)
			membersWithNull[i + 1] = members[i];

		this.view.setPropertyOptions("owner", membersWithNull);
		this.view.setProperty("owner", newOwner);

		this.validateClaimable(newOwner);
	}

	public void validateClaimable(User newOwner) {
		this.view.setPropertyEnabled("claimOwnership", true);
		if (newOwner != null)
			// Can't claim what you already own
			if (newOwner.getID() == this.container.getState().getUser().getID())
				this.view.setPropertyEnabled("claimOwnership", false);
	}

	public void setNewFeature() {
		this.setFeature(null);
	}

	@Override
	public void validateProperty(String propertyName, Object newValue)
			throws InvalidInputException {
		if (this.getMode() == Mode.SHOW)
			this.setMode(Mode.EDIT);

		this.setInvalid(propertyName);

		if (propertyName.equals("title"))
			this.validateTitle((String) newValue);
		else if (propertyName.equals("description"))
			this.validateDescription((String) newValue);
		else if (propertyName.equals("owner"))
			this.validateOwner((User) newValue);
		else if (propertyName.equals("release"))
			this.validateRelease((Release) newValue);

		// We only get here if validator didn't throw an exception
		this.setValid(propertyName);
	}

	@Override
	public void commandPerformed(ActionEvent evt) throws IOException {
		String commandType = evt.getActionCommand();
		if (commandType.equals("cancel"))
			this.cancelChanges();
		else if (commandType.equals("action"))
			this.performAction();
		else if (commandType.equals("claimOwnership"))
			this.claimOwnership();
		else if (commandType.equals("delete"))
			this.delete();

	}

	public void validateOwner(User owner) throws InvalidInputException {
		this.validateClaimable(owner);
	}

	public void validateTitle(String title) throws InvalidInputException {
		if (title == null)
			throw new InvalidInputException("Cannot be empty");

		if (title.equals(""))
			throw new InvalidInputException("Cannot be empty");
	}

	public void validateDescription(String desc) throws InvalidInputException {
		if (desc == null)
			throw new InvalidInputException("Cannot be empty");

		if (desc.equals(""))
			throw new InvalidInputException("Cannot be empty");
	}

	public void validateRelease(Release release) throws InvalidInputException {
		// blank by design
	}

	public void cancelChanges() throws IOException {
		if (this.getMode() == Mode.CREATE)
			this.cancelCreate();
		else
			this.cancelEdit();
	}

	public void claimOwnership() throws IOException {
		if (this.feature != null && this.feature.getOwner() != null)
			if (!this.view
					.confirmAction("This requirement is already owned by user "
							+ this.feature.getOwner().getUsername() + ".\n"
							+ "Replace this user as this requirement's owner?"))
				return; // User did NOT confirm

		// Either no owner, or user confirmed
		this.setOwnerProperties(this.container.getState().getUser());
	}

	public void performAction() throws IOException {
		if (this.getMode() == Mode.CREATE)
			this.createFeature();
		else if (this.getMode() == Mode.EDIT)
			this.saveFeature();
	}

	public void delete() throws IOException {
		if (!this.view
				.confirmAction("Are you sure you want to delete this requirement?"))
			return; // User didn't confirm - don't delete

		try {
			Database.manager().delete(this.feature);
		} catch (SQLException e) {
			throw new IOException(
					"A database error occured while deleting this Requirement");
		}
		this.fireStateChange();
		this.close();
	}

	public void cancelCreate() throws IOException {
		this.close();
	}

	public void cancelEdit() throws IOException {
		this.setFeature(this.feature);
	}

	public void createFeature() throws IOException {
		try {
			this.feature = Database.manager().create(Feature.class,
					new DBParam("title", view.getProperty("title")));
		} catch (SQLException e) {
			throw new IOException(
					"A database error occured while creating this feature.");
		}

		this.feature.setAuthor(this.container.getState().getUser());
		this.feature.setProject(this.container.getState().getProject());

		this.saveFeature();
	}

	public void saveFeature() throws IOException {
		this.feature.setTitle((String) this.view.getProperty("title"));
		this.feature.setDescription((String) this.view
				.getProperty("description"));
		this.feature.setOwner((User) this.view.getProperty("owner"));
		this.feature.setRelease((Release) this.view.getProperty("release"));

		this.feature.save();

		this.setMode(Mode.SHOW);
		this.fireStateChange();
		this.setFeature(this.feature); // resets buttons
	}

}
