/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *    Ian Williams - some implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components.editor.entity;

import java.awt.event.ActionEvent;
import java.io.IOException;
import java.util.Date;

import org.jfree.util.Log;


import edu.wpi.wpisuite.components.EntityIconSet.IconType;
import edu.wpi.wpisuite.components.editor.Editor;
import edu.wpi.wpisuite.components.editor.EditorContainer;
import edu.wpi.wpisuite.components.editor.InvalidInputException;
import edu.wpi.wpisuite.data.Iteration;
import edu.wpi.wpisuite.data.Project;
import edu.wpi.wpisuite.data.Release;
import edu.wpi.wpisuite.data.UserStory;
import edu.wpi.wpisuite.data.errors.InvalidIterationInputException;
import edu.wpi.wpisuite.data.funcs.Artifact;
import edu.wpi.wpisuite.data.funcs.IterationManager;
import edu.wpi.wpisuite.data.util.IterationAccessor;

/**
 * Defines the idea of a Iteration editing interface without actually specifying
 * an implementation thereof. Interfaces between a View which provides that
 * implementation and the Requirements Manager itself.
 */
public class IterationEditor extends Editor implements IterationAccessor {
	private Iteration iteration;

	public IterationEditor(EditorContainer rm) {
		super(rm, IterationEditorPanelView.class);
		this.setNewIteration();
	}

	public IterationEditor(EditorContainer rm, Iteration iteration) {
		super(rm, IterationEditorPanelView.class);
		this.setIteration(iteration);
	}

	public void setIteration(Iteration it) {
		this.iteration = it;

		this.view.setProperty("header", "Iteration - "
				+ new Artifact(it).toString());
		this.view.setProperty("name", it.getName());
		this.setReleaseOptions();
		this.view.setProperty("release", it.getRelease());
		this.view.setProperty("startDate", it.getStartDate());
		this.view.setProperty("endDate", it.getEndDate());
		this.view.setProperty("totalUnits", this.getTotalIterationUnits());

		this.view.setPropertyIcon("header", entityIcons
				.getIterationIcon(IconType.DEFAULT));

		this.setMode(Mode.SHOW);
	}

	public int getTotalIterationUnits() {
		try {
			return IterationManager.countEstimateUnit(iteration);
		} catch (InvalidIterationInputException e) {
			return 0;
		}
	}

	public void setNewIteration() {
		this.iteration = null;

		this.view.setProperty("header", "New Iteration");
		this.view.setProperty("name", "");
		this.view.setProperty("startDate", null);
		this.view.setProperty("endDate", null);
		this.setReleaseOptions();
		this.view.setProperty("release", null);
		this.view.setProperty("totalUnits", 0);

		this.view.setPropertyIcon("header", entityIcons
				.getIterationIcon(IconType.CREATE));

		this.setMode(Mode.CREATE);
	}

	public void setReleaseOptions() {
		Release[] rels = this.container.getState().getProject().getReleases();
		Release[] relsWithNull = new Release[1 + rels.length];
		relsWithNull[0] = null;
		for (int i = 0; i < rels.length; i++)
			relsWithNull[i + 1] = rels[i];

		this.view.setPropertyOptions("release", relsWithNull);
	}

	@Override
	public void validateProperty(String propertyName, Object newValue)
			throws InvalidInputException {
		if (this.getMode() == Mode.SHOW)
			this.setMode(Mode.EDIT);
		this.setInvalid(propertyName);
		String otherDate = null;

		if (propertyName.equals("name"))
			this.validateName();
		else if (propertyName.equals("startDate")) {
			otherDate = "endDate";
			this.setInvalid(otherDate);
			this.validateDates();
		} else if (propertyName.equals("endDate")) {
			otherDate = "startDate";
			this.setInvalid(otherDate);
			this.validateDates();
		} else if (propertyName.equals("release"))
			this.validateRelease();

		// We only get here if validator didn't throw an exception
		this.setValid(propertyName);
		// Dates should be valid or invalid together, simplifies things for the
		// user
		if (otherDate != null)
			this.setValid(otherDate);
	}

	@Override
	public void commandPerformed(ActionEvent evt) throws IOException {
		String commandType = evt.getActionCommand();
		if (commandType.equals("cancel"))
			this.cancelChanges();
		else if (commandType.equals("action"))
			try {
				this.performAction();
			} catch (InvalidInputException e) {
				// TODO The validations should prevents an actual error, but if
				// this happens, what do we want to do?
				e.printStackTrace();
			}
		else if (commandType.equals("delete"))
			try {
				this.delete();
			} catch (InvalidIterationInputException e) {
				// TODO see the above to-do
				e.printStackTrace();
			}
	}

	private void validateName() throws InvalidInputException {
		IterationManager.validateName(this);
	}

	private void validateDates() throws InvalidInputException {
		IterationManager.validateDates(this);
	}

	private void validateRelease() throws InvalidInputException {
		IterationManager.validateRelease(this);
	}

	public void cancelChanges() throws IOException {
		if (this.getMode() == Mode.CREATE)
			this.cancelCreate();
		else
			this.cancelEdit();
	}

	public void performAction() throws InvalidInputException {
		if (this.getMode() == Mode.CREATE)
			this.createIteration();
		else if (this.getMode() == Mode.EDIT)
			this.updateIteration();
	}

	public void cancelCreate() throws IOException {
		this.close();
	}

	public void cancelEdit() throws IOException {
		this.setIteration(this.iteration);
	}

	public void createIteration() throws InvalidInputException {
		IterationManager.create(this.getName(), this.getProject(), this
				.getRelease(), this.getStartDate(), this.getEndDate());
		this.fireStateChange();
	}

	public void updateIteration() throws InvalidInputException {
		IterationManager.update(iteration, this.getName(), this.getStartDate(),
				this.getEndDate(), this.getRelease());
		this.fireStateChange();
	}

	public void delete() throws InvalidIterationInputException {
		ConfirmResult choice = this.view
				.askYesNoCancelQuestion(
						"Delete Iteration",
						"Would you like to delete all user stories under this requirement?  \"No\" will send all requirements in this iteration to the backlog.");
		if (choice == ConfirmResult.CANCEL)
			return; // User didn't confirm - don't delete
		else if (choice == ConfirmResult.YES)
			IterationManager.delete(iteration, true);
		else if (choice == ConfirmResult.NO)
			IterationManager.delete(iteration, false);

		this.fireStateChange();
	}

	/*
	 * public void saveIteration() throws IOException {
	 * this.iteration.setName((String) this.view.getProperty("name"));
	 * this.iteration.setStartDate((Date) this.view.getProperty("startDate"));
	 * this.iteration.setEndDate((Date) this.view.getProperty("endDate"));
	 * this.iteration.setRelease((Release) this.view.getProperty("release"));
	 * 
	 * this.iteration.save();
	 * 
	 * this.setMode(Mode.SHOW); this.fireStateChange(); }
	 */
	@Override
	public Date getEndDate() {
		Log.debug("getEndDate():" + this.view.getProperty("endDate"));
		return (Date) this.view.getProperty("endDate");
	}

	@Override
	public String getName() {
		Log.debug("getName():" + this.view.getProperty("name"));
		return (String) this.view.getProperty("name");

	}

	@Override
	public Project getProject() {
		if (this.iteration == null)
			return this.container.getState().getProject();
		else
			return iteration.getProject();
	}

	@Override
	public Release getRelease() {
		Log.debug("getRelease():" + this.view.getProperty("release"));
		return (Release) this.view.getProperty("release");
	}

	@Override
	public Date getStartDate() {
		Log.debug("getStartDate():" + this.view.getProperty("startDate"));
		return (Date) this.view.getProperty("startDate");
	}

	@Override
	public UserStory[] getUserStories() {
		if (this.iteration == null)
			return new UserStory[0];
		else
			return iteration.getUserStories();
	}

	@Override
	public int getID() {
		if (this.iteration == null)
			return -1;
		else
			return iteration.getID();
	}
}
