/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *    Ian Williams - Initial design and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components.editor.entity;

import java.awt.event.ActionEvent;
import java.io.IOException;
import java.sql.SQLException;

import edu.wpi.wpisuite.components.EntityIconSet.IconType;
import edu.wpi.wpisuite.components.editor.Editor;
import edu.wpi.wpisuite.components.editor.EditorContainer;
import edu.wpi.wpisuite.components.editor.InvalidInputException;
import edu.wpi.wpisuite.data.Project;
import edu.wpi.wpisuite.data.ProjectMember;
import edu.wpi.wpisuite.data.errors.InvalidProjectInputException;
import edu.wpi.wpisuite.data.funcs.Artifact;
import edu.wpi.wpisuite.data.funcs.ProjectManager;
import edu.wpi.wpisuite.database.Database;


/**
 * Defines the idea of a Project editing interface without actually specifying
 * an implementation thereof. Interfaces between a View which provides that
 * implementation and the Requirements Manager itself.
 */
public class ProjectEditor extends Editor {
	private Project project;

	public ProjectEditor(EditorContainer rm) {
		super(rm, ProjectEditorPanelView.class);
		this.setNewProject();
	}

	public ProjectEditor(EditorContainer rm, Project project) {
		super(rm, ProjectEditorPanelView.class);
		this.setProject(project);
	}

	public void setProject(Project p) {
		this.project = p;

		String header = "";
		if (p != null) {
			header = "Project - " + new Artifact(p).toString();
			this.view.setPropertyIcon("header", entityIcons
					.getProjectIcon(IconType.DEFAULT));
		} else {
			header = "New Project";
			this.view.setPropertyIcon("header", entityIcons
					.getProjectIcon(IconType.CREATE));
		}
		this.view.setProperty("header", header);

		String name = "";
		if (p != null)
			name = p.getName();
		this.view.setProperty("name", name);

		String desc = "";
		if (p != null)
			desc = p.getDescription();
		this.view.setProperty("description", desc);

		if (p == null)
			this.setMode(Mode.CREATE);
		else
			this.setMode(Mode.SHOW);
	}

	public void setNewProject() {
		this.setProject(null);
	}

	@Override
	public void validateProperty(String propertyName, Object newValue)
			throws InvalidInputException {
		if (this.getMode() == Mode.SHOW)
			this.setMode(Mode.EDIT);
		this.setInvalid(propertyName);

		if (propertyName.equals("name"))
			this.validateName((String) newValue);
		else if (propertyName.equals("description"))
			this.validateDescription((String) newValue);

		// We only get here if validator didn't throw an exception
		this.setValid(propertyName);
	}

	@Override
	public void commandPerformed(ActionEvent evt) throws IOException {
		String commandType = evt.getActionCommand();
		if (commandType.equals("cancel"))
			this.cancelChanges();
		else if (commandType.equals("action"))
			this.performAction();
		else if (commandType.equals("delete"))
			this.delete();

	}

	public void validateName(String name) throws InvalidInputException {
		if (name == null || name.equals(""))
			throw new InvalidInputException("Cannot be empty");
	}

	public void validateDescription(String desc) throws InvalidInputException {
		if (desc == null || desc.equals(""))
			throw new InvalidInputException("Cannot be empty");
	}

	public void cancelChanges() throws IOException {
		if (this.getMode() == Mode.CREATE)
			this.cancelCreate();
		else
			this.cancelEdit();
	}

	public void performAction() throws IOException {
		if (this.getMode() == Mode.CREATE)
			this.createProject();
		else if (this.getMode() == Mode.EDIT)
			this.saveProject();
	}

	public void delete() throws IOException {
		if (!this.view
				.confirmAction("Are you sure you want to delete this project, and all associated requirements, iterations, and releases?"))
			return; // User didn't confirm - don't delete

		try {
			Project curProj = container.getState().getProject();			
			ProjectManager.delete(this.project);
			if (curProj != null && curProj.equals(this.project))
				this.container.getState().setProject(null);
			this.project = null;
			
		} catch (InvalidProjectInputException e) {
			throw new IOException(
			"A database error occured while deleting this project.  Error Message: " + e.getMessage());
		}
		this.fireStateChange();
		this.close();
	}

	public void cancelCreate() throws IOException {
		this.close();
	}

	public void cancelEdit() throws IOException {
		this.setProject(this.project);
	}

	public void createProject() throws IOException {
		ProjectMember newAssoc = null;

		try {
			this.project = Database.manager().create(Project.class);
			newAssoc = Database.manager().create(ProjectMember.class);
		} catch (SQLException e) {
			throw new IOException(
					"A database error occured while creating this feature.");
		}

		newAssoc.setProject(this.project);
		newAssoc.setMember(this.container.getState().getUser());
		newAssoc.save();

		this.saveProject();

	}

	public void saveProject() throws IOException {
		// Need to grab these BEFORE doing any updates, because otherwise any
		// update on a currently selected project triggers a global state
		// change,
		// which naturally resets these fields :P
		String name = (String) this.view.getProperty("name");
		String description = (String) this.view.getProperty("description");

		this.project.removePropertyChangeListener(this.container.getState());
		this.project.setName(name);
		this.project.setDescription(description);
		this.project.save();
		this.project.addPropertyChangeListener(this.container.getState());
		
		this.container.getState().setProject(project);
		this.setMode(Mode.SHOW);
		this.fireStateChange();
		this.setProject(this.project); // resets buttons
	}

	public Project getProject() {
		return this.project;
	}

}
