/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components.editor.entity;

import java.awt.event.ActionEvent;
import java.io.IOException;
import java.sql.SQLException;
import java.util.Date;


import edu.wpi.wpisuite.components.EntityIconSet.IconType;
import edu.wpi.wpisuite.components.editor.Editor;
import edu.wpi.wpisuite.components.editor.EditorContainer;
import edu.wpi.wpisuite.components.editor.InvalidInputException;
import edu.wpi.wpisuite.data.Release;
import edu.wpi.wpisuite.data.funcs.Artifact;
import edu.wpi.wpisuite.database.Database;

/**
 * Defines the idea of a Release editing interface without actually specifying
 * an implementation thereof. Interfaces between a View which provides that
 * implementation and the Requirements Manager itself.
 */
public class ReleaseEditor extends Editor {
	private Release release;

	public ReleaseEditor(EditorContainer rm) {
		super(rm, ReleaseEditorPanelView.class);
		this.setNewRelease();
	}

	public ReleaseEditor(EditorContainer rm, Release release) {
		super(rm, ReleaseEditorPanelView.class);
		this.setRelease(release);
	}

	public void setRelease(Release re) {
		this.release = re;

		this.view.setProperty("header", "Release - "
				+ new Artifact(re).toString());
		this.view.setProperty("name", re.getName());
		this.view.setProperty("expectedDate", re.getExpectedDate());
		this.view.setProperty("actualDate", re.getActualDate());

		this.view.setPropertyIcon("header", entityIcons
				.getReleaseIcon(IconType.DEFAULT));

		this.setMode(Mode.SHOW);
	}

	public void setNewRelease() {
		this.release = null;

		this.view.setProperty("header", "New Release");
		this.view.setProperty("name", "");
		this.view.setProperty("expectedDate", null);
		this.view.setProperty("actualDate", null);

		this.view.setPropertyIcon("header", entityIcons
				.getReleaseIcon(IconType.CREATE));

		this.setMode(Mode.CREATE);
	}

	@Override
	public void validateProperty(String propertyName, Object newValue)
			throws InvalidInputException {
		if (this.getMode() == Mode.SHOW)
			this.setMode(Mode.EDIT);
		this.setInvalid(propertyName);

		if (propertyName.equals("name"))
			this.validateName((String) newValue);
		else if (propertyName.equals("expectedDate"))
			this.validateExpectedDate((Date) newValue);
		else if (propertyName.equals("actualDate"))
			this.validateActualDate((Date) newValue);

		// We only get here if validator didn't throw an exception
		this.setValid(propertyName);
	}

	@Override
	public void commandPerformed(ActionEvent evt) throws IOException {
		String commandType = evt.getActionCommand();
		if (commandType.equals("cancel"))
			this.cancelChanges();
		else if (commandType.equals("action"))
			this.performAction();
	}

	public void validateName(String name) throws InvalidInputException {
		if (name.equals(""))
			throw new InvalidInputException("Cannot be empty");
	}

	public void validateExpectedDate(Date expectedDate)
			throws InvalidInputException {

	}

	public void validateActualDate(Date actualDate)
			throws InvalidInputException {

	}

	public void cancelChanges() throws IOException {
		if (this.getMode() == Mode.CREATE)
			this.cancelCreate();
		else
			this.cancelEdit();
	}

	public void performAction() throws IOException {
		if (this.getMode() == Mode.CREATE)
			this.createRelease();
		else if (this.getMode() == Mode.EDIT)
			this.saveRelease();
	}

	public void cancelCreate() throws IOException {
		this.close();
	}

	public void cancelEdit() throws IOException {
		this.setRelease(this.release);
	}

	public void createRelease() throws IOException {
		try {
			this.release = Database.manager().create(Release.class);
		} catch (SQLException e) {
			throw new IOException(
					"A database error occured while creating this release.");
		}

		this.release.setProject(this.container.getState().getProject());

		this.saveRelease();

		this.fireStateChange();
	}

	public void saveRelease() throws IOException {
		this.release.setName((String) this.view.getProperty("name"));
		this.release.setExpectedDate((Date) this.view
				.getProperty("expectedDate"));
		this.release.setActualDate((Date) this.view.getProperty("actualDate"));

		this.release.save();

		this.setMode(Mode.SHOW);
		this.fireStateChange();
	}
}
