/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components.editor.entity;

import java.awt.event.ActionEvent;
import java.io.IOException;
import java.sql.SQLException;


import edu.wpi.wpisuite.components.editor.Editor;
import edu.wpi.wpisuite.components.editor.EditorContainer;
import edu.wpi.wpisuite.components.editor.InvalidInputException;
import edu.wpi.wpisuite.data.ProjectMember;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.data.funcs.UserManager;
import edu.wpi.wpisuite.database.Database;

/**
 * Defines the idea of a User editing interface without actually specifying an
 * implementation thereof. Interfaces between a View which provides that
 * implementation and the Requirements Manager itself.
 */
public class UserEditor extends Editor {
	private User user;

	public UserEditor(EditorContainer rm) {
		super(rm, UserEditorPanelView.class);
		this.setNewUser();
	}

	public UserEditor(EditorContainer rm, User user) {
		super(rm, UserEditorPanelView.class);
		this.setUser(user);
	}

	public void setUser(User u) {
		this.user = u;

		this.view.setProperty("username", u.getUsername());
		this.view.setProperty("firstName", u.getFirstName());
		this.view.setProperty("lastName", u.getLastName());
		this.view.setProperty("emailAddress", u.getEmailAddress());
		this.view.setProperty("isAdmin", u.isAdmin());
		this.view.setProperty("isLocked", u.isLocked());

		if (u != null && u.equals(container.getState().getUser()))
			this.view.setPropertyEnabled("isAdmin", false); // can't modify own
		// admin status

		this.setMode(Mode.SHOW);
	}

	public void setNewUser() {
		this.user = null;

		this.view.setProperty("username", "");
		this.view.setProperty("firstName", "");
		this.view.setProperty("lastName", "");
		this.view.setProperty("emailAddress", "");
		this.view.setProperty("isAdmin", false);
		this.view.setProperty("isLocked", false);

		this.setMode(Mode.CREATE);
	}

	@Override
	public void validateProperty(String propertyName, Object newValue)
			throws InvalidInputException {
		if (this.getMode() == Mode.SHOW)
			this.setMode(Mode.EDIT);
		this.setInvalid(propertyName);

		if (propertyName.equals("username"))
			this.validateUsername((String) newValue);
		else if (propertyName.equals("firstName"))
			this.validateFirstName((String) newValue);
		else if (propertyName.equals("lastName"))
			this.validateLastName((String) newValue);
		else if (propertyName.equals("emailAddress"))
			this.validateEmailAddress((String) newValue);

		// We only get here if validator didn't throw an exception
		this.setValid(propertyName);
	}

	@Override
	public void commandPerformed(ActionEvent evt) throws IOException {
		String commandType = evt.getActionCommand();
		if (commandType.equals("cancel"))
			this.cancelChanges();
		else if (commandType.equals("action"))
			this.performAction();
		else if (commandType.equals("delete"))
			this.delete();
	}

	public void validateUsername(String name) throws InvalidInputException {
		if (name.equals(""))
			throw new InvalidInputException("Cannot be empty");
		try {
			User[] withUsername = Database.manager().find(User.class,
					"username = ?", name);
			if (withUsername.length > 0 && // there is someone with this name
					(this.user == null || // we're null, so we can't be it
					withUsername[0].getID() != this.user.getID()))
				throw new InvalidInputException("This username already exists");
		} catch (SQLException e) {
			throw new InvalidInputException(
					"Database error occured - username might already exist");
		}
	}

	public void validateFirstName(String name) throws InvalidInputException {
		if (name.equals(""))
			throw new InvalidInputException("Cannot be empty");
	}

	public void validateLastName(String name) throws InvalidInputException {
		if (name.equals(""))
			throw new InvalidInputException("Cannot be empty");
	}

	public void validateEmailAddress(String email) throws InvalidInputException {
		if (email.equals(""))
			throw new InvalidInputException("Cannot be empty");
	}

	public void cancelChanges() throws IOException {
		if (this.getMode() == Mode.CREATE)
			this.cancelCreate();
		else
			this.cancelEdit();
	}

	public void performAction() throws IOException {
		if (this.getMode() == Mode.CREATE)
			this.createUser();
		else if (this.getMode() == Mode.EDIT)
			this.saveUser();
	}

	public void cancelCreate() throws IOException {
		this.close();
	}

	public void cancelEdit() throws IOException {
		this.setUser(this.user);
	}

	public void createUser() throws IOException {
		try {
			this.user = UserManager.create((String) view
					.getProperty("username"), (String) view
					.getProperty("firstName"), (String) view
					.getProperty("lastName"), (String) view
					.getProperty("emailAddress"));
		} catch (Exception e) {
			throw new IOException(e.getMessage());
		}

		this.saveUser();

		this.fireStateChange();
	}

	public void delete() throws IOException {
		if (!this.view
				.confirmAction("Are you sure you want to delete this user?"))
			return;
		// user didn't confirm

		try {
			ProjectMember[] projectMemberAssocs = Database.manager().find(
					ProjectMember.class, "memberID = ?", this.user);
			Database.manager().delete(projectMemberAssocs);
			Database.manager().delete(this.user);
			this.user = null;
		} catch (SQLException e) {
			e.printStackTrace();
			throw new IOException(
					"A database error occured while deleting this user");
		}

		this.fireStateChange();
	}

	public void saveUser() throws IOException {
		this.user.setUsername((String) this.view.getProperty("username"));
		this.user.setFirstName((String) this.view.getProperty("firstName"));
		this.user.setLastName((String) this.view.getProperty("lastName"));
		this.user.setEmailAddress((String) this.view
				.getProperty("emailAddress"));
		this.user.setLocked((Boolean) this.view.getProperty("isLocked"));
		this.user.setAdmin((Boolean) this.view.getProperty("isAdmin"));
		this.user.save();

		this.setMode(Mode.SHOW);
		this.fireStateChange();
	}
}
