/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.components.editor.entity;

import java.awt.event.ActionEvent;
import java.io.IOException;



import edu.wpi.wpisuite.components.EntityIconSet.IconType;
import edu.wpi.wpisuite.components.editor.Editor;
import edu.wpi.wpisuite.components.editor.EditorContainer;
import edu.wpi.wpisuite.components.editor.InvalidInputException;
import edu.wpi.wpisuite.data.Iteration;
import edu.wpi.wpisuite.data.Project;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.data.UserStory;
import edu.wpi.wpisuite.data.UserStory.Status;
import edu.wpi.wpisuite.data.funcs.Artifact;
import edu.wpi.wpisuite.data.funcs.UserStoryManager;
import edu.wpi.wpisuite.data.util.UserStoryAccessor;
import edu.wpi.wpisuite.module.ActiveStateEvent;
import edu.wpi.wpisuite.module.ActiveStateListener;
import edu.wpi.wpisuite.module.ActiveStateEvent.ActiveStateEventType;

/**
 * Defines the idea of a UserStory editing interface without actually specifying
 * an implementation thereof. Interfaces between a View which provides that
 * implementation and the Requirements Manager itself.
 */
public class UserStoryEditor extends Editor implements UserStoryAccessor {
	private UserStory userStory;

	public UserStoryEditor(EditorContainer rm) {
		super(rm, UserStoryEditorPanelView.class);
		this.initIterationList();
		this.setNewUserStory();
	}

	public UserStoryEditor(EditorContainer rm, UserStory userStory) {
		super(rm, UserStoryEditorPanelView.class);
		this.initIterationList();
		this.setUserStory(userStory);
	}

	public void initIterationList() {
		this.updateIterationList();
		this.container.getState().registerListener(new ActiveStateListener() {
			@Override
			public void stateChange(ActiveStateEvent evt) {
				if (evt.getType().equals(ActiveStateEventType.PROJECT))
					UserStoryEditor.this.updateIterationList();
			}
		});
	}

	public void updateIterationList() {
		Project p = this.container.getState().getProject();
		if (p != null) {
			Iteration[] iterationList = p.getIterations();
			this.view.setPropertyOptions("iteration", iterationList);
		} else
			this.view.setPropertyOptions("iteration", null);
	}

	public void setUserStory(UserStory u) {
		this.userStory = u;

		this.view.setProperty("header", "User Story - "
				+ new Artifact(u).toString());
		this.view.setProperty("title", u.getTitle());
		this.view.setProperty("description", u.getDescription());
		this.view.setProperty("status", u.getStatus());
		this.view.setProperty("actualUnits", u.getActualUnits());
		this.view.setProperty("estimatedUnits", u.getEstimatedUnits());
		this.view.setProperty("iteration", u.getIteration());

		this.view.setPropertyIcon("header", entityIcons
				.getUserStoryIcon(IconType.DEFAULT));

		this.setOwnerProperty(u.getOwner());
		this.setMode(Mode.SHOW);
	}

	public void setNewUserStory() {
		this.userStory = null;

		this.view.setProperty("header", "New User Story");
		this.view.setProperty("title", "");
		this.view.setProperty("description", "");
		this.view.setProperty("status", Status.OPEN);
		this.view.setProperty("actualUnits", 0);
		this.view.setProperty("estimatedUnits", 0);
		this.view.setProperty("iteration", null);

		this.view.setPropertyIcon("header", entityIcons
				.getUserStoryIcon(IconType.CREATE));

		this.setOwnerProperty(null);
		this.setMode(Mode.CREATE);
	}

	public void setOwnerProperty(User newOwner) {
		Project p = this.container.getState().getProject();
		User[] members;
		if (p != null)
			members = this.container.getState().getProject().getMembers();
		else
			members = new User[0];
		User[] membersWithNull = new User[1 + members.length];
		membersWithNull[0] = null;
		for (int i = 0; i < members.length; i++)
			membersWithNull[i + 1] = members[i];

		this.view.setPropertyOptions("owner", membersWithNull);
		this.view.setProperty("owner", newOwner);

		this.validateClaimable(newOwner);
	}

	public void validateClaimable(User newOwner) {
		this.view.setPropertyEnabled("claimOwnership", true);
		if (newOwner != null)
			// Can't claim what you already own
			if (newOwner.getID() == this.container.getState().getUser().getID())
				this.view.setPropertyEnabled("claimOwnership", false);
	}

	@Override
	public void validateProperty(String propertyName, Object newValue)
			throws InvalidInputException {
		if (this.getMode() == Mode.SHOW)
			this.setMode(Mode.EDIT);
		this.setInvalid(propertyName);

		if (propertyName.equals("title"))
			this.validateTitle((String) newValue);
		else if (propertyName.equals("description"))
			this.validateDescription((String) newValue);
		else if (propertyName.equals("estimatedUnits"))
			this.validateEstimatedUnits((String) newValue);
		else if (propertyName.equals("actualUnits"))
			this.validateActualUnits((String) newValue);
		else if (propertyName.equals("status"))
			this.validateStatus((Status) newValue);
		else if (propertyName.equals("owner"))
			this.validateOwner((User) newValue);
		else if (propertyName.equals("iteration"))
			this.validateIteration((Iteration) newValue);

		// We only get here if validator didn't throw an exception
		this.setValid(propertyName);
	}

	@Override
	public void commandPerformed(ActionEvent evt) throws IOException {
		String commandType = evt.getActionCommand();
		if (commandType.equals("cancel"))
			this.cancelChanges();
		else if (commandType.equals("action"))
			try {
				this.performAction();
			} catch (InvalidInputException e) {
				view.error(e);
			}
		else if (commandType.equals("claimOwnership"))
			this.claimOwnership();
		else if (commandType.equals("delete"))
			try {
				this.delete();
			} catch (InvalidInputException e) {
				view.error(e);
			}
	}

	public void validateOwner(User owner) throws InvalidInputException {
		this.validateClaimable(owner);
		UserStoryManager.validateOwner(this);
	}

	public void validateTitle(String title) throws InvalidInputException {
		UserStoryManager.validateTitle(this);
	}

	public void validateDescription(String description)
			throws InvalidInputException {
		UserStoryManager.validateDescription(this);

	}

	public void validateStatus(Status status) throws InvalidInputException {
		// Intentionally blank, statuses may be any Status or even null (though
		// GUI prevents this)
	}

	public void validateEstimatedUnits(String estimatedUnits)
			throws InvalidInputException {
		try {
			Integer.parseInt(estimatedUnits);
			UserStoryManager.validateEstimatedUnits(this);
		} catch (NumberFormatException e) {
			throw new InvalidInputException("Must be an integer.");
		}
	}

	public void validateActualUnits(String actualUnits)
			throws InvalidInputException {
		try {
			Integer.parseInt(actualUnits);
			UserStoryManager.validateActualUnits(this);
		} catch (NumberFormatException e) {
			throw new InvalidInputException("Must be an integer.");
		}
	}

	public void validateIteration(Iteration iteration)
			throws InvalidInputException {
		if (userStory != null)
			UserStoryManager.validateIteration(this, userStory.getIteration());
		else
			UserStoryManager.validateIteration(this);
	}

	public void cancelChanges() throws IOException {
		if (this.getMode() == Mode.CREATE)
			this.cancelCreate();
		else
			this.cancelEdit();
	}

	public void claimOwnership() throws IOException {
		if (this.userStory != null && this.userStory.getOwner() != null)
			if (!this.view
					.confirmAction("This requirement is already owned by user "
							+ this.userStory.getOwner().getUsername() + ".\n"
							+ "Replace this user as this requirement's owner?"))
				return; // User did NOT confirm

		// Either no owner, or user confirmed
		this.setOwnerProperty(this.container.getState().getUser());
	}

	public void performAction() throws InvalidInputException {
		if (this.getMode() == Mode.CREATE)
			this.createUserStory();
		else if (this.getMode() == Mode.EDIT)
			this.updateUserStory();
	}

	public void cancelCreate() throws IOException {
		this.close();
	}

	public void delete() throws InvalidInputException {

		if (!this.view
				.confirmAction("Are you sure you want to delete this requirement?"))
			return; // User didn't confirm - don't delete

		UserStoryManager.delete(userStory);

		this.fireStateChange();
		this.close();
	}

	public void cancelEdit() throws IOException {
		this.setUserStory(this.userStory);
	}

	public void createUserStory() throws InvalidInputException {

		userStory = UserStoryManager.create(getTitle(), getDescription(),
				getEstimatedUnits(), getActualUnits(), getStatus(),
				getAuthor(), getOwner(), getIteration(), getProject());
		this.saveUserStory();
	}

	public void updateUserStory() throws InvalidInputException {
		UserStoryManager.update(userStory, getTitle(), getDescription(),
				getEstimatedUnits(), getActualUnits(), getStatus(),
				getAuthor(), getOwner(), getIteration(), getProject());
		this.saveUserStory();
	}

	public void saveUserStory() {
		this.setMode(Mode.SHOW);
		this.fireStateChange();
		this.setUserStory(this.userStory); // resets buttons
	}

	@Override
	public int getActualUnits() {
		return Integer.parseInt((String) this.view.getProperty("actualUnits"));
	}

	@Override
	public User getAuthor() {
		if (userStory == null)
			return this.container.getState().getUser();
		else
			return userStory.getAuthor();
	}

	@Override
	public int getEstimatedUnits() {
		return Integer.parseInt((String) this.view
				.getProperty("estimatedUnits"));
	}

	@Override
	public Iteration getIteration() {
		return (Iteration) this.view.getProperty("iteration");
	}

	@Override
	public User getOwner() {
		return (User) this.view.getProperty("owner");
	}

	@Override
	public Project getProject() {
		if (userStory == null)
			return this.container.getState().getProject();
		else
			return userStory.getProject();
	}

	@Override
	public Status getStatus() {
		return (Status) this.view.getProperty("status");
	}

	@Override
	public String getDescription() {
		return (String) this.view.getProperty("description");
	}

	@Override
	public int getID() {
		if (userStory == null)
			return -1;
		else
			return userStory.getID();
	}

	@Override
	public String getTitle() {
		return (String) this.view.getProperty("title");
	}
}
