/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.data.funcs;

import net.java.ao.Entity;
import net.java.ao.RawEntity;
import edu.wpi.wpisuite.data.Config;
import edu.wpi.wpisuite.data.Feature;
import edu.wpi.wpisuite.data.Iteration;
import edu.wpi.wpisuite.data.Project;
import edu.wpi.wpisuite.data.Release;
import edu.wpi.wpisuite.data.TraceabilityAssociation;
import edu.wpi.wpisuite.data.TraceabilityMatrix;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.data.UserStory;
import edu.wpi.wpisuite.database.Database;

/**
 * Represents the artifact string for any given entity. Implements a constructor
 * based on an entity and a toString method which returns the appropriate
 * artifact string.
 */
public class Artifact {
	/**
	 * The minimum length for the zero-padded ID numbers in the Artifact strings
	 */
	public static final int ID_LENGTH = 5;

	private Entity ent;
	private Class<? extends RawEntity<?>> entClass;

	/**
	 * Constructs an Artifact string generator for the given entity
	 * 
	 * @param ent
	 *            The entity to generate an artifact string for
	 */
	public Artifact(Entity ent) {
		this.ent = ent;
		this.entClass = ent.getEntityType();
	}

	/**
	 * Constructs an artifact with a non-numeric id for the given entity class
	 * 
	 * @param entClass
	 *            The class to generate a template artifact for
	 */
	public Artifact(Class<? extends RawEntity<?>> entClass) {
		this.ent = null;
		this.entClass = entClass;
	}

	/**
	 * Grabs from the database, or if the database doesn't contain the given
	 * type string configuration, sets the database config to the default and
	 * returns that
	 * 
	 * @return The type string for the contained entity.
	 */
	private String getTypeString() {
		String configName = this.entClass + "_artifact_string";

		String defaultTypeString = this.getDefaultTypeString();

		Config[] configs = {};
		try {
			configs = Database.manager().find(Config.class, "name = ?",
					configName);
		} catch (Exception e) {
			return defaultTypeString;
		}

		Config config;
		if (configs.length == 0)
			try {
				config = Database.manager().create(Config.class);
				config.setName(configName);
				config.setValue(defaultTypeString);
				config.save();
			} catch (Exception e) {
				return defaultTypeString;
			}
		else
			config = configs[0];

		return config.getValue();
	}

	/**
	 * @return The default type string for the contained entity
	 */
	private String getDefaultTypeString() {
		if (this.entClass.isAssignableFrom(Project.class))
			return "PRJ";
		else if (this.entClass.isAssignableFrom(User.class))
			return "USR";
		else if (this.entClass.isAssignableFrom(Feature.class))
			return "FR";
		else if (this.entClass.isAssignableFrom(Iteration.class))
			return "ITR";
		else if (this.entClass.isAssignableFrom(Release.class))
			return "REL";
		else if (this.entClass.isAssignableFrom(UserStory.class))
			return "US";
		else if (this.entClass.isAssignableFrom(TraceabilityAssociation.class))
			return "TRA";
		else if (this.entClass.isAssignableFrom(TraceabilityMatrix.class))
			return "TM";
		else
			return "???";
	}

	/**
	 * @return The contained entity's ID number, padded to the left with zeros
	 */
	private String getPaddedID() {
		if (this.ent == null)
			return Database.joinCopies("x", Artifact.ID_LENGTH, "");

		String idStr = this.ent.getID() + "";
		while (idStr.length() < Artifact.ID_LENGTH)
			idStr = "0" + idStr;
		return idStr;
	}

	/**
	 * @return A string representation (an Artifact string) of the contained
	 *         entity
	 */
	@Override
	public String toString() {
		return this.getTypeString() + this.getPaddedID();
	}

	/**
	 * @return A string representation (usually a name or title) of the
	 *         contained entity
	 */
	public String getSimpleName() {
		if (this.ent == null)
			return "Null Entity";
		else if (this.ent instanceof Project)
			return ((Project) this.ent).getName();
		else if (this.ent instanceof User)
			return ((User) this.ent).getUsername();
		else if (this.ent instanceof Feature)
			return ((Feature) this.ent).getTitle();
		else if (this.ent instanceof Iteration)
			return ((Iteration) this.ent).getName();
		else if (this.ent instanceof Release)
			return ((Release) this.ent).getName();
		else if (this.ent instanceof UserStory)
			return ((UserStory) this.ent).getTitle();
		else if (this.ent instanceof TraceabilityMatrix)
			return ((TraceabilityMatrix) this.ent).getName();

		return "Unknown Entity: " + this.ent.toString();
	}
}
