/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Ian Williams - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.data.funcs;

import java.sql.SQLException;

import net.java.ao.DBParam;
import net.java.ao.Query;
import edu.wpi.wpisuite.data.Feature;
import edu.wpi.wpisuite.data.Project;
import edu.wpi.wpisuite.data.Release;
import edu.wpi.wpisuite.data.TraceabilityAssociation;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.data.UserStory;
import edu.wpi.wpisuite.data.errors.FeatureErrorType;
import edu.wpi.wpisuite.data.errors.InvalidFeatureInputException;
import edu.wpi.wpisuite.data.errors.InvalidUserStoryInputException;
import edu.wpi.wpisuite.data.util.FeatureAccessor;
import edu.wpi.wpisuite.database.Database;

// TODO: Shouldn't be able to put a feature in a release that has always
// TODO: Needs a unit test

/**
 * Class of CRUD functions for Features Needs unit testing.
 */
public class FeatureManager extends EntityManager {

	// Singleton, used for managing listeners
	private FeatureManager() {
		super();
	}

	private static FeatureManager manager = new FeatureManager();

	// Public CRUD functions

	/**
	 * Creates a new Feature. Fills in null for all missing fields.
	 * 
	 * @param title
	 *            Must not begin with a space, contain only spaces, be empty,
	 *            null or duplicate of any other feature's title in this project
	 * @param description
	 *            Any non-null or empty string.
	 * @param project
	 * @return A reference to the newly created Feature.
	 * @throws InvalidFeatureInputException
	 *             One of any error detailed in FeatureErrorType.
	 */
	public static Feature create(String title, String description,
			Project project) throws InvalidFeatureInputException {

		Feature feature;

		checkProject(project);
		checkTitle(title, project);
		checkDescription(description);

		try {
			feature = Database.manager().create(Feature.class,
					new DBParam("title", title));
		} catch (SQLException e) {
			throw new InvalidFeatureInputException(
					FeatureErrorType.DATABASE_FEATURE_CREATION, e.getMessage());
		}

		feature.setProject(project);
		feature.setDescription(description);
		feature.save();

		manager.fireChange(new EntityEvent(feature,
				EntityEvent.EventType.CREATE));

		return feature;
	}

	/**
	 * Creates a new Feature.
	 * 
	 * @param title
	 *            Must not begin with a space, contain only spaces, be empty,
	 *            null or duplicate of any other feature's title in this project
	 * @param description
	 *            Any non-null or empty string.
	 * @param project
	 * @param release
	 *            Is allowed to be null. Must be a release in the feature's
	 *            project.
	 * @param author
	 *            Is allowed to be null. Must be a user of this feature's
	 *            project.
	 * @param owner
	 *            Is allowed to be null. Must be a user of this feature's
	 *            project.
	 * @return A reference to the newly created Feature.
	 * @throws InvalidFeatureInputException
	 *             One of any errors detailed in FeatureErrorType.
	 */
	public static Feature create(String title, String description,
			Project project, Release release, User author, User owner)
			throws InvalidFeatureInputException {

		Feature feature;

		checkProject(project);
		checkTitle(title, project);
		checkDescription(description);
		checkRelease(release, project);
		checkAuthor(author, project);
		checkOwner(owner, project);

		try {
			feature = Database.manager().create(Feature.class,
					new DBParam("title", title));
		} catch (SQLException e) {
			throw new InvalidFeatureInputException(
					FeatureErrorType.DATABASE_FEATURE_CREATION, e.getMessage());
		}

		feature.setProject(project);
		feature.setRelease(release);
		feature.setDescription(description);
		feature.setAuthor(author);
		feature.setOwner(owner);
		feature.save();

		manager.fireChange(new EntityEvent(feature,
				EntityEvent.EventType.CREATE));

		return feature;
	}

	/**
	 * Updates the title of this feature.
	 * 
	 * @param story
	 *            Should be non-null.
	 * @param title
	 *            Must meet the usual guidelines detailed in this class's create
	 *            functions.
	 * @throws InvalidFeatureInputException
	 *             One of any errors detailed in FeatureErrorType.
	 */
	public static void updateTitle(Feature feature, String title)
			throws InvalidFeatureInputException {
		if (feature == null)
			throw new InvalidFeatureInputException(FeatureErrorType.NULL);
		checkTitle(title, feature.getProject(), feature.getID());
		feature.setTitle(title);
		feature.save();
		manager.fireChange(new EntityEvent(feature,
				EntityEvent.EventType.UPDATE));
	}

	/**
	 * Updates this feature's description.
	 * 
	 * @param story
	 *            Should be non-null.
	 * @param description
	 *            Must meet the usual guidelines detailed in this class's create
	 *            functions.
	 * @throws InvalidFeatureInputException
	 *             One of any error detailed in FeatureErrorType.
	 */
	public static void updateDescription(Feature feature, String description)
			throws InvalidFeatureInputException {
		if (feature == null)
			throw new InvalidFeatureInputException(FeatureErrorType.NULL);
		checkTitle(description, feature.getProject(), feature.getID());
		feature.setDescription(description);
		feature.save();
		manager.fireChange(new EntityEvent(feature,
				EntityEvent.EventType.UPDATE));
	}

	/**
	 * Updates this feature's author.
	 * 
	 * @param story
	 *            Should be non-null.
	 * @param author
	 *            Must meet the usual guidelines detailed in this class's create
	 *            functions.
	 * @throws InvalidUserStoryInputException
	 *             One of any errors detailed in FeatureErrorType.
	 */
	public static void updateAuthor(Feature feature, User author)
			throws InvalidFeatureInputException {
		if (feature == null)
			throw new InvalidFeatureInputException(FeatureErrorType.NULL);
		checkAuthor(author, feature.getProject());
		feature.setAuthor(author);
		feature.save();
		manager.fireChange(new EntityEvent(feature,
				EntityEvent.EventType.UPDATE));
	}

	/**
	 * Updates this feature's owner.
	 * 
	 * @param story
	 *            Should be non-null.
	 * @param owner
	 *            Must meet the usual guidelines detailed in this class's create
	 *            functions.
	 * @throws InvalidUserStoryInputException
	 *             One of any errors detailed in FeatureErrorType.
	 */
	public static void updateOwner(Feature feature, User owner)
			throws InvalidFeatureInputException {
		if (feature == null)
			throw new InvalidFeatureInputException(FeatureErrorType.NULL);
		checkOwner(owner, feature.getProject());
		feature.setOwner(owner);
		feature.save();
		manager.fireChange(new EntityEvent(feature,
				EntityEvent.EventType.UPDATE));
	}

	/**
	 * Updates the given feature's release.
	 * 
	 * @param feature
	 *            The feature can be any non-null feature.
	 * @param release
	 *            Can be null. Must be in this feature's project.
	 * @throws InvalidFeatureInputException
	 *             One of any errors detailed in FeatureErrorType.
	 */
	public static void updateRelease(Feature feature, Release release)
			throws InvalidFeatureInputException {
		if (feature == null)
			throw new InvalidFeatureInputException(FeatureErrorType.NULL);
		checkRelease(release, feature.getProject());
		feature.setRelease(release);
		feature.save();
		manager.fireChange(new EntityEvent(feature,
				EntityEvent.EventType.UPDATE));
	}

	/**
	 * Moves the feature to a different project. As a result of this, the
	 * feature will have it's owner and author set to null.
	 * 
	 * @param feature
	 *            Works on any non-null user story.
	 * @param project
	 *            The new project (non-null or an exception is thrown)
	 * @throws InvalidUserStoryInputException
	 *             On any invalid input, detailed in this exceptinon's
	 *             getErrorType()
	 * @throws InvalidFeatureInputException
	 *             One of any errors detailed in FeatureErrorType.
	 */
	public static void updateProject(Feature feature, Project project)
			throws InvalidFeatureInputException {
		if (feature == null)
			throw new InvalidFeatureInputException(FeatureErrorType.NULL);
		checkProject(project);
		feature.setAuthor(null);
		feature.setOwner(null);
		feature.setProject(project);
		feature.save();
		manager.fireChange(new EntityEvent(feature,
				EntityEvent.EventType.UPDATE));
	}

	/**
	 * Updates all fields with the provided values.
	 * 
	 * @param title
	 *            Must not begin with a space, contain only spaces, be empty,
	 *            null or duplicate of any other feature's title in this project
	 * @param description
	 *            Any non-null or empty string.
	 * @param project
	 *            Must not be null.
	 * @param release
	 *            Is allowed to be null. Must be a release in the feature's
	 *            project.
	 * @param author
	 *            Is allowed to be null. Must be a user of this feature's
	 *            project.
	 * @param owner
	 *            Is allowed to be null. Must be a user of this feature's
	 *            project.
	 * @return A reference to the newly created Feature.
	 * @throws InvalidFeatureInputException
	 *             One of any errors detailed in FeatureErrorType.
	 */
	public static void update(Feature feature, String title,
			String description, User author, User owner, Release release,
			Project project) throws InvalidFeatureInputException {
		if (feature == null)
			throw new InvalidFeatureInputException(FeatureErrorType.NULL);
		checkProject(project);
		checkRelease(release, project);
		checkAuthor(author, project);
		checkOwner(owner, project);
		checkDescription(description);
		checkTitle(title, project, feature.getID());
		feature.setProject(project);
		feature.setTitle(title);
		feature.setDescription(description);
		feature.setAuthor(author);
		feature.setOwner(owner);
		feature.setRelease(release);
		feature.save();
		manager.fireChange(new EntityEvent(feature,
				EntityEvent.EventType.UPDATE));
	}

	public static void delete(Feature feature)
			throws InvalidFeatureInputException {
		if (feature == null)
			throw new InvalidFeatureInputException(FeatureErrorType.NULL);

		try {
			TraceabilityAssociation[] associations = Database.manager().find(TraceabilityAssociation.class, Query.select().where("from = ? OR to = ?", feature, feature));
			for(TraceabilityAssociation t : associations) {
				Database.manager().delete(t);
			}
		} catch (SQLException e) {
			throw new InvalidFeatureInputException(
					FeatureErrorType.DATABASE_FEATURE_DELETION, e
							.getMessage());
		}
		
		try {
			Database.manager().delete(feature);
		} catch (SQLException e) {
			throw new InvalidFeatureInputException(
					FeatureErrorType.DATABASE_FEATURE_DELETION, e.getMessage());
		}
		manager.fireChange(new EntityEvent(feature,
				EntityEvent.EventType.DELETE));
	}

	// Public Validation functions

	/**
	 * Validates the title as if updateIteration had been called.
	 * 
	 * @param accessor
	 *            getTitle(), getProject(), and getID() must all give valid
	 *            results.
	 * @throws InvalidFeatureInputException
	 *             One of any error detailed in FeatureErrorType.
	 */
	public static void validateTitle(FeatureAccessor accessor)
			throws InvalidFeatureInputException {
		if (accessor != null)
			checkTitle(accessor.getTitle(), accessor.getProject(), accessor
					.getID());
		else
			throw new InvalidFeatureInputException(
					"UserStoryAccessor accessor is null.");
	}

	/**
	 * Validates the title as if create had been called.
	 * 
	 * @param accessor
	 *            getTitle() and getProject() must both give valid results.
	 * @throws InvalidFeatureInputException
	 *             One of any error detailed in FeatureErrorType.
	 */
	public static void validateNewTitle(FeatureAccessor accessor)
			throws InvalidFeatureInputException {
		if (accessor != null)
			checkTitle(accessor.getTitle(), accessor.getProject());
		else
			throw new InvalidFeatureInputException(
					"UserStoryAccessor accessor is null.");
	}

	/**
	 * Validates the description as if updateDescription or create had been
	 * called.
	 * 
	 * @param accessor
	 *            getDescription() must give a valid result.
	 * @throws InvalidFeatureInputException
	 *             One of any error detailed in FeatureErrorType.
	 */
	public static void validateDescription(FeatureAccessor accessor)
			throws InvalidFeatureInputException {
		if (accessor != null)
			checkDescription(accessor.getDescription());
		else
			throw new InvalidFeatureInputException(
					"UserStoryAccessor accessor is null.");
	}

	/**
	 * Validates the author as if updateAuthor or create had been called.
	 * 
	 * @param accessor
	 *            getAuthor() and getProject() must both give valid results.
	 * @throws InvalidFeatureInputException
	 *             One of any error detailed in FeatureErrorType.
	 */
	public static void validateAuthor(FeatureAccessor accessor)
			throws InvalidFeatureInputException {
		if (accessor != null)
			checkAuthor(accessor.getAuthor(), accessor.getProject());
		else
			throw new InvalidFeatureInputException(
					"UserStoryAccessor accessor is null.");
	}

	/**
	 * Validates the owner as if updateOwner or create had been called.
	 * 
	 * @param accessor
	 *            getOwner() and getProject() must both give valid results.
	 * @throws InvalidFeatureInputException
	 *             One of any error detailed in FeatureErrorType.
	 */
	public static void validateOwner(FeatureAccessor accessor)
			throws InvalidFeatureInputException {
		if (accessor != null)
			checkOwner(accessor.getOwner(), accessor.getProject());
		else
			throw new InvalidFeatureInputException(
					"UserStoryAccessor accessor is null.");
	}

	/**
	 * Validates the release as if updateRelease or create had been called.
	 * 
	 * @param accessor
	 *            getRelease() and getProject() must both give valid results.
	 * @throws InvalidFeatureInputException
	 *             One of any error detailed in FeatureErrorType.
	 */
	public static void validateRelease(FeatureAccessor accessor)
			throws InvalidFeatureInputException {
		if (accessor != null)
			checkRelease(accessor.getRelease(), accessor.getProject());
		else
			throw new InvalidFeatureInputException(
					"UserStoryAccessor accessor is null.");
	}

	/**
	 * Validates the project as if updateProject or create had been called.
	 * 
	 * @param accessor
	 *            getProject() must give a valid result.
	 * @throws InvalidFeatureInputException
	 *             One of any error detailed in FeatureErrorType.
	 */
	public static void validateProject(FeatureAccessor accessor)
			throws InvalidFeatureInputException {
		if (accessor != null)
			checkProject(accessor.getProject());
		else
			throw new InvalidFeatureInputException(
					"UserStoryAccessor accessor is null.");
	}

	// Helper functions

	/**
	 * Validates the release. Ensures it is in the same project..
	 * 
	 * @param release
	 * @param project
	 * @throws InvalidFeatureInputException
	 *             The type of the error would be REALEASE_INVALID.
	 */
	private static void checkRelease(Release release, Project project)
			throws InvalidFeatureInputException {
		if (release != null)
			if (release.getProject() != project)
				throw new InvalidFeatureInputException(
						FeatureErrorType.RELEASE_INVALID);
	}

	private static void checkTitle(String title, Project project, int ID)
			throws InvalidFeatureInputException {

		if (title == null)
			throw new InvalidFeatureInputException(FeatureErrorType.TITLE_NULL);
		if (title.isEmpty())
			throw new InvalidFeatureInputException(FeatureErrorType.TITLE_EMPTY);
		if (title.length() > 255)
			throw new InvalidFeatureInputException(
					FeatureErrorType.TITLE_TOO_LONG);
		boolean valid = false;
		for (int i = 0; i < title.length(); i++)
			if (title.toCharArray()[i] != ' ') {
				valid = true;
				break;
			}
		if (!valid)
			throw new InvalidFeatureInputException(
					FeatureErrorType.TITLE_ALL_SPACES);
		if (title.startsWith(" "))
			throw new InvalidFeatureInputException(
					FeatureErrorType.TITLE_BEGIN_WITH_SPACE);

		if (project != null) {
			UserStory[] stories = project.getUserStories();
			for (UserStory story : stories)
				if (ID != story.getID())
					if (story.getTitle() == title)
						throw new InvalidFeatureInputException(
								FeatureErrorType.TITLE_NOT_UNIQUE);
		}
	}

	private static void checkTitle(String title, Project project)
			throws InvalidFeatureInputException {

		if (title == null)
			throw new InvalidFeatureInputException(FeatureErrorType.TITLE_NULL);
		if (title.isEmpty())
			throw new InvalidFeatureInputException(FeatureErrorType.TITLE_EMPTY);
		if (title.length() > 255)
			throw new InvalidFeatureInputException(
					FeatureErrorType.TITLE_TOO_LONG);
		boolean valid = false;
		for (int i = 0; i < title.length(); i++)
			if (title.toCharArray()[i] != ' ') {
				valid = true;
				break;
			}
		if (!valid)
			throw new InvalidFeatureInputException(
					FeatureErrorType.TITLE_ALL_SPACES);
		if (title.startsWith(" "))
			throw new InvalidFeatureInputException(
					FeatureErrorType.TITLE_BEGIN_WITH_SPACE);

		if (project != null) {
			UserStory[] stories = project.getUserStories();
			for (UserStory story : stories)
				if (story.getTitle() == title)
					throw new InvalidFeatureInputException(
							FeatureErrorType.TITLE_NOT_UNIQUE);
		}
	}

	private static void checkDescription(String description)
			throws InvalidFeatureInputException {

		if (description == null)
			throw new InvalidFeatureInputException(
					FeatureErrorType.DESCRIPTION_NULL);
		if (description.isEmpty())
			throw new InvalidFeatureInputException(
					FeatureErrorType.DESCRIPTION_EMPTY);
		if (description.length() > 255)
			throw new InvalidFeatureInputException(
					FeatureErrorType.DESCRIPTION_TOO_LONG);
		boolean valid = false;
		for (int i = 0; i < description.length(); i++)
			if (description.toCharArray()[i] != ' ') {
				valid = true;
				break;
			}
		if (!valid)
			throw new InvalidFeatureInputException(
					FeatureErrorType.DESCRIPTION_ALL_SPACES);
	}

	private static void checkProject(Project project)
			throws InvalidFeatureInputException {
		if (project == null)
			throw new InvalidFeatureInputException(FeatureErrorType.NO_PROJECT);
	}

	private static void checkAuthor(User author, Project project)
			throws InvalidFeatureInputException {
		if (author == null)
			return;
		boolean valid = false;
		for (User u : project.getMembers())
			if (u.equals(author)) {
				valid = true;
				break;
			}
		if (!valid)
			throw new InvalidFeatureInputException(
					FeatureErrorType.AUTHOR_INVALID);
	}

	private static void checkOwner(User owner, Project project)
			throws InvalidFeatureInputException {
		if (owner == null)
			return;
		boolean valid = false;
		for (User u : project.getMembers())
			if (u.equals(owner)) {
				valid = true;
				break;
			}
		if (!valid)
			throw new InvalidFeatureInputException(
					FeatureErrorType.OWNER_INVALID);
	}

}
