/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Ian Williams - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.data.funcs;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import edu.wpi.wpisuite.data.Iteration;
import edu.wpi.wpisuite.data.Project;
import edu.wpi.wpisuite.data.Release;
import edu.wpi.wpisuite.data.UserStory;
import edu.wpi.wpisuite.data.errors.InvalidIterationInputException;
import edu.wpi.wpisuite.data.errors.IterationErrorType;
import edu.wpi.wpisuite.data.util.IterationAccessor;
import edu.wpi.wpisuite.data.util.IterationComparator;
import edu.wpi.wpisuite.database.Database;

// TODO: Better the documentation to match UserStoryManager and FeatureManager
// TODO: test TOTAL rubbish dates
// TODO: hard coded 255 for max stringlength.  Have it look at active objects to get that value. do this for all managers.
/**
 * Class of CRUD functions for Iterations
 */
public final class IterationManager extends EntityManager {

	// Singleton, used for managing listeners
	private IterationManager() {
		super();
	}

	private static IterationManager manager = new IterationManager();

	// Public CRUD functions

	/**
	 * Creates and returns a new iteration. Fills its data using the specified
	 * parameters. Error checking is done inside this function.
	 * 
	 * @param name
	 *            Name for this iteration.
	 * @param project
	 *            Project that this iteration is under.
	 * @param release
	 *            Release that this Iteration belongs to.
	 * @param startDate
	 *            The starting date of this Iteration. May be null.
	 * @param endDate
	 *            The ending date of this Iteration. May be null. If not null,
	 *            must be after the start date.
	 * @return The newly created Iteration.
	 * @throws InvalidIterationInputException
	 *             On any error. Exception message will detail the problem.
	 */
	public static Iteration create(String name, Project project,
			Release release, Date startDate, Date endDate)
			throws InvalidIterationInputException {

		Iteration iteration;

		// Make sure the project exists
		if (project == null)
			throw new InvalidIterationInputException(
					IterationErrorType.NO_PROJECT);

		// Make sure our dates are valid. null is okay
		checkDates(startDate, endDate, project);

		// Make sure name is okay
		checkName(name, project);

		// Check that release is in this project in the dates are okay
		checkRelease(release, startDate, endDate, project);

		try {
			iteration = Database.manager().create(Iteration.class);
		} catch (SQLException e) {
			throw new InvalidIterationInputException(
					IterationErrorType.DATABASE_ITERATION_CREATION, e
							.getMessage());
		}

		iteration.setRelease(release);
		iteration.setName(name);
		iteration.setEndDate(endDate);
		iteration.setStartDate(startDate);
		iteration.setProject(project);
		iteration.save();

		manager.fireChange(new EntityEvent(iteration,
				EntityEvent.EventType.CREATE));

		return iteration;
	}

	/**
	 * Creates and returns a new iteration. Fills its data using the specified
	 * parameters. Error checking is done inside this function.
	 * 
	 * @param name
	 *            Name for this iteration
	 * @return The newly created Iteration.
	 * @throws InvalidIterationInputException
	 *             On any error
	 */
	public static Iteration create(String name, Project project)
			throws InvalidIterationInputException {

		Iteration iteration;

		try {
			iteration = Database.manager().create(Iteration.class);
		} catch (SQLException e) {
			throw new InvalidIterationInputException(
					IterationErrorType.DATABASE_ITERATION_CREATION, e
							.getMessage());

		}

		if (project == null)
			throw new InvalidIterationInputException(
					IterationErrorType.NO_PROJECT);
		iteration.setProject(project);

		// Make sure name is okay
		checkName(name, project, iteration.getID());
		iteration.setName(name);

		iteration.save();

		manager.fireChange(new EntityEvent(iteration,
				EntityEvent.EventType.CREATE));

		return iteration;
	}

	/**
	 * Updates the given iteration.
	 * 
	 * @param iteration
	 *            The iteration to be updated.
	 * @param name
	 *            The new name for the iteration
	 * @throws InvalidIterationInputException
	 *             Thrown if the name is invalid for one of many reasons,
	 *             specified in the message text.
	 */
	public static void updateName(Iteration iteration, String name)
			throws InvalidIterationInputException {
		if (iteration == null)
			throw new InvalidIterationInputException(IterationErrorType.NULL);
		checkName(name, iteration.getProject(), iteration.getID());
		iteration.setName(name);
		iteration.save();
		manager.fireChange(new EntityEvent(iteration,
				EntityEvent.EventType.UPDATE));
	}

	/**
	 * Updates the given iteration.
	 * 
	 * @param iteration
	 *            The iteration to be updated.
	 * @param startDate
	 *            The new startDate for the iteration
	 * @param endDate
	 *            The new endDate for the iteration
	 * @throws InvalidIterationInputException
	 *             Thrown if the dates are invalid for one of many reasons,
	 *             specified in the message text.
	 */
	public static void updateDates(Iteration iteration, Date startDate,
			Date endDate) throws InvalidIterationInputException {
		if (iteration == null)
			throw new InvalidIterationInputException(IterationErrorType.NULL);
		checkDates(startDate, endDate, iteration.getProject(), iteration
				.getID());
		checkRelease(iteration.getRelease(), startDate, endDate, iteration
				.getProject());
		iteration.setStartDate(startDate);
		iteration.setEndDate(endDate);
		iteration.save();
		manager.fireChange(new EntityEvent(iteration,
				EntityEvent.EventType.UPDATE));
	}

	/**
	 * Updates the given iteration.
	 * 
	 * @param iteration
	 *            The iteration to be updated.
	 * @param release
	 *            The new release this iteration will be part of
	 * @throws InvalidIterationInputException
	 *             Thrown if the name is invalid for one of many reasons,
	 *             specified in the message text.
	 */
	public static void updateRelease(Iteration iteration, Release release)
			throws InvalidIterationInputException {
		if (iteration == null)
			throw new InvalidIterationInputException(IterationErrorType.NULL);
		checkRelease(release, iteration.getStartDate(), iteration.getEndDate(),
				iteration.getProject());
		iteration.setRelease(release);
		iteration.save();
		manager.fireChange(new EntityEvent(iteration,
				EntityEvent.EventType.UPDATE));
	}

	/**
	 * Updates the given iteration.
	 * 
	 * @param iteration
	 *            The iteration to be updated.
	 * @param name
	 *            The new name for the iteration
	 * @param startDate
	 *            The new startDate for the iteration
	 * @param endDate
	 *            The new endDate for the iteration
	 * @param release
	 *            The new release this iteration will be part of
	 * @throws InvalidIterationInputException
	 *             Thrown if the release is invalid for one of many reasons,
	 *             specified in the message text.
	 */
	public static void update(Iteration iteration, String name, Date startDate,
			Date endDate, Release release)
			throws InvalidIterationInputException {
		if (iteration == null)
			throw new InvalidIterationInputException(IterationErrorType.NULL);
		checkName(name, iteration.getProject(), iteration.getID());
		checkDates(startDate, endDate, iteration.getProject(), iteration
				.getID());
		checkRelease(release, iteration.getStartDate(), iteration.getEndDate(),
				iteration.getProject());
		iteration.setName(name);
		iteration.setStartDate(startDate);
		iteration.setEndDate(endDate);
		iteration.setRelease(release);
		iteration.save();
		manager.fireChange(new EntityEvent(iteration,
				EntityEvent.EventType.UPDATE));
	}

	/**
	 * Deletes the given iteration. If deleteuserStories is set, it will delete
	 * all associated user stories too. Otherwise it will remove the associated
	 * user stories relationship but not destroy them.
	 * 
	 * @param iteration
	 *            The iteration to delete.
	 * @param deleteUserStories
	 *            Whether to delete or keep the associated user stories.
	 * @throws InvalidIterationInputException
	 *             If one of many errors occurs.
	 */
	public static void delete(Iteration iteration, boolean deleteUserStories)
			throws InvalidIterationInputException {
		if (iteration == null)
			throw new InvalidIterationInputException(IterationErrorType.NULL);

		UserStory[] stories = iteration.getUserStories();
		if (deleteUserStories)
			for (UserStory us : stories)
				try {
					Database.manager().delete(us);
				} catch (SQLException e) {
					throw new InvalidIterationInputException(
							IterationErrorType.DATABASE_USERSTORY_DELETION, e
									.getMessage());
				}
		else
			for (UserStory us : stories) {
				us.setIteration(null);
				us.save();
			}

		try {
			Database.manager().delete(iteration);
		} catch (SQLException e) {
			throw new InvalidIterationInputException(
					IterationErrorType.DATABASE_ITERATION_DELETION, e
							.getMessage());
		}
		manager.fireChange(new EntityEvent(iteration,
				EntityEvent.EventType.DELETE));
	}

	// Public validation functions

	/**
	 * Validates the name obtained from i.getName(). This function expects
	 * i.getID() to return a valid iteration ID, and will thrown an exception
	 * otherwise. To validate the new of a not yet created iteration, use
	 * validateNewName().
	 * 
	 * @param i
	 *            This function will behave fine for any non-null i..
	 * @throws InvalidIterationInputException
	 *             If i is null.
	 */
	public static void validateName(IterationAccessor i)
			throws InvalidIterationInputException {
		if (i != null)
			checkName(i.getName(), i.getProject(), i.getID());
		else
			throw new InvalidIterationInputException(
					"IterationAccessor i is null.");
	}

	/**
	 * Validates the name obtained from i.getName(). This function will not use
	 * i.getID(). This function is intended to validate the name on a not yet
	 * created iteration only.
	 * 
	 * @param i
	 *            This function will behave fine for any non-null i..
	 * @throws InvalidIterationInputException
	 * @throws InvalidIterationInputException
	 *             If i is null.
	 */
	public static void validateNewName(IterationAccessor i)
			throws InvalidIterationInputException {
		if (i != null)
			checkName(i.getName(), i.getProject());
		else
			throw new InvalidIterationInputException(
					"IterationAccessor i is null.");
	}

	/**
	 * Validates the start and end dates obtained through i.getStartDate() and
	 * i.getEndDate()
	 * 
	 * @param i
	 *            This function will behave fine for any non-null i.
	 * @throws InvalidIterationInputException
	 *             If I is null.
	 */
	public static void validateDates(IterationAccessor i)
			throws InvalidIterationInputException {
		if (i != null)
			checkDates(i.getStartDate(), i.getEndDate(), i.getProject(), i
					.getID());
		else
			throw new InvalidIterationInputException(
					"IterationAccessor i is null.");
	}

	/**
	 * Validates the release of obtained through i.getRelease()
	 * 
	 * @param i
	 *            This function will behave fine for any non-null i.
	 * @throws InvalidIterationInputException
	 *             If i is null
	 */
	public static void validateRelease(IterationAccessor i)
			throws InvalidIterationInputException {
		if (i != null)
			checkRelease(i.getRelease(), i.getStartDate(), i.getEndDate(), i
					.getProject());
		else
			throw new InvalidIterationInputException(
					"IterationAccessor i is null.");
	}

	// Helper functions

	/**
	 * Validates the name of an iteration. Ensures the name of the iteration is
	 * not empty, and contains more than just spaces.
	 * 
	 * @param name
	 *            The name (String) to validate
	 * @param project
	 *            The project the iteration whose name is being validated
	 *            belongs to
	 * @param ID
	 *            The ID of the iteration whose name is being validated
	 */
	private static void checkName(String name, Project project, int ID)
			throws InvalidIterationInputException {
		if (name == null)
			throw new InvalidIterationInputException(
					IterationErrorType.NAME_NULL);
		if (name.isEmpty())
			throw new InvalidIterationInputException(
					IterationErrorType.NAME_EMPTY);
		if (name.length() > 255)
			throw new InvalidIterationInputException(
					IterationErrorType.NAME_TOO_LONG);
		boolean valid = false;
		for (int i = 0; i < name.length(); i++)
			if (name.toCharArray()[i] != ' ') {
				valid = true;
				break;
			}
		if (!valid)
			throw new InvalidIterationInputException(
					IterationErrorType.NAME_ALL_SPACES);
		if (name.startsWith(" "))
			throw new InvalidIterationInputException(
					IterationErrorType.NAME_BEGIN_WITH_SPACE);

		if (project != null) {
			Iteration[] iterations = project.getIterations();
			for (Iteration i : iterations)
				if (ID != i.getID())
					if (i.getName().equals(name))
						throw new InvalidIterationInputException(
								IterationErrorType.NAME_NOT_UNIQUE);
		}
	}

	/**
	 * Validates the name of an iteration. Ensures the name of the iteration is
	 * not empty, and contains more than just spaces.
	 * 
	 * @param name
	 *            The name (String) to validate
	 * @param project
	 *            The project the iteration whose name is being validated
	 *            belongs to
	 */
	private static void checkName(String name, Project project)
			throws InvalidIterationInputException {
		if (name == null)
			throw new InvalidIterationInputException(
					IterationErrorType.NAME_NULL);
		if (name.isEmpty())
			throw new InvalidIterationInputException(
					IterationErrorType.NAME_EMPTY);
		if (name.length() > 255)
			throw new InvalidIterationInputException(
					IterationErrorType.NAME_TOO_LONG);
		boolean valid = false;
		for (int i = 0; i < name.length(); i++)
			if (name.toCharArray()[i] != ' ') {
				valid = true;
				break;
			}
		if (!valid)
			throw new InvalidIterationInputException(
					IterationErrorType.NAME_ALL_SPACES);
		if (name.startsWith(" "))
			throw new InvalidIterationInputException(
					IterationErrorType.NAME_BEGIN_WITH_SPACE);

		if (project != null) {
			Iteration[] iterations = project.getIterations();
			for (Iteration i : iterations)
				if (i.getName().equals(name))
					throw new InvalidIterationInputException(
							IterationErrorType.NAME_NOT_UNIQUE);
		}
	}

	/**
	 * Validates the dates of an iteration. Ensures that the iteration begins
	 * before it ends. Also ensures there is no iteration overlap.
	 * 
	 * @param startDate
	 *            New startDate to check. May be null. Must be before the new
	 *            end date.
	 * @param endDate
	 *            New endDate to check. May be null. Must be after the new
	 *            endDate.
	 * @param project
	 *            The project this iteration these dates are for belongs to. If
	 *            project is null, this function can only validate that
	 *            stateDate and endDate are in order. It cannot check for
	 *            overlap with other iterations.
	 * @param ID
	 *            The ID of the iteration these dates apply to.
	 * @throws InvalidIterationInputException
	 */
	private static void checkDates(Date startDate, Date endDate,
			Project project, int ID) throws InvalidIterationInputException {
		if (endDate == null || startDate == null)
			return;
		if (!endDate.after(startDate))
			throw new InvalidIterationInputException(
					IterationErrorType.DATE_INVALID);
		if (project != null) {
			Iteration[] iterations = project.getIterations();
			for (Iteration i : iterations)
				if (i.getID() != ID)
					if (overlap(startDate, endDate, i.getStartDate(), i
							.getEndDate()))
						throw new InvalidIterationInputException(
								IterationErrorType.DATE_OVERLAP,
								"Overlaps with '" + i.getName() + "'.");
		}
	}

	/**
	 * Validates the dates of an iteration. Ensures that the iteration begins
	 * before it ends. Also ensures there is no iteration overlap. This version
	 * of checkDate doesn't have the ID of the iteration in question as a
	 * parameter, and therefore, where checking for overlaps it has no way of
	 * ensuring the current iteration doesn't overlap with "iself" This is best
	 * used only before creations, and the other checkDates should be used on
	 * all updates.
	 * 
	 * @param startDate
	 *            New startDate to check. May be null. Must be before the new
	 *            end date.
	 * @param endDate
	 *            New endDate to check. May be null. Must be after the new
	 *            endDate.
	 * @param project
	 *            The project this iteration these dates are for belongs to. If
	 *            project is null, this function can only validate that
	 *            stateDate and endDate are in order. It cannot check for
	 *            overlap with other iterations. s * @throws
	 *            InvalidIterationInputException
	 */
	private static void checkDates(Date startDate, Date endDate, Project project)
			throws InvalidIterationInputException {
		if (endDate == null || startDate == null)
			return;
		if (!endDate.after(startDate))
			throw new InvalidIterationInputException(
					IterationErrorType.DATE_INVALID);
		if (project != null) {
			Iteration[] iterations = project.getIterations();
			for (Iteration i : iterations)
				if (overlap(startDate, endDate, i.getStartDate(), i
						.getEndDate()))
					throw new InvalidIterationInputException(
							IterationErrorType.DATE_OVERLAP, "Overlaps with '"
									+ i.getName() + "'.");
		}
	}

	/**
	 * Validates the release. Checks its estimated date with the iterations
	 * dates, and ensures it is in the same project as the iteration.
	 * 
	 * @param release
	 * @param startDate
	 * @param endDate
	 * @param project
	 * @throws InvalidIterationInputException
	 */
	private static void checkRelease(Release release, Date startDate,
			Date endDate, Project project)
			throws InvalidIterationInputException {
		if (release != null) {
			if (release.getProject() != project)
				throw new InvalidIterationInputException(
						IterationErrorType.RELEASE_INVALID);
			if (startDate != null)
				if (release.getExpectedDate().before(startDate))
					throw new InvalidIterationInputException(
							IterationErrorType.RELEASE_BEFORE_ITERATION_START);
			if (endDate != null)
				if (release.getExpectedDate().before(endDate))
					throw new InvalidIterationInputException(
							IterationErrorType.RELEASE_BEFORE_ITERATION_END);
		}
	}

	/**
	 * Checks whether date range start1 to end1 overlaps with date range start2
	 * to end2
	 * 
	 * @param start1
	 * @param end1
	 * @param start2
	 * @param end2
	 * @return
	 */
	private static boolean overlap(Date start1, Date end1, Date start2,
			Date end2) {
		if (start1 != null && start2 != null && end2 != null)
			if (start1.after(start2) && start1.before(end2))
				return true;

		if (end1 != null && start2 != null && end2 != null)
			if (end1.after(start2) && end1.before(end2))
				return true;

		if (start2 != null && start1 != null && end1 != null)
			if (start2.after(start1) && start2.before(end1))
				return true;

		if (end2 != null && start1 != null && end1 != null)
			if (end2.after(start1) && end2.before(end1))
				return true;

		return false;
	}

	// Other public functions

	/**
	 * Check whether an Iteration is active
	 * 
	 * @author Nick
	 * @param it
	 *            the iteration to be checked
	 * @return true for yes.
	 */
	public static boolean isActive(Iteration it) {
		if (it.getStartDate() != null && it.getEndDate() != null)
			return it.getStartDate().getTime() <= System.currentTimeMillis()
					&& System.currentTimeMillis() <= it.getEndDate().getTime();
		return false;
	}

	/**
	 * return the estimate units of an iterations
	 * 
	 * @author Nick
	 * @param it
	 *            an iteration
	 * @return the sum of the estimate units of the array of iterations
	 * @throws InvalidIterationInputException
	 *             If the iteration is null.
	 */
	public static int countEstimateUnit(Iteration it)
			throws InvalidIterationInputException {
		if (it == null)
			throw new InvalidIterationInputException(IterationErrorType.NULL);
		int sum = 0;
		UserStory[] us;
		us = it.getUserStories();
		for (int j = 0; j < us.length; j++)
			sum = sum + us[j].getEstimatedUnits();
		return sum;
	}

	/**
	 * return the actual units of an iterations
	 * 
	 * @param it
	 *            an array of iterations
	 * @return the sum of the estimate units of the array of iterations
	 * @throws InvalidIterationInputException
	 *             If the iteration is null.
	 */
	public static int countActualUnit(Iteration it)
			throws InvalidIterationInputException {
		if (it == null)
			throw new InvalidIterationInputException(IterationErrorType.NULL);
		int sum = 0;
		for (UserStory story : it.getUserStories()) {
			Integer units = story.getActualUnits();
			if (units != null)
				sum += units;
		}

		return sum;
	}

	/**
	 * Gets and sorts a list of iterations by date TODO move to project manager
	 * 
	 * @param project
	 *            The project to get iterations from
	 * @return An ordered list of iterations
	 */
	public static List<Iteration> getOrderedIterations(Project project) {
		if (project == null)
			return new ArrayList<Iteration>();
		return sort(Arrays.asList(project.getIterations()));
	}

	public static List<Iteration> sort(Collection<Iteration> iterations) {
		List<Iteration> itList = new ArrayList<Iteration>(iterations);
		Collections.sort(itList, new IterationComparator());
		return itList;
	}

	/**
	 * Returns whether the given iteration is "done", meaning that the current
	 * time is past the iteration's end time.
	 * 
	 * @param it
	 *            The iteration to check.
	 * @return
	 */
	public static boolean isDone(Iteration it) {
		if (it.getEndDate() != null)
			return System.currentTimeMillis() > it.getEndDate().getTime();
		return false;
	}
}
