/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Ian Williams - Initial design, API, and implementation
 *    Karl Wajcs - Initial design, API, and implementation
 *    Dan Bjorge - Additional implementation (password validation)
 *******************************************************************************/

package edu.wpi.wpisuite.data.funcs;

import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.sql.SQLException;

import edu.wpi.wpisuite.data.Config;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.database.Database;

/**
 * Class of functions having to do with logging in.
 */
public class LoginFunctions {

	private static String failedLoginUsername;
	private static int consecutiveFailedLogins = 0;

	/**
	 * Attempts to login a user.
	 * 
	 * @param userName
	 * @param password
	 * @return The User object of the person who just logged in, or null if the
	 *         password was incorrect.
	 * @throws IOException
	 *             If the username is invalid, this functions throws a generic
	 *             IOExcpetion.
	 * @throws NoSuchAlgorithmException
	 *             If Java doesn't have the neccesary function. Should never
	 *             happen.
	 */
	public static User Login(String username, String password)
			throws IOException {
		User temp;
		try {
			temp = Database.manager()
					.find(User.class, "username = ?", username)[0];
		} catch (Exception e) {
			throw new IOException("User '" + username + "' not found.");
		}

		if (temp.isLocked())
			throw new IOException(
					"This user has been locked due to excessive failed login attempts. Contact your administrator.");

		// Everything is OK
		if (temp.getPasswordHash().equals(Database.hashPassword(password))) {
			LoginFunctions.consecutiveFailedLogins = 0;
			return temp;
		}
		// Some sort of problem - figure it out and return it in the form of an
		// IOException
		else {
			int maxTries = LoginFunctions.getMaxFailedLogins();
			int remainingTries = maxTries
					- LoginFunctions.consecutiveFailedLogins;

			if (LoginFunctions.failedLoginUsername != null
					&& LoginFunctions.failedLoginUsername.equals(username)) {
				LoginFunctions.failedLoginUsername = username;
				LoginFunctions.consecutiveFailedLogins++;
				remainingTries = maxTries
						- LoginFunctions.consecutiveFailedLogins;
				if (maxTries > 0 && remainingTries <= 0) {
					temp.setLocked(true);
					temp.save();
					try {
						UserManager.notifyAccountLockOut(username);
					} catch (Exception e) {
						throw new IOException(
								"Invalid password. This user has been locked due to excessive failed login attempts. Contact your administrator. Additionally, an error occured while emailing administrators about this issue.");
					}
					throw new IOException(
							"Invalid password. This user has been locked due to excessive failed login attempts. Contact your administrator.");
				}
			} else {
				LoginFunctions.failedLoginUsername = username;
				LoginFunctions.consecutiveFailedLogins = 1;
			}
			String invalidString = "Invalid Password.";
			if (maxTries > 0)
				invalidString += " (" + remainingTries + " attempt"
						+ (remainingTries == 1 ? "" : "s") + " remaining)";
			throw new IOException(invalidString);
		}
	}

	public static int getMaxFailedLogins() {
		String configName = "maxFailedLoginAttempts";
		int defaultConfigValue = 3;

		Config[] configs = {};
		try {
			configs = Database.manager().find(Config.class, "name = ?",
					configName);
		} catch (Exception e) {
			return defaultConfigValue;
		}

		Config config;
		if (configs.length == 0)
			try {
				config = Database.manager().create(Config.class);
				config.setName(configName);
				config.setValue(defaultConfigValue + "");
				config.save();
			} catch (Exception e) {
				return defaultConfigValue;
			}
		else
			config = configs[0];

		int value = Integer.parseInt(config.getValue());
		return value;
	}

	public static void setMaxFailedLogins(int max) {
		String configName = "maxFailedLoginAttempts";

		Config config = null;
		try {
			Config[] configs = Database.manager().find(Config.class,
					"name = ?", configName);
			if (configs.length > 0)
				config = configs[0];
			else
				throw new Exception("Config not found");
		} catch (Exception e) {
			try {
				config = Database.manager().create(Config.class);
			} catch (SQLException e2) {
				return;
			}
			config.setName(configName);
		}

		config.setValue(max + "");

		config.save();
	}

	public static void changePassword(User currentUser, String oldPass,
			String newPass, String verificationPass) throws Exception {
		if (!currentUser.getPasswordHash().equals(
				Database.hashPassword(oldPass)))
			throw new Exception("Incorrect old password");
		if (!newPass.equals(verificationPass))
			throw new Exception("New password must match verification password");
		if (newPass.equals(oldPass))
			throw new Exception(
					"New password cannot be the same as the old password");
		// This throws exceptions if failures happen
		LoginFunctions.validatePassword(newPass);

		// The password is good. Now make the change.
		currentUser.setPasswordHash(Database.hashPassword(newPass));
		currentUser.setPasswordExpired(false);
		currentUser.save();
	}

	public static void validatePassword(String password) throws Exception {
		if (password.length() < 6)
			throw new Exception("Passwords must be at least 6 characters long");
		if (!password.matches(".*[A-Z,a-z].*$"))
			throw new Exception("Passwords must contain at least one letter");
		if (!password.matches(".*[0-9].*$"))
			throw new Exception("Passwords must contain at least one number");
	}
}
