/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Ravi Singh - API, and implementation
 *    Ian Williams - Design, some implementation
 *******************************************************************************/

package edu.wpi.wpisuite.data.funcs;

import java.sql.SQLException;
import java.util.ArrayList;

import edu.wpi.wpisuite.data.Iteration;
import edu.wpi.wpisuite.data.Project;
import edu.wpi.wpisuite.data.ProjectMember;
import edu.wpi.wpisuite.data.TraceabilityMatrix;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.data.UserStory;
import edu.wpi.wpisuite.data.errors.InvalidProjectInputException;
import edu.wpi.wpisuite.data.errors.ProjectErrorType;
import edu.wpi.wpisuite.data.util.ProjectAccessor;
import edu.wpi.wpisuite.database.Database;


/**
 * Class of CRUD functions for Iterations
 */
public final class ProjectManager extends EntityManager {

	// Singleton, used for managing listeners
	private ProjectManager() {
		super();
	}

	private static ProjectManager manager = new ProjectManager();

	// Public CRUD functions

	/**
	 * Creates and returns a new project. Fills its data using the specified
	 * parameters. Error checking is done inside this function.
	 * 
	 * @param name
	 *            Name for this project.
	 * @param description
	 *            Description of the project.
	 * @return The newly created Project.
	 * @throws InvalidProjectInputException
	 *             On any error. Exception message will detail the problem.
	 */
	public static Project create(String name, String description)
			throws InvalidProjectInputException {

		Project project;

		// Make sure name and description are valid.
		checkName(name);
		checkDescription(description);

		try {
			project = Database.manager().create(Project.class);
		} catch (SQLException e) {
			throw new InvalidProjectInputException(
					ProjectErrorType.DATABASE_PROJECT_CREATION, e.getMessage());
		}

		project.setName(name);
		project.setDescription(description);

		project.save();

		manager.fireChange(new EntityEvent(project,
				EntityEvent.EventType.CREATE));

		return project;
	}

	/**
	 * Updates the given project.
	 * 
	 * @param project
	 *            The project to be updated.
	 * @param name
	 *            The new name for the project
	 * @throws InvalidProjectInputException
	 *             Thrown if the name is invalid for one of many reasons,
	 *             specified in the message text.
	 */
	public static void updateName(Project project, String name)
			throws InvalidProjectInputException {

		if (project == null)
			throw new InvalidProjectInputException(ProjectErrorType.NULL);
		checkName(name);
		project.setName(name);
		project.save();
		manager.fireChange(new EntityEvent(project,
				EntityEvent.EventType.UPDATE));
	}

	/**
	 * Updates the given project.
	 * 
	 * @param project
	 *            The project to be updated.
	 * @throws InvalidProjectInputException
	 *             Thrown if the description is invalid for one of many reasons,
	 *             specified in the message text.
	 */
	public static void updateDescription(String description, Project project)
			throws InvalidProjectInputException {
		if (project == null)
			throw new InvalidProjectInputException(ProjectErrorType.NULL);
		checkDescription(description);
		project.setDescription(description);
		project.save();
		manager.fireChange(new EntityEvent(project,
				EntityEvent.EventType.UPDATE));
	}

	/**
	 * Updates the given project.
	 * 
	 * @param project
	 *            The project to be updated.
	 * @param name
	 *            The new name for the project.
	 * @param description
	 *            The new description for the project.
	 * @throws InvalidProjectInputException
	 *             Thrown if the project is invalid for one of many reasons,
	 *             specified in the message text.
	 */
	public static void update(Project project, String name, String description)
			throws InvalidProjectInputException {

		if (project == null)
			throw new InvalidProjectInputException(ProjectErrorType.NULL);

		checkName(name);
		checkDescription(description);
		project.setName(name);
		project.setDescription(description);
		project.save();
		manager.fireChange(new EntityEvent(project,
				EntityEvent.EventType.UPDATE));
	}

	/**
	 * Deletes the given project. Also deletes all user stories, iterations
	 * releases, features, traceability matricies associated with the project.
	 * 
	 * @param project
	 *            The project to delete.
	 * @throws InvalidProjectInputException
	 *             If one of many errors occurs.
	 */
	public static void delete(Project project)
			throws InvalidProjectInputException {

		if (project == null)
			throw new InvalidProjectInputException(ProjectErrorType.NULL);

		try {
			ProjectMember[] projectMemberAssocs = Database.manager().find(
					ProjectMember.class, "projectID = ?", project);
			Database.manager().delete(projectMemberAssocs);

			for (TraceabilityMatrix t : project.getTraceabilityMatrices()) {
				TraceabilityManager.deleteMatrix(t);
			}

			Database.manager().delete(project.getUserStories());
			Database.manager().delete(project.getFeatures());
			Database.manager().delete(project.getIterations());
			Database.manager().delete(project.getReleases());

			User[] usersToNull = Database.manager().find(User.class,
					"lastOpenProjectID = ?", project);
			for (User u : usersToNull) {
				u.setLastOpenProject(null);
				u.save();

			}
			Database.manager().delete(project);
		} catch (SQLException e) {
			throw new InvalidProjectInputException(
					ProjectErrorType.DATABASE_PROJECT_DELETION, e.getMessage());
		}

		manager.fireChange(new EntityEvent(project,
				EntityEvent.EventType.DELETE));
	}

	// Public validation functions

	/**
	 * Validates the name obtained from p.getName()
	 * 
	 * @param p
	 *            This function will behave fine for any non-null p..
	 * @throws InvalidProjectInputException
	 *             If p is null.
	 */
	public static void validateName(ProjectAccessor p)
			throws InvalidProjectInputException {
		if (p != null)
			checkName(p.getName());
		else
			throw new InvalidProjectInputException("ProjectAccessor is null.");
	}

	/**
	 * Validates the description obtained through p.getDescription()
	 * 
	 * @param p
	 *            This function will behave fine for any non-null p.
	 * @throws InvalidProjectInputException
	 *             If p is null.
	 */
	public static void validateDescription(ProjectAccessor p)
			throws InvalidProjectInputException {
		if (p != null)
			checkDescription(p.getDescription());
		else
			throw new InvalidProjectInputException("ProjectAccessor is null.");
	}

	// Helper functions

	/**
	 * Validates the name of a project. Ensures the name of the iteration is not
	 * empty, and contains more than just spaces.
	 * 
	 * @param name
	 *            The name (String) to validate
	 */
	private static void checkName(String name)
			throws InvalidProjectInputException {
		if (name == null)
			throw new InvalidProjectInputException(ProjectErrorType.NAME_NULL);
		if (name.isEmpty())
			throw new InvalidProjectInputException(ProjectErrorType.NAME_EMPTY);
		if (name.length() > 255)
			throw new InvalidProjectInputException(
					ProjectErrorType.NAME_TOO_LONG);
		boolean valid = false;
		for (int i = 0; i < name.length(); i++)
			if (name.toCharArray()[i] != ' ') {
				valid = true;
				break;
			}
		if (!valid)
			throw new InvalidProjectInputException(
					ProjectErrorType.NAME_ALL_SPACES);
		if (name.startsWith(" "))
			throw new InvalidProjectInputException(
					ProjectErrorType.NAME_BEGIN_WITH_SPACE);

		// check to see if name is unique
		Project[] allprojects;
		try {
			allprojects = Database.manager().find(Project.class);
		} catch (SQLException e) {
			throw new InvalidProjectInputException(
					ProjectErrorType.DATABASE_PROJECT_CREATION, e.getMessage());
		}
		if (allprojects != null)
			for (Project p : allprojects)
				if (p.getName().equals(name))
					throw new InvalidProjectInputException(
							ProjectErrorType.NAME_NOT_UNIQUE);

	}

	/**
	 * Validates the description of a project. Ensures the description of the
	 * project is not empty, and contains more than just spaces.
	 * 
	 * @param description
	 *            The description (String) to validate
	 */
	private static void checkDescription(String description)
			throws InvalidProjectInputException {
		if (description == null)
			throw new InvalidProjectInputException(
					ProjectErrorType.DESCRIPTION_NULL);
		if (description.isEmpty())
			throw new InvalidProjectInputException(
					ProjectErrorType.DESCRIPTION_EMPTY);
		if (description.length() > 255)
			throw new InvalidProjectInputException(
					ProjectErrorType.NAME_TOO_LONG);
		boolean valid = false;
		for (int i = 0; i < description.length(); i++)
			if (description.toCharArray()[i] != ' ') {
				valid = true;
				break;
			}
		if (!valid)
			throw new InvalidProjectInputException(
					ProjectErrorType.DESCRIPTION_ALL_SPACES);
	}

	/**
	 * Returns all the completed iterations in the project p. An empty array is
	 * returned if no completed iterations exist in the project.
	 * 
	 * @param p
	 *            , project
	 * @return an iterations array, containing completed iterations.
	 */
	public static Iteration[] getCompletedIterations(Project p) {

		Iteration[] allIterations = p.getIterations();
		ArrayList<Iteration> completedIterations = new ArrayList<Iteration>();

		for (Iteration i : allIterations)
			if (IterationManager.isDone(i))
				completedIterations.add(i);

		Iteration[] compIter = new Iteration[completedIterations.size()];
		int index = 0;
		for (Iteration i : completedIterations) {
			compIter[index] = i;
			index++;
		}

		return compIter;

	}

	/**
	 * This function returns all the normal iterations in the project p. Normal
	 * iterations are ones that are not completed. Hence, these are iterations
	 * in the future.
	 * 
	 * An empty array is returned if no normal iterations exist in the project.
	 * 
	 * @param p
	 *            , project
	 * @return an iteration array that contains the normal iterations.
	 */
	public static Iteration[] getNormalIterations(Project p) {
		Iteration[] allIterations = p.getIterations();
		ArrayList<Iteration> normalIterations = new ArrayList<Iteration>();

		for (Iteration i : allIterations)
			if (!IterationManager.isDone(i))
				normalIterations.add(i);

		Iteration[] normIter = new Iteration[normalIterations.size()];
		int index = 0;
		for (Iteration i : normalIterations) {
			normIter[index] = i;
			index++;
		}

		return normIter;
	}

	/**
	 * Gets the active iterations for the project p. If there are no active
	 * iterations returns null.
	 * 
	 * @param p
	 *            , project
	 * @return Iteration object, that is active.
	 */
	public static Iteration[] getActiveIterations(Project p) {
		Iteration[] allIterations = p.getIterations();
		ArrayList<Iteration> activeIterations = new ArrayList<Iteration>();

		for (Iteration i : allIterations)
			if (IterationManager.isActive(i))
				activeIterations.add(i);

		Iteration[] actIter = new Iteration[activeIterations.size()];
		int index = 0;
		for (Iteration i : activeIterations) {
			actIter[index] = i;
			index++;
		}

		return actIter;
	}

	/**
	 * Function returns an array of UserStories not assigned to any iterations.
	 * Error message if SQLException occurs
	 * 
	 * @return backlog
	 */
	public static UserStory[] getBacklog(Project p) {
		UserStory[] backlog = null;
		if (p == null)
			return new UserStory[0];
		try {
			backlog = Database.manager().find(UserStory.class,
					"iterationID IS NULL AND projectID = ?", p.getID());
		} catch (SQLException e) {
			System.err.println("backlog list could not be obtained.");
		}

		return backlog;
	}

}
