/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Ravi Singh - API, implementation
 *    Ian Williams - Design, some implmentation
 *******************************************************************************/

package edu.wpi.wpisuite.data.funcs;

import java.sql.SQLException;
import java.util.Date;

import edu.wpi.wpisuite.data.Iteration;
import edu.wpi.wpisuite.data.Project;
import edu.wpi.wpisuite.data.Release;
import edu.wpi.wpisuite.data.UserStory;
import edu.wpi.wpisuite.data.errors.InvalidReleaseInputException;
import edu.wpi.wpisuite.data.errors.ReleaseErrorType;
import edu.wpi.wpisuite.data.util.ReleaseAccessor;
import edu.wpi.wpisuite.database.Database;

/**
 * Class of CRUD functions for Iterations
 */
public final class ReleaseManager extends EntityManager {

	// Singleton, used for managing listeners
	private ReleaseManager() {
		super();
	}

	private static ReleaseManager manager = new ReleaseManager();

	// Public CRUD functions

	/**
	 * Creates and returns a new release. Fills its data using the specified
	 * parameters. Error checking is done inside this function.
	 * 
	 * @param name
	 *            Name for this release.
	 * @param expectedDate
	 *            The expected date of this Release. May not be null.
	 * @param actualDate
	 *            The actual date of this Release. May be null.
	 * @return The newly created Release.
	 * @throws InvalidReleaseInputException
	 *             On any error. Exception message will detail the problem.
	 */
	public static Release create(String name, Date expectedDate,
			Date actualDate, Project project)
			throws InvalidReleaseInputException {

		Release release;

		checkName(name, project);
		checkDates(expectedDate, actualDate);

		try {
			release = Database.manager().create(Release.class);
		} catch (SQLException e) {
			throw new InvalidReleaseInputException(
					ReleaseErrorType.DATABASE_RELEASE_CREATION, e.getMessage());
		}

		release.setActualDate(actualDate);
		release.setExpectedDate(expectedDate);
		release.setProject(project);
		release.setName(name);

		release.save();

		manager.fireChange(new EntityEvent(release,
				EntityEvent.EventType.CREATE));

		return release;
	}

	/**
	 * Updates the given release.
	 * 
	 * @param release
	 *            The release to be updated.
	 * @param name
	 *            The new name for the release
	 * @throws InvalidReleaseInputException
	 *             Thrown if the name is invalid for one of many reasons,
	 *             specified in the message text.
	 */
	public static void updateName(Release release, String name)
			throws InvalidReleaseInputException {
		if (release == null)
			throw new InvalidReleaseInputException(ReleaseErrorType.NULL);
		checkName(name, release.getProject());
		release.setName(name);
		release.save();
		manager.fireChange(new EntityEvent(release,
				EntityEvent.EventType.UPDATE));
	}

	/**
	 * Updates the given release
	 * 
	 * @param release
	 *            The release to be updated.
	 * @param actualDate
	 *            The actual date of the release.
	 * @throws InvalidReleaseInputException
	 *             Thrown if the input params are invalid.
	 */
	public static void updateActualDate(Release release, Date actualDate)
			throws InvalidReleaseInputException {
		if (release == null)
			throw new InvalidReleaseInputException(ReleaseErrorType.NULL);
		checkDates(release.getExpectedDate(), actualDate);
		release.setActualDate(actualDate);
		release.save();

		manager.fireChange(new EntityEvent(release,
				EntityEvent.EventType.UPDATE));
	}

	/**
	 * Updates the given release.
	 * 
	 * @param release
	 *            The release to be updated.
	 * @param actualDate
	 *            The new actual date for the iteration
	 * @param expectedDate
	 *            The new expected date for the iteration
	 * @throws InvalidReleaseInputException
	 *             Thrown if the dates are invalid for one of many reasons,
	 *             specified in the message text.
	 */
	public static void update(Release release, String name, Date expectedDate,
			Date actualDate) throws InvalidReleaseInputException {
		if (release == null)
			throw new InvalidReleaseInputException(ReleaseErrorType.NULL);
		checkDates(expectedDate, actualDate);
		checkName(name, release.getProject());

		release.setActualDate(actualDate);
		release.setExpectedDate(expectedDate);
		release.setName(name);
		release.save();
		manager.fireChange(new EntityEvent(release,
				EntityEvent.EventType.UPDATE));
	}

	/**
	 * Deletes the given release.
	 * 
	 * @param release
	 *            The release to delete.
	 * @throws InvalidReleaseInputException
	 *             If one of many errors occurs.
	 */
	public static void delete(Release release)
			throws InvalidReleaseInputException {
		if (release == null)
			throw new InvalidReleaseInputException(ReleaseErrorType.NULL);

		try {
			Database.manager().delete(release);
		} catch (SQLException e) {
			throw new InvalidReleaseInputException(
					ReleaseErrorType.DATABASE_RELEASE_DELETION, e.getMessage());
		}

		manager.fireChange(new EntityEvent(release,
				EntityEvent.EventType.DELETE));

	}

	// Public validation functions

	/**
	 * Validates the name obtained from r.getName()
	 * 
	 * @param r
	 *            This function will behave fine for any non-null i..
	 * @throws InvalidReleaseInputException
	 *             If i is null.
	 */
	public static void validateName(ReleaseAccessor r)
			throws InvalidReleaseInputException {
		if (r != null)
			checkName(r.getName(), r.getProject());
		else
			throw new InvalidReleaseInputException("ReleaseAccessor r is null.");
	}

	/**
	 * Validates the start and end dates obtained through r.actualDate() and
	 * r.expectedDate()
	 * 
	 * @param r
	 *            This function will behave fine for any non-null r.
	 * @throws InvalidReleaseInputException
	 *             If r is null.
	 */
	public static void validateDates(ReleaseAccessor r)
			throws InvalidReleaseInputException {
		if (r != null)
			checkDates(r.getExpectedDate(), r.getActualDate());
		else
			throw new InvalidReleaseInputException("ReleaseAccessor r is null.");
	}

	// Helper functions

	/**
	 * Validates the name of a release. Ensures the name of the release is not
	 * empty, and contains more than just spaces and is unique within a project.
	 * 
	 * @param name
	 *            The name (String) to validate
	 * @param release
	 *            The release whose name is being validated
	 * @throws InvalidReleaseInputException
	 *             If any one of the validations fail.
	 */
	private static void checkName(String name, Project project)
			throws InvalidReleaseInputException {
		if (name == null)
			throw new InvalidReleaseInputException(ReleaseErrorType.NAME_NULL);
		if (name.isEmpty())
			throw new InvalidReleaseInputException(ReleaseErrorType.NAME_EMPTY);
		if (name.length() > 255)
			throw new InvalidReleaseInputException(
					ReleaseErrorType.NAME_TOO_LONG);
		boolean valid = false;
		for (int i = 0; i < name.length(); i++)
			if (name.toCharArray()[i] != ' ') {
				valid = true;
				break;
			}
		if (!valid)
			throw new InvalidReleaseInputException(
					ReleaseErrorType.NAME_ALL_SPACES);
		if (name.startsWith(" "))
			throw new InvalidReleaseInputException(
					ReleaseErrorType.NAME_BEGIN_WITH_SPACE);

		if (project != null) {
			Release[] releases = project.getReleases();
			for (Release r : releases)
				if (r.getName().equals(name))
					throw new InvalidReleaseInputException(
							ReleaseErrorType.NAME_NOT_UNIQUE);
		}
	}

	/**
	 * Validates the dates of a release. Ensures that the expected date is
	 * present. The actual date may be null.
	 * 
	 * @param actualDate
	 *            The actual date to validate
	 * @param expectedDate
	 *            The expected date to validate
	 * @throws InvalidReleaseInputException
	 *             If any one of the validations fail.
	 */
	private static void checkDates(Date actualDate, Date expectedDate)
			throws InvalidReleaseInputException {
		if (expectedDate == null)
			throw new InvalidReleaseInputException(
					ReleaseErrorType.EXPECTED_DATE_NULL);

		// actual date may be null

		// these were the only ones I could think of for validation involving
		// actualDate and expectedDate.
		// if there are more constraints they can be added here..
	}

	/**
	 * return the estimate units of a release
	 * 
	 * @param release
	 *            a release
	 * @return the sum of the estimate units of the array of user stories
	 *         contained in the release
	 * @throws InvalidReleaseInputException
	 *             If the iteration is null.
	 */
	public static int countEstimatedUnits(Release r)
			throws InvalidReleaseInputException {
		if (r == null)
			throw new InvalidReleaseInputException(ReleaseErrorType.NULL);

		int sum = 0;
		Iteration[] releaseIterations = r.getIterations();
		for (Iteration i : releaseIterations) {
			UserStory[] iuserStories = i.getUserStories();
			for (UserStory us : iuserStories)
				sum = sum + us.getEstimatedUnits();
		}
		return sum;
	}

	/**
	 * return the actual units of a release
	 * 
	 * @param release
	 *            a release
	 * @return the sum of the actual units of the array of user stories
	 *         contained in the release
	 * @throws InvalidReleaseInputException
	 *             If the iteration is null.
	 */
	public static int countActualUnits(Release r)
			throws InvalidReleaseInputException {
		if (r == null)
			throw new InvalidReleaseInputException(ReleaseErrorType.NULL);

		int sum = 0;
		Iteration[] releaseIterations = r.getIterations();
		for (Iteration i : releaseIterations) {
			UserStory[] iuserStories = i.getUserStories();
			for (UserStory us : iuserStories)
				sum = sum + us.getActualUnits();
		}
		return sum;
	}

}
