/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alex Kindle - Email
 *    Dan Bjorge - Email
 *******************************************************************************/

package edu.wpi.wpisuite.data.funcs;

// TODO Convert to format of other managers
// Or at least error check this better

import java.io.IOException;
import java.sql.SQLException;
import java.util.Date;
import java.util.HashMap;
import java.util.Properties;
import java.security.Security;

import javax.mail.Authenticator;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.NoSuchProviderException;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import net.java.ao.DBParam;
import edu.wpi.wpisuite.data.Config;
import edu.wpi.wpisuite.data.Project;
import edu.wpi.wpisuite.data.ProjectMember;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.database.Database;

/**
 * Contains static utility and CRUD functions for User management
 */
public class UserManager {

	private final static int GENERATED_PASSWORD_LENGTH = 8;
	private final static String legalChars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";

	public static User create(String username, String firstName,
			String lastName, String emailAddress) throws Exception {

		String randomPassword = UserManager.generatePassword();

		if (Database.manager().find(User.class, "username = ?", username).length > 0)
			throw new Exception("A user with that username exists");

		try {
			UserManager.emailPassword(emailAddress, username, randomPassword,
					firstName, "Welcome to WPISuite, " + firstName,
					"Welcome to WPISuite, " + username + "! Your username is: "
							+ username + ", and your temporary password is: ");
		} catch (Exception e) {
			// e.printStackTrace();
			throw new IOException("Error sending email to user: "
					+ e.getMessage());

		}

		User newUser = Database.manager().create(User.class,
				new DBParam("username", username));

		newUser.setFirstName(firstName);
		newUser.setLastName(lastName);
		newUser.setEmailAddress(emailAddress);
		newUser.setPasswordExpired(true);
		newUser.setAdmin(false);
		newUser.setPasswordHash(Database.hashPassword(randomPassword));
		newUser.save();

		return newUser;
	}

	/**
	 * This function e-mails the user their user name and temporary, randomly
	 * generated password. The body is emailMessage + password
	 * 
	 * @param emailAddress
	 *            The user's e-mail address
	 * @param username
	 *            The user's username
	 * @param password
	 *            The user's temporary password
	 * @param name
	 *            The user's name
	 * @param subject
	 *            The subject of the email to send
	 * @param emailMessage
	 *            The preamble to the message to send
	 * @throws NoSuchProviderException
	 *             Session cannot be initiated
	 * @throws MessagingException
	 *             Message cannot be created
	 * @throws SQLException
	 *             Mail sender data could not be found in configuration table of
	 *             database
	 */
	public static void emailPassword(String emailAddress, String username,
			String password, String name, String subject, String emailMessage)
			throws NoSuchProviderException, MessagingException, SQLException {

		emailSetupAndSend(emailAddress, emailMessage + password, subject);
	}

	public static String generatePassword() {

		String newPassword = "";

		for (int i = 0; i < GENERATED_PASSWORD_LENGTH; i++)
			newPassword += legalChars.charAt((int) (Math.random() * legalChars
					.length()));

		return newPassword;
	}

	/**
	 * set a user to be an admin
	 * 
	 * @param setter
	 *            the setter
	 * @param user
	 *            the user to be set
	 * @throws Exception
	 *             if the setter have not enough power to do that
	 */
	// need to combine the following 2 functions in to 1 in the future
	public static void setToBeAdmin(User setter, User user) throws Exception {
		if (!setter.isAdmin())
			throw new Exception("The setter is not an administrator");
		user.setAdmin(true);
		user.save();
	}

	/**
	 * remove the admin power of a user
	 * 
	 * @param setter
	 *            the setter
	 * @param user
	 *            the user to be set
	 * @throws Exception
	 *             if the setter is not an admin or he wants to remove himself
	 */
	public static void removeAdmin(User setter, User user) throws Exception {
		if (!setter.isAdmin())
			throw new Exception("The setter is not an administrator");
		if (setter.getUsername().equals(user.getUsername()))
			throw new Exception("The setter cannot remove himself");
		user.setAdmin(false);
		user.save();
	}

	/**
	 * This sends out an email to the user who was locked out of his/her account
	 * Also it will need to send to the system admin (to be implemented)
	 * 
	 * @param username
	 *            - username of the locked account
	 * @throws NoSuchProviderException
	 * @throws MessagingException
	 * @throws SQLException
	 */
	public static void notifyAccountLockOut(String username)
			throws NoSuchProviderException, MessagingException, SQLException {
		User thisUser = Database.manager().find(User.class, " username LIKE ?",
				username)[0];
		User[] adminUsers = Database.manager()
				.find(User.class, " admin = ?", 1);

		String emailAddress = thisUser.getEmailAddress();
		String userMailMessage = "Your account on WPI Suite, "
				+ thisUser.getUsername() + ", has been locked. ";
		userMailMessage += "Please contact the system administrator to unlock your account or use the \"Forgot password?\" function on the login screen.";
		userMailMessage += "\n\n~The WPI Suite Team";

		emailSetupAndSend(emailAddress, userMailMessage, "Account Locked");

		String adminMailMessage = "User " + thisUser.getUsername()
				+ " has been locked out of his/her account.";

		for (int i = 0; i < adminUsers.length; i++)
			emailSetupAndSend(adminUsers[i].getEmailAddress(),
					adminMailMessage, "Account Locked");
	}

	/**
	 * General function that performs the send the actual email to the email
	 * address specified with the message
	 * 
	 * @param emailAddress
	 *            - receiver
	 * @param emailMessage
	 *            - message body
	 * @param emailSubject
	 *            - message subject
	 * @throws NoSuchProviderException
	 * @throws MessagingException
	 * @throws SQLException
	 */
	private static void emailSetupAndSend(String emailAddress,
			String emailMessage, String emailSubject)
			throws NoSuchProviderException, MessagingException, SQLException {
		String mailProtocol;
		String mailServer;
		final String mailUser;
		final String mailPassword;
		String mailSender;

		Config[] mailProtocolConfigs = Database.manager().find(Config.class,
				"name = ?", "mail_protocol");
		if (mailProtocolConfigs.length < 1)
			throw new SQLException("No mail protocol defined in configuration");
		else
			mailProtocol = mailProtocolConfigs[0].getValue();
		Config[] mailServerConfigs = Database.manager().find(Config.class,
				"name = ?", "mail_server");
		if (mailServerConfigs.length < 1)
			throw new SQLException("No smtp server defined in configuration");
		else
			mailServer = mailServerConfigs[0].getValue();
		Config[] mailUserConfigs = Database.manager().find(Config.class,
				"name= ?", "mail_user");
		if (mailUserConfigs.length < 1)
			throw new SQLException("No smtp username defined in configuration");
		else
			mailUser = mailUserConfigs[0].getValue();
		Config[] mailPasswordConfigs = Database.manager().find(Config.class,
				"name = ?", "mail_password");
		if (mailPasswordConfigs.length < 1)
			throw new SQLException("No smtp password defined in configuration");
		else
			mailPassword = mailPasswordConfigs[0].getValue();
		Config[] mailSenderConfigs = Database.manager().find(Config.class,
				"name = ?", "mail_from");
		if (mailSenderConfigs.length < 1)
			throw new SQLException("No mail Sender defined in configuration");
		else
			mailSender = mailSenderConfigs[0].getValue();

		Properties prop = new Properties();
		Session sMail;
		Transport tMail;
		MimeMessage message;

		prop.setProperty("mail.transport.protocol", mailProtocol);
		prop.setProperty("mail.smtp.auth", "true");
		prop.setProperty("mail.smtp.port", "465");
		prop.setProperty("mail.smtp.host", mailServer);
		prop.setProperty("mail.smtp.user", mailUser);
		prop.setProperty("mail.smtp.password", mailPassword);
		prop.setProperty("mail.smtp.from", mailSender);

		Authenticator auth = new javax.mail.Authenticator() {
			@Override
			public PasswordAuthentication getPasswordAuthentication() {
				return new PasswordAuthentication(mailUser, mailPassword);
			}
		};

		//java.security.Security.addProvider(new com.sun.net.ssl.internal.ssl.Provider());
		prop.setProperty("mail.smtp.socketFactory.class",
				"javax.net.ssl.SSLSocketFactory");
		prop.setProperty("mail.smtp.socketFactory.fallback", "false");
		prop.setProperty("mail.smtp.socketFactory.port", "465");

		sMail = Session.getDefaultInstance(prop, auth);
		tMail = sMail.getTransport();
		message = new MimeMessage(sMail);
		message.setSubject(emailSubject);
		message.setText(emailMessage);
		message.setHeader("X-Mailer", "Autogenerated e-mail from WPISuite");
		message.setSentDate(new Date());
		message.setFrom(new InternetAddress("noreply@wpisuite.wpi.edu"));
		message.addRecipient(Message.RecipientType.TO, new InternetAddress(
				emailAddress));

		tMail.connect();
		tMail.sendMessage(message, message
				.getRecipients(Message.RecipientType.TO));
		tMail.close();
	}

	// TODO: Change this. This is abysmal. This should have a ProjectPermissions
	// class or something similar.
	public static HashMap<String, Boolean> getPermissions(User user,
			Project project) {
		HashMap<String, Boolean> permissions = new HashMap<String, Boolean>();
		ProjectMember pm = null;

		try {
			ProjectMember[] assocs = Database.manager().find(
					ProjectMember.class, "projectID = ? and memberID = ?",
					project, user);
			if (assocs.length == 0)
				return permissions;
			else
				pm = assocs[0];
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		permissions.put("canReadUserStory", pm.canReadUserStory());
		permissions.put("canCreateUserStory", pm.canCreateUserStory());
		permissions.put("canEditUserStory", pm.canEditUserStory());

		permissions.put("canReadFeature", pm.canReadFeature());
		permissions.put("canCreateFeature", pm.canCreateFeature());
		permissions.put("canEditFeature", pm.canEditFeature());

		permissions.put("canReadIteration", pm.canReadIteration());
		permissions.put("canCreateIteration", pm.canCreateIteration());
		permissions.put("canEditIteration", pm.canEditIteration());

		permissions.put("canReadRelease", pm.canReadRelease());
		permissions.put("canCreateRelease", pm.canCreateRelease());
		permissions.put("canEditRelease", pm.canEditRelease());

		permissions.put("isAdmin", pm.isAdmin());

		return permissions;

	}

	public static HashMap<String, Boolean> getPermissions(ProjectMember pm) {
		HashMap<String, Boolean> permissions = new HashMap<String, Boolean>();

		permissions.put("canReadUserStory", pm.canReadUserStory());
		permissions.put("canCreateUserStory", pm.canCreateUserStory());
		permissions.put("canEditUserStory", pm.canEditUserStory());

		permissions.put("canReadFeature", pm.canReadFeature());
		permissions.put("canCreateFeature", pm.canCreateFeature());
		permissions.put("canEditFeature", pm.canEditFeature());

		permissions.put("canReadIteration", pm.canReadIteration());
		permissions.put("canCreateIteration", pm.canCreateIteration());
		permissions.put("canEditIteration", pm.canEditIteration());

		permissions.put("canReadRelease", pm.canReadRelease());
		permissions.put("canCreateRelease", pm.canCreateRelease());
		permissions.put("canEditRelease", pm.canEditRelease());

		return permissions;
	}

	public static void setPermissions(User user,
			HashMap<String, Boolean> perms, Project proj) {
		ProjectMember pm = null;
		try {
			pm = Database.manager()
					.find(ProjectMember.class, "projectID = ? and memberID= ?",
							proj.getID(), user.getID())[0];
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		pm.setCanCreateUserStory(perms.get("canCreateUserStory"));
		pm.setCanEditUserStory(perms.get("canEditUserStory"));
		pm.setCanReadUserStory(perms.get("canReadUserStory"));
		pm.setCanCreateFeature(perms.get("canCreateFeature"));
		pm.setCanEditFeature(perms.get("canEditFeature"));
		pm.setCanReadFeature(perms.get("canReadFeature"));
		pm.setCanCreateRelease(perms.get("canCreateRelease"));
		pm.setCanEditRelease(perms.get("canEditRelease"));
		pm.setCanReadRelease(perms.get("canReadRelease"));
		pm.setCanCreateIteration(perms.get("canCreateIteration"));
		pm.setCanEditIteration(perms.get("canEditIteration"));
		pm.setCanReadIteration(perms.get("canReadIteration"));
		pm.setAdmin(perms.get("isAdmin"));

		pm.save();
	}

}
