/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Ian Williams - Initial design, API, and implementation
 *    Ravi Singh
 *******************************************************************************/

package edu.wpi.wpisuite.data.funcs;

import java.sql.SQLException;
import java.util.Arrays;

import net.java.ao.DBParam;
import net.java.ao.Query;
import edu.wpi.wpisuite.data.Iteration;
import edu.wpi.wpisuite.data.Project;
import edu.wpi.wpisuite.data.TraceabilityAssociation;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.data.UserStory;
import edu.wpi.wpisuite.data.UserStory.Status;
import edu.wpi.wpisuite.data.errors.InvalidUserStoryInputException;
import edu.wpi.wpisuite.data.errors.UserStoryErrorType;
import edu.wpi.wpisuite.data.util.UserStoryAccessor;
import edu.wpi.wpisuite.database.Database;

//TODO Javadoc: throws should say which error types are possible for that function
/**
 * @author Ian Williams Class of CRUD functions for UserStory Needs more unit
 *         testing.
 */
public class UserStoryManager extends EntityManager {

	public static final Iteration BACKLOG = null;

	// Singleton, used for managing listeners
	private UserStoryManager() {
		super();
	}

	private static UserStoryManager manager = new UserStoryManager();

	// Public CRUD functions

	/**
	 * Creates a user story with all unprovided values set to null.
	 * 
	 * @param title
	 *            Must not begin with a space, contain only spaces, be empty,
	 *            null or duplicate of any other story's title in this project
	 * @param description
	 *            Any non-null or empty string.
	 * @param project
	 *            A non-null project.
	 * @return A reference to the newly created user story.
	 * @throws InvalidUserStoryInputException
	 *             One of any error detailed in UserStoryErrorType.
	 */
	public static UserStory create(String title, String description,
			Project project) throws InvalidUserStoryInputException {

		UserStory story;

		checkProject(project);
		checkTitle(title, project);
		checkDescription(description);

		try {
			story = Database.manager().create(UserStory.class,
					new DBParam("title", title));
		} catch (SQLException e) {
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.DATABASE_USERSTORY_CREATION, e
							.getMessage());
		}

		story.setDescription(description);
		story.setProject(project);
		story.save();

		manager
				.fireChange(new EntityEvent(story, EntityEvent.EventType.CREATE));

		return story;
	}

	/**
	 * Creates a user story using the provided values.
	 * 
	 * @return A reference to the newly created user story.
	 * @param title
	 *            Must not begin with a space, contain only spaces, be empty,
	 *            null or duplicate of any other story's title in this project
	 * @param description
	 *            Any non-null or empty string.
	 * @param estimatedUnits
	 *            May be any non-negative number.
	 * @param actualUnits
	 *            May be any non-negative number.
	 * @param status
	 *            Is allowed to be null.
	 * @param author
	 *            Is allowed to be null. Must be a user of this story's project.
	 * @param owner
	 *            Is allowed to be null. Must be a user of this story's project.
	 * @param iteration
	 *            Is allowed to be null (BACKLOG). Must be an iteration of this
	 *            story's project.
	 * @param project
	 *            A non-null project.
	 * @return A reference to the newly created user story.
	 * @throws InvalidUserStoryInputException
	 *             One of any error detailed in UserStoryErrorType.
	 */
	public static UserStory create(String title, String description,
			int estimatedUnits, int actualUnits, Status status, User author,
			User owner, Iteration iteration, Project project)
			throws InvalidUserStoryInputException {

		UserStory story;

		checkProject(project);
		checkTitle(title, project);
		checkDescription(description);
		checkEstimatedUnits(estimatedUnits);
		checkActualUnits(actualUnits);
		checkAuthor(author, project);
		checkOwner(owner, project);
		checkIteration(iteration, project);

		try {
			story = Database.manager().create(UserStory.class,
					new DBParam("title", title));
		} catch (SQLException e) {
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.DATABASE_USERSTORY_CREATION, e
							.getMessage());
		}

		story.setProject(project);
		story.setDescription(description);
		story.setEstimatedUnits(estimatedUnits);
		story.setActualUnits(actualUnits);
		story.setStatus(status);
		story.setAuthor(author);
		story.setOwner(owner);
		story.setIteration(iteration);
		story.save();

		manager
				.fireChange(new EntityEvent(story, EntityEvent.EventType.CREATE));

		return story;
	}

	/**
	 * Updates the title of this user story.
	 * 
	 * @param story
	 *            Should be non-null.
	 * @param title
	 *            Must meet the usual guidelines detailed in this class's create
	 *            functions.
	 * @throws InvalidUserStoryInputException
	 *             One of any error detailed in UserStoryErrorType.
	 */
	public static void updateTitle(UserStory story, String title)
			throws InvalidUserStoryInputException {
		if (story == null)
			throw new InvalidUserStoryInputException(UserStoryErrorType.NULL);
		checkTitle(title, story.getProject(), story.getID());
		story.setTitle(title);
		story.save();
		manager
				.fireChange(new EntityEvent(story, EntityEvent.EventType.UPDATE));
	}

	/**
	 * Updates this story's description.
	 * 
	 * @param story
	 *            Should be non-null.
	 * @param description
	 *            Must meet the usual guidelines detailed in this class's create
	 *            functions.
	 * @throws InvalidUserStoryInputException
	 *             One of any error detailed in UserStoryErrorType.
	 */
	public static void updateDescription(UserStory story, String description)
			throws InvalidUserStoryInputException {
		if (story == null)
			throw new InvalidUserStoryInputException(UserStoryErrorType.NULL);
		checkDescription(description);
		story.setDescription(description);
		story.save();
		manager
				.fireChange(new EntityEvent(story, EntityEvent.EventType.UPDATE));

	}

	/**
	 * Updates this story's estimated units.
	 * 
	 * @param story
	 *            Should be non-null.
	 * @param estimatedUnits
	 *            Must meet the usual guidelines detailed in this class's create
	 *            functions.
	 * @throws InvalidUserStoryInputException
	 *             One of any error detailed in UserStoryErrorType.
	 */
	public static void updateEstimatedUnits(UserStory story, int estimatedUnits)
			throws InvalidUserStoryInputException {
		if (story == null)
			throw new InvalidUserStoryInputException(UserStoryErrorType.NULL);
		checkEstimatedUnits(estimatedUnits);
		story.setEstimatedUnits(estimatedUnits);
		story.save();
		manager
				.fireChange(new EntityEvent(story, EntityEvent.EventType.UPDATE));

	}

	/**
	 * Updates this story's actual units.
	 * 
	 * @param story
	 *            Should be non-null.
	 * @param actualUnits
	 *            Must meet the usual guidelines detailed in this class's create
	 *            functions.
	 * @throws InvalidUserStoryInputException
	 *             One of any error detailed in UserStoryErrorType.
	 */
	public static void updateActualUnits(UserStory story, int actualUnits)
			throws InvalidUserStoryInputException {
		if (story == null)
			throw new InvalidUserStoryInputException(UserStoryErrorType.NULL);
		checkActualUnits(actualUnits);
		story.setActualUnits(actualUnits);
		story.save();
		manager
				.fireChange(new EntityEvent(story, EntityEvent.EventType.UPDATE));

	}

	/**
	 * Updates this story's status.
	 * 
	 * @param story
	 *            Should be non-null.
	 * @param status
	 *            Must meet the usual guidelines detailed in this class's create
	 *            functions.
	 * @throws InvalidUserStoryInputException
	 *             One of any error detailed in UserStoryErrorType.
	 */
	public static void updateStatus(UserStory story, Status status)
			throws InvalidUserStoryInputException {
		if (story == null)
			throw new InvalidUserStoryInputException(UserStoryErrorType.NULL);
		story.setStatus(status);
		story.save();
		manager
				.fireChange(new EntityEvent(story, EntityEvent.EventType.UPDATE));

	}

	/**
	 * Updates this story's author.
	 * 
	 * @param story
	 *            Should be non-null.
	 * @param author
	 *            Must meet the usual guidelines detailed in this class's create
	 *            functions.
	 * @throws InvalidUserStoryInputException
	 *             One of any error detailed in UserStoryErrorType.
	 */
	public static void updateAuthor(UserStory story, User author)
			throws InvalidUserStoryInputException {
		if (story == null)
			throw new InvalidUserStoryInputException(UserStoryErrorType.NULL);
		checkAuthor(author, story.getProject());
		story.setAuthor(author);
		story.save();
		manager
				.fireChange(new EntityEvent(story, EntityEvent.EventType.UPDATE));

	}

	/**
	 * Updates this story's owner.
	 * 
	 * @param story
	 *            Should be non-null.
	 * @param owner
	 *            Must meet the usual guidelines detailed in this class's create
	 *            functions.
	 * @throws InvalidUserStoryInputException
	 *             One of any error detailed in UserStoryErrorType.
	 */
	public static void updateOwner(UserStory story, User owner)
			throws InvalidUserStoryInputException {
		if (story == null)
			throw new InvalidUserStoryInputException(UserStoryErrorType.NULL);
		checkOwner(owner, story.getProject());
		story.setOwner(owner);
		story.save();
		manager
				.fireChange(new EntityEvent(story, EntityEvent.EventType.UPDATE));

	}

	/**
	 * Updates this story's status. Note that if this user story is closed, this
	 * function will always throw an exception with errorType STORY_COMPLETED.
	 * 
	 * @param story
	 *            Should be non-null.
	 * @param iteration
	 *            Must meet the usual guidelines detailed in this class's create
	 *            functions.
	 * @throws InvalidUserStoryInputException
	 *             One of any error detailed in UserStoryErrorType.
	 */
	public static void updateIteration(UserStory story, Iteration iteration)
			throws InvalidUserStoryInputException {
		if (story == null)
			throw new InvalidUserStoryInputException(UserStoryErrorType.NULL);
		if (story.getStatus().equals(Status.COMPLETED)
				&& !iteration.equals(story.getIteration()))
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.STORY_COMPLETED);
		checkIteration(iteration, story.getProject());
		story.setIteration(iteration);
		story.save();
		manager
				.fireChange(new EntityEvent(story, EntityEvent.EventType.UPDATE));

	}

	/**
	 * Moves the user story to a different project. As a result of this, the
	 * story will have it's iteration set to BACKLOG, and it's owner and author
	 * set to null.
	 * 
	 * @param story
	 *            Works on any non-null user story.
	 * @param project
	 *            The new project (non-null or an exception is thrown)
	 * @throws InvalidUserStoryInputException
	 *             On any invalid input, detailed in this exceptinon's
	 *             getErrorType()
	 */
	public static void updateProject(UserStory story, Project project)
			throws InvalidUserStoryInputException {
		if (story == null)
			throw new InvalidUserStoryInputException(UserStoryErrorType.NULL);
		checkProject(project);
		story.setIteration(BACKLOG);
		story.setAuthor(null);
		story.setOwner(null);
		story.setProject(project);
		story.save();
		manager
				.fireChange(new EntityEvent(story, EntityEvent.EventType.UPDATE));

	}
//TODO feature delete
	/**
	 * Deletes this user story and every <code> TraceabilityAssociation </code> involving it.
	 * 
	 * @param story
	 *            Should be non-null.
	 * @throws InvalidUserStoryInputException
	 *             One of any error detailed in UserStoryErrorType, though none
	 *             are likely to occur except NULL.
	 */
	public static void delete(UserStory story)
			throws InvalidUserStoryInputException {
		if (story == null)
			throw new InvalidUserStoryInputException(UserStoryErrorType.NULL);

		try {
			TraceabilityAssociation[] associations = Database.manager().find(TraceabilityAssociation.class, Query.select().where("from = ? OR to = ?", story, story));
			for(TraceabilityAssociation t : associations) {
				Database.manager().delete(t);
			}
		} catch (SQLException e) {
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.DATABASE_USERSTORY_DELETION, e
							.getMessage());
		}
		
		try {
			Database.manager().delete(story);
			manager.fireChange(new EntityEvent(story,
					EntityEvent.EventType.DELETE));

		} catch (SQLException e) {
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.DATABASE_USERSTORY_DELETION, e
							.getMessage());
		}
	}

	/**
	 * Updates this user story will totally new information. To keep a
	 * particular field unchanged, pass the current value in. If a single error
	 * occurs, none of the changes will be made.
	 * 
	 * @param story
	 *            Should be non-null.
	 * @param title
	 *            Must meet the usual guidelines detailed in this class's create
	 *            functions.
	 * @param description
	 *            Must meet the usual guidelines detailed in this class's create
	 *            functions.
	 * @param estimatedUnits
	 *            Must meet the usual guidelines detailed in this class's create
	 *            functions.
	 * @param actualUnits
	 *            Must meet the usual guidelines detailed in this class's create
	 *            functions.
	 * @param status
	 *            Must meet the usual guidelines detailed in this class's create
	 *            functions.
	 * @param author
	 *            Must meet the usual guidelines detailed in this class's create
	 *            functions.
	 * @param owner
	 *            Must meet the usual guidelines detailed in this class's create
	 *            functions.
	 * @param iteration
	 *            Must meet the usual guidelines detailed in this class's create
	 *            functions.
	 * @param project
	 *            Must meet the usual guidelines detailed in this class's create
	 *            functions.
	 * @throws InvalidUserStoryInputException
	 *             One of any error detailed in UserStoryErrorType.
	 */
	public static void update(UserStory story, String title,
			String description, int estimatedUnits, int actualUnits,
			Status status, User author, User owner, Iteration iteration,
			Project project) throws InvalidUserStoryInputException {
		if (title == null)
			throw new InvalidUserStoryInputException(UserStoryErrorType.NULL);
		checkProject(project);
		checkIteration(iteration, project);
		checkOwner(owner, project);
		checkAuthor(author, project);
		checkActualUnits(actualUnits);
		checkEstimatedUnits(estimatedUnits);
		checkDescription(description);
		checkTitle(title, project, story.getID());
		if (story.getStatus().equals(Status.COMPLETED)
				&& !iteration.equals(story.getIteration()))
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.STORY_COMPLETED);
		story.setProject(project);
		story.setIteration(iteration);
		story.setOwner(owner);
		story.setAuthor(author);
		story.setActualUnits(actualUnits);
		story.setEstimatedUnits(estimatedUnits);
		story.setStatus(status);
		story.setDescription(description);
		story.setTitle(title);
		story.save();
	}

	// Public Validation Functions

	/**
	 * Validates the title as if updateIteration had been called.
	 * 
	 * @param accessor
	 *            getTitle(), getProject(), and getID() must all give valid
	 *            results.
	 * @throws InvalidUserStoryInputException
	 *             One of any error detailed in UserStoryErrorType.
	 */
	public static void validateTitle(UserStoryAccessor accessor)
			throws InvalidUserStoryInputException {
		if (accessor != null)
			checkTitle(accessor.getTitle(), accessor.getProject(), accessor
					.getID());
		else
			throw new InvalidUserStoryInputException(
					"UserStoryAccessor accessor is null.");
	}

	/**
	 * Validates the title as if create had been called.
	 * 
	 * @param accessor
	 *            getTitle() and getProject() must both give valid results.
	 * @throws InvalidUserStoryInputException
	 *             One of any error detailed in UserStoryErrorType.
	 */
	public static void validateNewTitle(UserStoryAccessor accessor)
			throws InvalidUserStoryInputException {
		if (accessor != null)
			checkTitle(accessor.getTitle(), accessor.getProject());
		else
			throw new InvalidUserStoryInputException(
					"UserStoryAccessor accessor is null.");
	}

	/**
	 * Validates the description as if updateDescription or create had been
	 * called.
	 * 
	 * @param accessor
	 *            getDescription() must give a valid result.
	 * @throws InvalidUserStoryInputException
	 *             One of any error detailed in UserStoryErrorType.
	 */
	public static void validateDescription(UserStoryAccessor accessor)
			throws InvalidUserStoryInputException {
		if (accessor != null)
			checkDescription(accessor.getDescription());
		else
			throw new InvalidUserStoryInputException(
					"UserStoryAccessor accessor is null.");
	}

	/**
	 * Validates the estimated units as if updateEstimatedUnits or create had
	 * been called.
	 * 
	 * @param accessor
	 *            getEstimatedUnits() must give a valid result.
	 * @throws InvalidUserStoryInputException
	 *             One of any error detailed in UserStoryErrorType.
	 */
	public static void validateEstimatedUnits(UserStoryAccessor accessor)
			throws InvalidUserStoryInputException {
		if (accessor != null)
			checkEstimatedUnits(accessor.getEstimatedUnits());
		else
			throw new InvalidUserStoryInputException(
					"UserStoryAccessor accessor is null.");
	}

	/**
	 * Validates the estimated units as if updateActualUnits() or create had
	 * been called.
	 * 
	 * @param accessor
	 *            getActualUnits() must give a valid result.
	 * @throws InvalidUserStoryInputException
	 *             One of any error detailed in UserStoryErrorType.
	 */
	public static void validateActualUnits(UserStoryAccessor accessor)
			throws InvalidUserStoryInputException {
		if (accessor != null)
			checkActualUnits(accessor.getActualUnits());
		else
			throw new InvalidUserStoryInputException(
					"UserStoryAccessor accessor is null.");
	}

	/**
	 * Validates the iteration as if updateIteration or create had been called,
	 * but without being given the actual user story's old iteration, this
	 * function cannot make a proper check with regards the iteration. Use the
	 * other version of this function if possible.
	 * 
	 * @param accessor
	 *            getIteration() and getProject() must both give valid results.
	 * @throws InvalidUserStoryInputException
	 *             One of any error detailed in UserStoryErrorType.
	 */
	public static void validateIteration(UserStoryAccessor accessor)
			throws InvalidUserStoryInputException {
		if (accessor != null)
			checkIteration(accessor.getIteration(), accessor.getProject());
		else
			throw new InvalidUserStoryInputException(
					"UserStoryAccessor accessor is null.");
	}

	/**
	 * Validates the iteration as if updateIteration or create had been called.
	 * Use over other validateIteration where possible.
	 * 
	 * @param accessor
	 *            getIteration() and getProject() must both give valid results.
	 * @throws InvalidUserStoryInputException
	 * @throws InvalidUserStoryInputException
	 *             One of any error detailed in UserStoryErrorType.
	 */
	public static void validateIteration(UserStoryAccessor accessor,
			Iteration oldIteration) throws InvalidUserStoryInputException {
		if (accessor != null) {
			if (accessor.getStatus().equals(Status.COMPLETED)
					&& !oldIteration.equals(accessor.getIteration()))
				throw new InvalidUserStoryInputException(
						UserStoryErrorType.STORY_COMPLETED);
			checkIteration(accessor.getIteration(), accessor.getProject());
		} else
			throw new InvalidUserStoryInputException(
					"UserStoryAccessor accessor is null.");

	}

	/**
	 * Validates the author as if updateAuthor or create had been called.
	 * 
	 * @param accessor
	 *            getAuthor() and getProject() must both give valid results.
	 * @throws InvalidUserStoryInputException
	 *             One of any error detailed in UserStoryErrorType.
	 */
	public static void validateAuthor(UserStoryAccessor accessor)
			throws InvalidUserStoryInputException {
		if (accessor != null)
			checkAuthor(accessor.getAuthor(), accessor.getProject());
		else
			throw new InvalidUserStoryInputException(
					"UserStoryAccessor accessor is null.");
	}

	/**
	 * Validates the owner as if updateOwner() or create had been called.
	 * 
	 * @param accessor
	 *            getOwner() and getProject() must both give valid results.
	 * @throws InvalidUserStoryInputException
	 *             One of any error detailed in UserStoryErrorType.
	 */
	public static void validateOwner(UserStoryAccessor accessor)
			throws InvalidUserStoryInputException {
		if (accessor != null)
			checkOwner(accessor.getOwner(), accessor.getProject());
		else
			throw new InvalidUserStoryInputException(
					"UserStoryAccessor accessor is null.");
	}

	/**
	 * Validates the project as if updateAuthor or create had been called.
	 * 
	 * @param accessor
	 *            getProject() must give a valid result.
	 * @throws InvalidUserStoryInputException
	 *             One of any error detailed in UserStoryErrorType.
	 */
	public static void validateProject(UserStoryAccessor accessor)
			throws InvalidUserStoryInputException {
		if (accessor != null)
			checkProject(accessor.getProject());
		else
			throw new InvalidUserStoryInputException(
					"UserStoryAccessor accessor is null.");
	}

	// Helper Functions

	private static void checkTitle(String title, Project project, int ID)
			throws InvalidUserStoryInputException {

		if (title == null)
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.NAME_NULL);
		if (title.isEmpty())
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.NAME_EMPTY);
		if (title.length() > 255)
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.NAME_TOO_LONG);
		boolean valid = false;
		for (int i = 0; i < title.length(); i++)
			if (title.toCharArray()[i] != ' ') {
				valid = true;
				break;
			}
		if (!valid)
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.NAME_ALL_SPACES);
		if (title.startsWith(" "))
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.NAME_BEGIN_WITH_SPACE);

		if (project != null) {
			UserStory[] stories = project.getUserStories();
			for (UserStory story : stories)
				if (ID != story.getID())
					if (story.getTitle().equals(title))
						throw new InvalidUserStoryInputException(
								UserStoryErrorType.NAME_NOT_UNIQUE);
		}
	}

	private static void checkTitle(String title, Project project)
			throws InvalidUserStoryInputException {

		if (title == null)
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.NAME_NULL);
		if (title.isEmpty())
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.NAME_EMPTY);
		if (title.length() > 255)
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.NAME_TOO_LONG);
		boolean valid = false;
		for (int i = 0; i < title.length(); i++)
			if (title.toCharArray()[i] != ' ') {
				valid = true;
				break;
			}
		if (!valid)
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.NAME_ALL_SPACES);
		if (title.startsWith(" "))
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.NAME_BEGIN_WITH_SPACE);

		if (project != null) {
			UserStory[] stories = project.getUserStories();
			for (UserStory story : stories)
				if (story.getTitle().equals(title))
					throw new InvalidUserStoryInputException(
							UserStoryErrorType.NAME_NOT_UNIQUE);
		}
	}

	private static void checkDescription(String description)
			throws InvalidUserStoryInputException {

		if (description == null)
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.DESCRIPTION_NULL);
		if (description.isEmpty())
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.DESCRIPTION_EMPTY);
		if (description.length() > 255)
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.DESCRIPTION_TOO_LONG);
		boolean valid = false;
		for (int i = 0; i < description.length(); i++)
			if (description.toCharArray()[i] != ' ') {
				valid = true;
				break;
			}
		if (!valid)
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.DESCRIPTION_ALL_SPACES);
	}

	private static void checkEstimatedUnits(int estimatedUnits)
			throws InvalidUserStoryInputException {
		if (estimatedUnits < 0)
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.ESTIMATED_UNITS_NEGATIVE);
	}

	private static void checkActualUnits(int actualUnits)
			throws InvalidUserStoryInputException {
		if (actualUnits < 0)
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.ACTUAL_UNITS_NEGATIVE);
	}

	private static void checkIteration(Iteration iteration, Project project)
			throws InvalidUserStoryInputException {
		if (iteration == BACKLOG)
			return;
		if (!iteration.getProject().equals(project))
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.ITERATION_INVALID);
	}

	private static void checkAuthor(User author, Project project)
			throws InvalidUserStoryInputException {
		if (author == null)
			return;
		boolean valid = false;
		for (User u : project.getMembers())
			if (u.equals(author)) {
				valid = true;
				break;
			}
		if (!valid)
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.AUTHOR_INVALID);
	}

	private static void checkOwner(User owner, Project project)
			throws InvalidUserStoryInputException {
		if (owner == null)
			return;
		boolean valid = false;
		for (User u : project.getMembers())
			if (u.equals(owner)) {
				valid = true;
				break;
			}
		if (!valid)
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.OWNER_INVALID);
	}

	private static void checkProject(Project project)
			throws InvalidUserStoryInputException {
		if (project == null)
			throw new InvalidUserStoryInputException(
					UserStoryErrorType.NO_PROJECT);
	}

	/**
	 * copy a UserStory to another Project(artf4372)
	 * 
	 * @param story
	 *            the story to be copied
	 * @param project
	 *            the project copied to
	 * @param iteration
	 *            the new project's iteration it goes to
	 * @param copyer
	 *            the user who did the copying
	 * @param keepOwner
	 *            whether to keep the previous owner if there exist a owner
	 * @return The newly copied user story
	 * @throws InvalidUserStoryInputException
	 */
	public static UserStory copyStory(UserStory story, Project project,
			Iteration iteration, User copyer, boolean keepOwner)
			throws InvalidUserStoryInputException {

		User newOwner = null;
		User newAuthor = null;
		if (keepOwner
				&& story.getOwner() != null
				&& Arrays.asList(project.getMembers()).contains(
						story.getOwner()))
			newOwner = story.getOwner();
		if (Arrays.asList(project.getMembers()).contains(copyer))
			newAuthor = copyer;

		UserStory u = UserStoryManager.create(story.getTitle(), story
				.getDescription(), story.getEstimatedUnits(), story
				.getActualUnits(), story.getStatus(), newAuthor, newOwner,
				iteration, project);

		return u;
	}

	/**
	 * Function creates a new user story and stores it in the database if the
	 * new user story object passes all validations. An exception is thrown
	 * otherwise.
	 * 
	 * @param title
	 * @param description
	 * @param estimatedUnits
	 * @param actualUnits
	 * @param status
	 * @param author
	 * @param owner_usrname
	 * @param iteration
	 * @param current_project
	 * @throws Exception
	 * @throws Exception
	 * @return the newly created user story
	 * @deprecated
	 */
	@Deprecated
	public static UserStory createUserStory(String title, String description,
			Integer estimatedUnits, Integer actualUnits,
			UserStory.Status status, User author, User owner,
			Iteration iteration, Project current_project) throws Exception {
		try {
			if (title.isEmpty() || description.isEmpty()) {
				System.out.println(title);
				System.out.println(description);
				// JOptionPane.showMessageDialog(null,
				// "Title and Description field must not be empty", "ERROR", 1);
				throw new Exception();
			}

			Iteration[] projectIterations = current_project.getIterations();

			UserStory newUserStory = Database.manager().create(UserStory.class);
			newUserStory.setAuthor(author);
			newUserStory.setTitle(title);
			newUserStory.setDescription(description);

			if (iteration == null)
				newUserStory.setIteration(null);
			else if (Arrays.asList(projectIterations).contains(iteration))
				newUserStory.setIteration(iteration);
			else
				throw new Exception(
						"Iteration specified does not exist in this project");
			// add user dialog to create desired iteration...
			newUserStory.setOwner(owner);
			newUserStory.setStatus(status);

			newUserStory.setActualUnits(actualUnits);
			newUserStory.setEstimatedUnits(estimatedUnits);

			newUserStory.save();
			return newUserStory;

		} catch (Exception e) {
			throw e;
		}
	}

	/**
	 * 
	 * @param user1
	 *            User who wants to assign others
	 * @param user2
	 *            User to be assigned
	 * @param story
	 *            Story to be assigned
	 * @throws Exception
	 *             for some failing situation
	 * @deprecated
	 */
	@Deprecated
	public static void assignUser(User user1, User user2, UserStory story)
			throws Exception {
		if (story.getOwner() != null)
			throw new Exception("There's already a owner");
		if (user1.isAdmin()) {
			story.setOwner(user2);
			story.save();
		} else
			throw new Exception("You don't have enough power to do this");
	}

	public static boolean isCompleted(UserStory us) {
		return us.getStatus().equals(Status.COMPLETED);
	}

}
