/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *    Lucas Scotta - Additional implementation
 *******************************************************************************/

package edu.wpi.wpisuite.database;

import java.io.IOException;
import java.lang.reflect.Array;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import net.java.ao.DatabaseProvider;
import net.java.ao.DefaultPolymorphicTypeMapper;
import net.java.ao.EntityManager;
import net.java.ao.Polymorphic;
import net.java.ao.RawEntity;
import net.java.ao.types.TypeManager;
import edu.wpi.wpisuite.data.ActiveMatrixRelation;
import edu.wpi.wpisuite.data.Config;
import edu.wpi.wpisuite.data.Feature;
import edu.wpi.wpisuite.data.Iteration;
import edu.wpi.wpisuite.data.Project;
import edu.wpi.wpisuite.data.ProjectMember;
import edu.wpi.wpisuite.data.Release;
import edu.wpi.wpisuite.data.Requirement;
import edu.wpi.wpisuite.data.TraceabilityAssociation;
import edu.wpi.wpisuite.data.TraceabilityMatrix;
import edu.wpi.wpisuite.data.TraceabilityMatrixCriteria;
import edu.wpi.wpisuite.data.TraceabilityMatrixCriterion;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.data.UserStory;

/**
 * Database is a wrapper for the database connection. It provides a single
 * global connection for the program to use, instantiating the connection lazily
 * and automatically. Its construction is very similar to but not exactly the
 * same as the singleton design pattern. It should never need to be
 * instantiated, and for the most part only the Database.getConnection() method
 * should be needed.
 * 
 * However, this class also contains several static helper functions that relate
 * to database access - for example, hashPassword and questionMarks
 * 
 * @see java.sql.Connection
 */
public class Database {

	/** Contains the EntityManager itself, or null if we haven't needed it yet */
	private static EntityManager managerInstance;

	/**
	 * Gets the single EntityManager that all objects from the database are
	 * based on
	 * 
	 * @return The single instance of EntityManager for this program to use
	 */
	public static EntityManager manager() {
		if (Database.managerInstance == null)
			try {
				Database.initManager();
			} catch (IOException e) {
				throw new RuntimeException(e.getMessage());
			}
		return Database.managerInstance;
	}

	/**
	 * The Database class is purely static, so we make this method protected and
	 * put Database in its own package.
	 */
	protected Database() {
	}

	/**
	 * Initializes the connection to the database. Attempts to connect to our
	 * current testing MySQL database on the CCC's servers. Note that there is
	 * NO GUARANTEE that the schema is correct - this function merely provides a
	 * connection.
	 * 
	 * @throws If
	 *             the connection to the database cannot be made, an
	 *             SQLException is thrown.
	 */
	@SuppressWarnings("unchecked")
	private static void initManager() throws IOException {
		DatabaseConfig config = DatabaseConfig.instance();
		try {
			config.getConfig();
		} catch (IOException e) {
			throw new IOException("Invalid database configuration: "
					+ e.getMessage());
		}

		String username = config.username();
		String password = config.password();
		String url = "jdbc:" + config.url();

		TypeManager.getInstance().addType(new IntArrayType());

		Database.managerInstance = new EntityManager(url, username, password);
		// This is one of the few things ActiveObjects can't do on its own.
		// We need to tell it the classes of any data interfaces which are
		// subinterfaces of any "abstract" database types
		Database.managerInstance
				.setPolymorphicTypeMapper(new DefaultPolymorphicTypeMapper(
						Feature.class, UserStory.class));
	}

	/**
	 * MD5s the given string and returns the resulting 32 character hash.
	 * 
	 * @param arg
	 *            The string to get the MD5 hash of
	 * @return arg's MD5 hash (32 hex characters)
	 * @throws NoSuchAlgorithmException
	 *             If the system cannot find the MD5 algorithm
	 */
	protected static String md5(String arg) throws NoSuchAlgorithmException {
		MessageDigest digest = MessageDigest.getInstance("MD5");
		digest.update(arg.getBytes(), 0, arg.length());
		String hash = new BigInteger(1, digest.digest()).toString(16);
		while (hash.length() < 32)
			hash = "0" + hash;
		return hash;
	}

	/**
	 * Salts and hashes the given password for storage or lookup in the
	 * database. Plaintext passwords should never be stored for longer than it
	 * takes to hash them with this function.
	 * 
	 * @param password
	 *            The plaintext password
	 * @return The salted and hashed version of the password, for comparison in
	 *         the database
	 * @throws RuntimeException
	 *             If the md5 algorithm couldn't be found on the host system
	 */
	public static String hashPassword(String password) {
		String salt = "k+5Po_jNN23";
		try {
			return Database.md5(password + salt);
		} catch (NoSuchAlgorithmException e) {
			throw new RuntimeException(
					"Could not find the password hashing algorithm. Please install the latest version of Java.");
		}
	}

	/**
	 * Generates a variable number of ", "-separated question marks for use with
	 * prepared SQL statements. For example, questionMarks(3) returns "?, ?, ?"
	 * 
	 * @param numMarks
	 *            The desired number of question marks.
	 * @return The string of numMarks question marks
	 */
	public static String questionMarks(int numMarks) {
		return Database.joinCopies("?", numMarks, ", ");
	}

	/**
	 * Generates a string that is numCopies of original, each separated by
	 * separator eg, joinCopies("a", 3, " : ") returns "a : a : a"
	 * 
	 * If numCopies is <= 0, returns ""
	 * 
	 * @param original
	 *            The item to copy
	 * @param numCopies
	 *            The amount of copies of original.
	 * @param separator
	 *            The string to separate copies by
	 * @return The generated string, as described above
	 */
	public static String joinCopies(String original, int numCopies,
			String separator) {
		if (numCopies <= 0)
			return "";
		else {
			String ret = original;
			for (int i = 1; i < numCopies; i++) {
				ret += separator;
				ret += original;
			}
			return ret;
		}
	}

	/**
	 * Joins a list of strings into one string, each element separated by the
	 * given separator.
	 * 
	 * Example: joinStrings(["a", "b", "c"], ";") returns "a;b;c"
	 * 
	 * @param strings
	 *            A list of strings to join
	 * @param separator
	 *            The separator to put between list items
	 * @return The joined string
	 */
	public static String joinStrings(List<String> strings, String separator) {
		if (strings.size() <= 0)
			return "";
		else {
			Iterator<String> i = strings.iterator();
			String ret = i.next();
			while (i.hasNext()) {
				ret += separator;
				ret += i.next();
			}
			return ret;
		}
	}

	/**
	 * Attempts to migrate the database tables according to the schema necessary
	 * for the objects in use. Attempts but does not guarantee to preserve all
	 * previous data data.
	 * 
	 * @throws IOException
	 *             If any error occurs in database connection or migration
	 */
	public static void migrateTables() throws IOException {
		try {
			Database.manager().migrate(getEntityClasses());
		} catch (SQLException e) {
			throw new IOException(e);
		}
	}

	/**
	 * Migrates the database tables
	 * 
	 * @param args
	 *            Ignored
	 * @throws IOException
	 *             If anything goes wrong in getting the database configuration
	 */
	public static void main(String[] args) throws IOException {
		Database.migrateTables();
	}

	/**
	 * Tests whether a given database connection info allows for the successful
	 * creation of an EntityManager. Throws an exception on an error, does
	 * nothing but returns successfully on success.
	 * 
	 * @param username
	 *            The database username to try
	 * @param password
	 *            The cleartext database password to try
	 * @param url
	 *            The database url to try
	 * @throws Exception
	 *             If anything goes wrong in attempting a database connection
	 */
	public static void testConnection(String username, String password,
			String url) throws Exception {
		String fullurl = "jdbc:" + url;
		DatabaseProvider provider = DatabaseProvider.getInstance(fullurl,
				username, password);
		Connection conn = provider.getConnection();
		conn.close();
	}

	/**
	 * Reinitializes the entity manager according to the current DatabaseConfig
	 * 
	 * @throws IOException
	 *             If any error occurs in setting up the manager
	 * @see initManager()
	 */
	public static void reinitializeManager() throws IOException {
		initManager();
	}

	/**
	 * Assuming the EntityManager has been initiailized properly, validates that
	 * the table it is associated with is valid for use with this application.
	 * 
	 * At this point, this method merely checks that the correct tables exist.
	 * It does NOT verify that those tables are appropriately constructed.
	 * 
	 * @throws Exception
	 *             If the database is not valid.
	 */
	public static void validateDatabase() throws Exception {

		Collection<String> necessaryTables = getNecessaryTableNames();
		Collection<String> actualTables = getActualTableNames();

		Set<String> matchingNTables = new HashSet<String>();
		Set<String> matchingATables = new HashSet<String>();

		for (String n : necessaryTables)
			for (String a : actualTables)
				if (n.equalsIgnoreCase(a)) {
					matchingNTables.add(n);
					matchingATables.add(a);
					break;
				}

		if (matchingNTables.size() < necessaryTables.size()) {
			necessaryTables.removeAll(matchingNTables);
			throw new Exception(
					"Did not find all necessary tables. Missing tables: "
							+ necessaryTables);
		}

		else if (matchingATables.size() < actualTables.size()) {
			actualTables.removeAll(matchingATables);
			throw new Exception("Found unnecessary tables: " + actualTables);
		}
	}

	/**
	 * @return A list of table names for all the classes returned by
	 *         {@link #getEntityClasses()}
	 */
	private static Collection<String> getNecessaryTableNames() {
		Set<String> necessaryTableNames = new HashSet<String>();
		for (Class<? extends RawEntity<?>> clazz : getEntityClasses())
			// We don't need a table for a polymorphic class
			if (clazz.getAnnotation(Polymorphic.class) == null)
				necessaryTableNames.add(manager().getTableNameConverter()
						.getName(clazz));
		return necessaryTableNames;
	}

	/**
	 * @return A list of all entity classes referenced in our database
	 */
	@SuppressWarnings("unchecked")
	private static Class<? extends RawEntity<?>>[] getEntityClasses() {
		List<Class<? extends RawEntity<?>>> list = new ArrayList<Class<? extends RawEntity<?>>>();

		list.add(ActiveMatrixRelation.class);
		list.add(Config.class);
		list.add(Feature.class);
		list.add(Iteration.class);
		list.add(Project.class);
		list.add(ProjectMember.class);
		list.add(Release.class);
		list.add(Requirement.class);
		list.add(TraceabilityAssociation.class);
		list.add(TraceabilityMatrix.class);
		list.add(TraceabilityMatrixCriteria.class);
		list.add(TraceabilityMatrixCriterion.class);
		list.add(User.class);
		list.add(UserStory.class);

		Class<? extends RawEntity<?>>[] arr = (Class<? extends RawEntity<?>>[]) Array
				.newInstance(Class.class, list.size());

		int index = 0;
		for (Class<? extends RawEntity<?>> clazz : list)
			arr[index++] = clazz;

		return arr;
	}

	/**
	 * Drops all tables which are needed by this program, then migrates the
	 * database with a fresh set of those tables
	 * 
	 * @throws SQLException
	 *             If anything goes wrong in accessing the database
	 * @throws IOException
	 *             If anything goes wrong in accessing the database
	 *             configuration file
	 */
	public static void purge() throws SQLException, IOException {
		dropTables();
		migrateTables();
	}

	/**
	 * Drops all the tables which are needed by this program (ie, the
	 * intersection of {@link #getActualTableNames()} and
	 * {@link #getNecessaryTableNames()}. This is somewhat inefficient, since
	 * instead of trying to figure out dependencies (for which tables to drop
	 * first - they often reference each other with foreign keys), this method
	 * just tries for as many times as there are tables before giving up and
	 * throwing an exception about it.
	 * 
	 * @throws SQLException
	 *             If anything goes wrong in accessing the database
	 */
	public static void dropTables() throws SQLException {
		Collection<String> actualTables = getActualTableNames();
		Collection<String> necessaryTables = getNecessaryTableNames();

		List<String> toDrop = new ArrayList<String>();

		for (String a : actualTables)
			for (String n : necessaryTables)
				if (a.equalsIgnoreCase(n)) {
					toDrop.add(a);
					break;
				}
		java.sql.Connection conn = manager().getProvider().getConnection();

		try {
			Statement stmt = conn.createStatement();
			try {
				for (int i = toDrop.size(); i > 0 && !toDrop.isEmpty(); i--) {
					List<String> erroredDrops = new ArrayList<String>();
					for (String tableName : toDrop)
						try {
							stmt.execute("DROP TABLE " + tableName);
						} catch (SQLException e) {
							erroredDrops.add(tableName);
						}
					toDrop = erroredDrops;
				}

				if (!toDrop.isEmpty())
					throw new SQLException("Could not drop tables");
			} finally {
				stmt.close();
			}
		} finally {
			conn.close();
		}
	}

	/**
	 * Attempts to retrieve a list of all of the table names currently in the
	 * database.
	 * 
	 * @return A collection of all table names actually found in the database
	 * @throws SQLException
	 *             If anything goes wrong in accessing the database
	 */
	public static Collection<String> getActualTableNames() throws SQLException {
		List<String> names = new ArrayList<String>();

		java.sql.Connection conn = manager().getProvider().getConnection();
		try {
			ResultSet rs = manager().getProvider().getTables(conn);
			try {
				while (rs.next())
					names.add(rs.getString("TABLE_NAME"));
			} finally {
				rs.close();
			}
		} finally {
			conn.close();
		}

		return names;
	}
}