/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *    Ian Williams - Additional design and implementation (non-persistence)
 *******************************************************************************/

package edu.wpi.wpisuite.database;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import edu.wpi.wpisuite.components.editor.InvalidInputException;

/**
 * This singleton provides a way to access the configuration file that reads
 * database info without actually needing to deal with file IO.
 * 
 * Use {@link #instance()} to access the single instance of this class
 */
public class DatabaseConfig {
	/** The filename of the configuration file to use */
	public static final String CONFIG_FILE = "database_config.ini";

	private String username;
	private String password;
	private String url;
	private boolean useFile = true;

	private DatabasePasswordCipher passwordCipher;

	/**
	 * Gets the username as set by the last reading of the config file
	 * 
	 * @return The username for the configured database
	 */
	public String username() {
		if (this.username == null)
			return "";
		return this.username;
	}

	/**
	 * Gets the password as set by the last reading of the config file
	 * 
	 * @return The password for the configured database
	 */
	public String password() {
		if (this.password == null)
			return "";
		return this.password;
	}

	/**
	 * Gets the url as set by the last reading of the config file
	 * 
	 * @return The url for the configured database
	 */
	public String url() {
		if (this.url == null)
			return "";
		return this.url;
	}

	/**
	 * Resets the configuration file to the given parameters.
	 * 
	 * @throws IOException
	 *             If the Database config file location cannot be written to.
	 */
	public void setConfig(String username, String password, String url)
			throws IOException {
		this.url = url;
		this.password = password;
		this.username = username;

		if (useFile) {
			BufferedWriter outputStream = new BufferedWriter(new FileWriter(
					DatabaseConfig.CONFIG_FILE));

			outputStream.write("username=");
			outputStream.write(username);
			outputStream.write("\r\n");

			outputStream.write("password=");
			outputStream.write(passwordCipher.encrypt(password));
			outputStream.write("\r\n");

			outputStream.write("url=");
			outputStream.write(url);
			outputStream.write("\r\n");

			outputStream.close();
		}
	}

	/**
	 * Sets this object up with the current version of the config file, such
	 * that {@link #username()}, {@link #password()}, and {@link #url()} will
	 * work.
	 * 
	 * The config file requires only that there exist lines of the following
	 * forms. Any line NOT of any of these forms is treated as a comment and
	 * ignored. If any of the following forms do not appear in the file, or if
	 * any line appears more than once, an IOException is thrown.
	 * 
	 * username=some_username password=a_password url=mysql://someurl.com
	 * 
	 * @throws IOException
	 *             If the Database config file does not exist, or if it is
	 *             incorrectly formatted
	 */
	public void getConfig() throws IOException {
		if (!useFile)
			return;
		BufferedReader inputStream = new BufferedReader(new FileReader(
				DatabaseConfig.CONFIG_FILE));

		String tempUsername = null;
		String tempPassword = null;
		String tempUrl = null;

		String line;

		// Go through each line, ignoring those which are not formatted
		// correctly.
		// We will allow these lines as comments.
		// However, we WILL require that all settings be set prior to actually
		// returning without an error
		while ((line = inputStream.readLine()) != null) {
			int eqIndex = line.indexOf('=');
			if (eqIndex == -1)
				continue; // invalid formatting

			String[] parts = { line.substring(0, eqIndex),
					line.substring(eqIndex + 1) };

			if (parts[0].equals("username")) {
				if (tempUsername != null)
					throw new IOException(
							"Database config file contains the field 'username' multiple times.");
				tempUsername = parts[1];
			} else if (parts[0].equals("password")) {
				if (tempPassword != null)
					throw new IOException(
							"Database config file contains the field 'password' multiple times.");
				tempPassword = passwordCipher.decrypt(parts[1]);
			} else if (parts[0].equals("url")) {
				if (tempUrl != null)
					throw new IOException(
							"Database config file contains the field 'url' multiple times.");
				tempUrl = parts[1];
			}
		}

		inputStream.close();

		// Make sure we have all of them. This keeps things atomic.
		if (tempUsername == null || tempPassword == null || tempUrl == null)
			throw new IOException(
					"Database config file does not contain all necessary fields in the correct format.");

		this.username = tempUsername;
		this.password = tempPassword;
		this.url = tempUrl;
	}

	// Setting up the singleton
	/** The single static instance for all to use */
	private static DatabaseConfig INSTANCE;

	/**
	 * Gets the single, static instance of the data from the database
	 * configuration file.
	 * 
	 * @return The sole instance of DatabaseConfig for all to use.
	 */
	public static DatabaseConfig instance() {
		if (DatabaseConfig.INSTANCE == null)
			DatabaseConfig.initInstance();
		return DatabaseConfig.INSTANCE;
	}

	/**
	 * Only initInstance should be using this, so private.
	 */
	private DatabaseConfig() {
		passwordCipher = new DatabasePasswordCipher();
	}

	/**
	 * Instantiates the single instance of this class, should it not already
	 * exist.
	 */
	private static void initInstance() {
		DatabaseConfig config = new DatabaseConfig();
		DatabaseConfig.INSTANCE = config;
	}

	public void validateConfig(String username, String password, String url)
			throws InvalidInputException {
		if (isBlank())
			throw new InvalidInputException(
					"No database details have been set.");

		try {
			Database.testConnection(username, password, url);
		} catch (Exception e) {
			throw new InvalidInputException(
					"Either the provided database details are invalid or no connection to the given url could be found.");
		}
	}

	public void setUseFile(boolean b) {
		useFile = b;
	}

	public void validateConfig() throws InvalidInputException {
		validateConfig(username(), password(), url());
	}

	public boolean isBlank() {
		return (username() == null || username().equals(""))
				&& (password() == null || password().equals(""))
				&& (url() == null || url().equals(""));
	}

}
