/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.database;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Method;
import java.sql.Blob;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

import net.java.ao.EntityManager;
import net.java.ao.types.DatabaseType;

/**
 * A data type to explain to ActiveObjects how to convert between int[]s and
 * database BLOBs (mostly by intermediate conversion to byte[])
 */
public class IntArrayType extends DatabaseType<int[]> {
	public IntArrayType() {
		super(Types.BLOB, -1, int[].class);
	}

	private byte[] intsToBytes(int[] array) {
		byte[] bytes = new byte[array.length * 4];

		for (int i = 0; i < array.length; i++) {

			bytes[4 * i] = ((byte) (array[i] >> 24));
			bytes[4 * i + 1] = ((byte) (array[i] >> 16));
			bytes[4 * i + 2] = ((byte) (array[i] >> 8));
			bytes[4 * i + 3] = ((byte) (array[i] >> 0));

		}

		return bytes;
	}

	private int[] bytesToInts(byte[] array) {
		ByteArrayInputStream bis = new ByteArrayInputStream(array);
		DataInputStream in = new DataInputStream(bis);

		int[] ints = new int[array.length / 4];
		try {
			for (int i = 0; i < ints.length; i++)
				ints[i] = in.readInt();
		} catch (Exception e) {
			// nothing
		}

		return ints;
	}

	@Override
	public int[] pullFromDatabase(EntityManager manager, ResultSet res,
			Class<? extends int[]> type, String field) throws SQLException {
		if (manager.getProvider().getURI().startsWith("jdbc:postgres"))
			return bytesToInts(res.getBytes(field));

		Blob blob = res.getBlob(field);

		return bytesToInts(blob.getBytes(1, (int) blob.length()));
	}

	@Override
	public void putToDatabase(int index, PreparedStatement stmt, int[] value)
			throws SQLException {
		InputStream is = null;

		is = new ByteArrayInputStream(intsToBytes(value));

		boolean handled = false;
		try { // for Java 6
			Method m = stmt.getClass().getMethod("setBinaryStream", int.class,
					InputStream.class);
			m.invoke(stmt, index, is);

			handled = true;
		} catch (Throwable t) {
		}

		if (!handled)
			try {
				stmt.setBinaryStream(index, is, is.available());
			} catch (IOException e) {
				SQLException e2 = new SQLException(e.getMessage());
				e2.initCause(e);

				throw e2;
			}
	}

	@Override
	public Object defaultParseValue(String value) {
		if (value.equalsIgnoreCase("null"))
			return null;

		throw new IllegalArgumentException(
				"Cannot assign a String representation to a BLOB");
	}

	@Override
	public String valueToString(Object value) {
		if (value == null)
			return null;

		throw new IllegalArgumentException(
				"Cannot assign a String representation to a BLOB");
	}

	@Override
	public String getDefaultName() {
		return "BLOB";
	}
}
