/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.module;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashSet;
import java.util.Set;

import edu.wpi.wpisuite.data.Project;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.module.ActiveStateEvent.ActiveStateEventType;
import edu.wpi.wpisuite.module.Status.StatusType;

/**
 * This class contains the active state of a running GUI.
 * 
 * It allows the registration of ActiveStateListeners to deal with keeping
 * things updated on state change. It also contains several versions of
 * setStatus, for setting the global status of the program
 */
public class ActiveState implements PropertyChangeListener, StatusState {
	/** The user which is logged in, or null if there isn't one */
	private User activeUser;
	/** The currently active project, or null if none are selected */
	private Project activeProject;
	/** The currently active module, or null if none are active */
	private IActiveModuleController moduleController;

	/** All listeners which are currently registered to this object */
	private Set<ActiveStateListener> listeners;

	/** ESP: final instance guarantees thread safe singleton */
	private static final ActiveState INSTANCE = new ActiveState();

	public static ActiveState getInstance() {
		return INSTANCE;
	}
	
	/**
	 * Initializes the state with no listeners and null'd active user and
	 * project
	 */
	private ActiveState() {
		this.activeUser = null;
		this.activeProject = null;
		this.moduleController = null;
		this.listeners = new HashSet<ActiveStateListener>();
	}

	/**
	 * Registers an ActiveStateListener to be informed of any changes to the
	 * state this object maintains.
	 * 
	 * @param listener
	 *            The object to be activated on state change
	 */
	public void registerListener(ActiveStateListener listener) {
		this.listeners.add(listener);
	}

	/**
	 * Unregisters a previously registered ActiveStateListener so that it will
	 * no longer be called on state updates.
	 * 
	 * @param listener
	 *            The previously activated listener to unregister
	 */
	public void unregisterListener(ActiveStateListener listener) {
		this.listeners.remove(listener);
	}

	/**
	 * Returns the currently active (ie, logged-in) user.
	 * 
	 * This State object listens to any changes in this User and will propogate
	 * all updates to any other listeners registered on this object.
	 * 
	 * @return The active user
	 */
	public User getUser() {
		return this.activeUser;
	}

	/**
	 * Sets the currently active, logged-in user to the given one.
	 * 
	 * Also sets up listeners to watch for updates to this user, should the new
	 * user not be null.
	 * 
	 * @param u
	 *            The new active user
	 */
	public void setUser(User u) {
		if (this.objectsEqual(u, this.activeUser))
			return;
		
		setUserHelper(u);
		this.fireStateChange(ActiveStateEventType.USER);
	}
	
	private void setUserHelper(User u) {

		// Unregister old listener
		if (this.activeUser != null)
			this.activeUser.removePropertyChangeListener(this);

		this.activeUser = u;

		// Register new listener
		if (this.activeUser != null)
			this.activeUser.addPropertyChangeListener(this);
	}

	/**
	 * Returns the currently active (selected) project.
	 * 
	 * This State object listens to any changes in this Project and will
	 * propogate all updates to any other listeners registered on this object.
	 * 
	 * @return The active project
	 */
	public Project getProject() {
		return this.activeProject;
	}

	/**
	 * Sets the currently active, selected project to the given one.
	 * 
	 * Also sets up listeners to watch for updates to this project, should the
	 * new project not be null.
	 * 
	 * @param p
	 *            The new active project
	 */
	public void setProject(Project p) {
		if (this.objectsEqual(p, this.activeProject))
			return;
		
		setProjectHelper(p);
		this.fireStateChange(ActiveStateEventType.PROJECT);
	}
	
	private void setProjectHelper(Project p) {
		// Unregister old listener
		if (this.activeProject != null)
			this.activeProject.removePropertyChangeListener(this);
	
		this.activeProject = p;
		// Register new listener
		if (this.activeProject != null) {
			this.activeProject.addPropertyChangeListener(this);
			if (this.activeUser != null) {
				this.activeUser.removePropertyChangeListener(this);
				this.activeUser.setLastOpenProject(p);
				this.activeUser.save();
				this.activeUser.addPropertyChangeListener(this);
			}
		}
	}
	
	/**
	 * Sets the currently active project and user. This is more efficient than
	 * setting them separately, since it results in only 1 state change.
	 * 
	 * @param p
	 *            The new active project
	 * @param u
	 *            The new active user
	 */
	public void setState(User u, Project p) {
		boolean newState = false; 
		
		if (!this.objectsEqual(p, this.activeProject)) {
			setProjectHelper(p);
			newState = true;
		}
		if (!this.objectsEqual(u, this.activeUser)) {
			setUserHelper(u);
			newState = true;
		}
		
		if (newState)
			this.fireStateChange(ActiveStateEventType.PROJECT);
	}
	
	
	/**
	 * Returns the currently active module.
	 * 
	 * @return The active module
	 */
	public Module getModule() {
		return this.moduleController.getActiveModule();
	}

	/**
	 * Sets the currently active module
	 * 
	 * @param m
	 *            The new active module
	 */
	public void setModuleController(IActiveModuleController m) {
		this.moduleController = m;
	}
	
	/**
	 * Informs all registered listeners of a state change of the given type
	 * 
	 * @param The
	 *            type of change to trigger an event for
	 */
	public void fireStateChange(ActiveStateEventType type) {
				
		ActiveStateEvent evt = new ActiveStateEvent(this, type);
		for (ActiveStateListener l : this.listeners)
			l.stateChange(evt);
	}

	/**
	 * Determines equality of two objects, either of which may be null
	 * 
	 * @return Whether params are equal
	 */
	private boolean objectsEqual(Object a, Object b) {
		if (a == null)
			return b == null;
		if (b == null)
			return false; // we already know a isn't

		return a.equals(b);
	}

	/**
	 * NOT PART OF THE PUBLIC API.
	 * 
	 * ActiveObjects calls this whenever the activeProject or activeUser is
	 * updated - since they are mutable, merely doing this on this class's set
	 * methods is not enough on its own.
	 */
	@Override
	public void propertyChange(PropertyChangeEvent evt) {
		if (evt.getSource() instanceof Project)
			this.fireStateChange(ActiveStateEventType.PROJECT);
		else if (evt.getSource() instanceof User)
			this.fireStateChange(ActiveStateEventType.USER);
	}

	/** The objects listening to Status changes. Usually just the StatusBar */
	private Set<StatusListener> statusListeners = new HashSet<StatusListener>();

	/**
	 * Sets the global program Status to the given Status value.
	 * 
	 * @param newStatus
	 *            The new Status to set
	 */
	public void setStatus(Status newStatus) {
		for (StatusListener l : statusListeners)
			l.statusChanged(newStatus);
	}

	/**
	 * Sets the global program Status to the given Status value.
	 * 
	 * @param statusMessage
	 *            The message to define the new status with
	 */
	public void setStatus(String statusMessage) {
		setStatus(new Status(statusMessage));
	}

	/**
	 * Sets the global program Status to the given Status value.
	 * 
	 * @param statusMessage
	 *            The message to define the new status with
	 * @param statusType
	 *            The type to define the Status as
	 */
	public void setStatus(String statusMessage, StatusType statusType) {
		setStatus(new Status(statusMessage, statusType));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see module.StatusState#registerStatusListener(module.StatusListener)
	 */
	public void registerStatusListener(StatusListener l) {
		statusListeners.add(l);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see module.StatusState#removeStatusListener(module.StatusListener)
	 */
	public void removeStatusListener(StatusListener l) {
		statusListeners.remove(l);
	}

}
