/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Ian Williams - Initial design, API, and implementation
 *    Lucas Scotta - Initial design, API, and implementation
 *    Dan Bjorge - Additional design and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.module;

/**
 * Each WPISuite module is a fully functional, self-contained JPanel that is
 * displayed in WPISuite's tabbedPane The Module class defines the functionality
 * of a module The core module code should be found in this class, and it should
 * be called by ModuleView
 * 
 * Though it cannot be explicitly required in java, all Modules are REQUIRED to
 * provide a constructor which takes in a single argument for the ActiveState of
 * the program and initializes its own ModuleView.
 * 
 * @see ModuleView
 */
public abstract class Module {
	/** The main application state */
	private ActiveState state;

	/**
	 * Initializes the Module at program startup with an instance of the calling
	 * application
	 * 
	 * @param wpiSuite
	 *            The main calling application
	 */
	public Module(ActiveState state) {
		this.state = state;
	}

	/**
	 * Most modules will probably want to create a listener on this object.
	 * 
	 * @return The current active state of the application-wide variables
	 */
	public ActiveState getState() {
		return this.state;
	}

	/**
	 * @return The name of this object. This will be used in menu items and in
	 *         tab titles.
	 */
	public abstract String getModuleName();

	/**
	 * @return The preferred keyboard shortcut to use in menu items referencing
	 *         this module's name.
	 */
	public abstract int getModuleKeyShortcut();

	/**
	 * @return The description of the menu item for this module
	 */
	public String getMenuDescription() {
		return "Start the " + this.getModuleName() + " module";
	}

	/**
	 * @return The view defining how this module looks
	 */
	public abstract ModuleView getView();

	/**
	 * This method defines when the module in question should be marked as
	 * available for use. By default, modules are marked as available at all
	 * times - a module might wish to override this with such criteria as
	 * "the active project is not null", for example.
	 * 
	 * @return Whether this module should be enabled for use at the moment of
	 *         calling, usually based on current application state
	 */
	public boolean shouldBeVisible() {
		return true;
	}

	/**
	 * This method defines when the module in question should be marked as
	 * visible in the application. By default, modules are marked as visible at
	 * all times - a module might wish to override this with such criteria as
	 * "the active user is a system admin", for example.
	 * 
	 * Note that this method does NOT determine which module is actually visible
	 * at a given moment in the main application view, merely whether it should
	 * be visible to the user as a choice to look at in that view
	 * 
	 * @return Whether this module should be visible for use at the moment of
	 *         calling, usually based on current application state
	 */
	public boolean shouldBeEnabled() {
		return true;
	}
	
}
