/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Ian Williams - Initial design, API, and implementation
 *    Lucas Scotta - Initial design, API, and implementation
 *    Dan Bjorge - Additional design and implementation
 *******************************************************************************/
package edu.wpi.wpisuite.module;

import java.util.ArrayList;
import java.util.List;

import javax.swing.ImageIcon;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import edu.wpi.wpisuite.components.EntityIconSet;

/**
 * Each WPISuite module is a fully functional, self-contained JPanel that is
 * displayed in WPISuite's tabbedPane This class defines the swing components
 * and layout of the module
 * 
 * @see Module
 */
public abstract class ModuleView extends JPanel {

	private static final long serialVersionUID = 500953652488932082L;
	private Module module;
	private ImageIcon icon;
	protected EntityIconSet icons = new EntityIconSet();
	protected List<JMenuItem> addMenuItems = new ArrayList<JMenuItem>();

	/**
	 * Generic constructor to assign the module this object is a view of.
	 * Automatically calls initAddMenuItems()
	 * 
	 * @param module
	 *            The Module that controls this ModuleView
	 */
	public ModuleView(Module module) {
		this.module = module;
		initAddMenuItems();
	}

	/**
	 * Constructor to assign the module this object is a view of and give it an
	 * icon. Automatically calls initAddMenuItems()
	 * 
	 * @param module
	 *            The Module that controls this ModuleView
	 * @param icon
	 *            The icon for this ModuleView
	 */
	public ModuleView(Module module, ImageIcon icon) {
		this.module = module;
		this.icon = icon;
		initAddMenuItems();
	}

	/**
	 * @return The module this object is a view for
	 */
	public Module getModule() {
		return this.module;
	}

	/**
	 * ModuleViews should use this function as a master updating function for
	 * the components of his panel. WPISuite will call this function whenever
	 * this module is viewed by the user.
	 */
	public abstract void updateAllComponents();

	/**
	 * This is called whenever this module's view is added to a tab pane in the
	 * main application.
	 * 
	 * Most ModuleViews will want to override this - the base implementation
	 * does nothing.
	 */
	public void onAddToTabPane() {
	}

	/**
	 * Returns all menu items which should be added to the global "Add" menu for
	 * the sake of this module. Handlers should be assigned to them, though a
	 * handler to select the setting ModuleView will be added automatically.
	 * 
	 * @return The list of Add menu items this module would like to appear in
	 *         WPISuite's Add menu
	 */
	public List<JMenuItem> getAddMenuItems() {
		return addMenuItems;
	}

	/**
	 * Returns the same thing of getAddMenuItems(), but only calls that method
	 * once, and then stores the results so you get the same references every
	 * time
	 * 
	 * @return The list of Add menu items to insert (defaults to none)
	 */
	public abstract void initAddMenuItems();

	/**
	 * Displays a popup representing an error
	 * 
	 * @param message
	 *            The error message to display
	 */
	public void popErrorMessage(String message) {
		JOptionPane.showMessageDialog(null, message);
	}

	/**
	 * Displays a popup representing an error
	 * 
	 * @param message
	 *            The Exception whose message to display
	 */
	public void popErrorMessage(Exception e) {
		this.popErrorMessage(e.getMessage());
	}

	/**
	 * Sets the icon for this Module.
	 * 
	 * @param icon
	 *            The ImageIcon to set.
	 */
	public void setIcon(ImageIcon icon) {
		this.icon = icon;
	}

	/**
	 * Gets the icon for this Module.
	 * 
	 * @return icon The icon for this Module.
	 */
	public ImageIcon getIcon() {
		return this.icon;
	}
}
