/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Mihai Tiberiu Luca
 *    Mihajlo Zeljkovic
 *    Bryan Crabtree
 *    Dan Bjorge
 *    Ian Williams
 *    Lucas Scotta
 *******************************************************************************/

package edu.wpi.wpisuite.module.dashboard;

import java.awt.event.KeyEvent;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;

import net.java.ao.Query;

import org.apache.log4j.Logger;

import edu.wpi.wpisuite.data.Project;
import edu.wpi.wpisuite.data.ProjectMember;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.data.UserStory;
import edu.wpi.wpisuite.data.util.ProjectComparator;
import edu.wpi.wpisuite.database.Database;
import edu.wpi.wpisuite.module.ActiveState;
import edu.wpi.wpisuite.module.Module;
import edu.wpi.wpisuite.module.ModuleView;

/**
 * Defines a module which let the user manipulate projects
 */
public class Dashboard extends Module {

	Logger logger = Logger.getLogger(this.getClass());
	private DashboardView view;
	final public static String moduleName = "Dashboard";

	public Dashboard(ActiveState state) {
		super(state);

		this.view = new DashboardView(this);

	}

	@Override
	public int getModuleKeyShortcut() {
		return KeyEvent.VK_D;
	}

	@Override
	public String getModuleName() {
		return moduleName;
	}

	@Override
	public ModuleView getView() {
		return this.view;
	}

	/**
	 * Gets the list of projects the current user is a member of. If there is no
	 * current user, returns an empty array.
	 * 
	 * @return The list of projects for the current user
	 */
	public Project[] getProjectList() {
		User curUser = this.getState().getUser();

		if (curUser == null) {
			logger.error("Current user is null");
			return new Project[0];
		} else {
			Project[] projects = curUser.getProjects();
			Arrays.sort(projects, new ProjectComparator());
			return projects;
		}
	}

	public ArrayList<Project> getAllOtherProjects() {
		Project[] allprojects = null;
		Project[] userProjects = null;

		if (this.getState().getUser() != null)
			userProjects = this.getState().getUser().getProjects();
		else {
			logger.error("Current user is null");
			return null;
		}

		ArrayList<Project> allOtherProjects = new ArrayList<Project>();
		try {
			Query query = Query.select().order("name ASC");

			if (userProjects.length != 0)
				query = query.where("id NOT IN ("
						+ Database.questionMarks(userProjects.length) + ")",
						Arrays.asList(userProjects).toArray(new Object[0]));

			allprojects = Database.manager().find(Project.class, query);
			allOtherProjects.addAll(Arrays.asList(allprojects));
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return allOtherProjects;
	}

	/**
	 * Generates a new, blank project with some default fields set.
	 * 
	 * @return The new project
	 * @throws SQLException
	 *             If any database error occurs
	 */
	public Project createProject() throws SQLException {
		try {
			Project newProject = Database.manager().create(Project.class);
			newProject.setName(""/* generateNewProjectName() */);
			newProject.setDescription("Description");
			newProject.save();

			ProjectMember newAssoc = Database.manager().create(
					ProjectMember.class);
			newAssoc.setMember(this.getState().getUser());
			newAssoc.setProject(newProject);
			newAssoc.save();

			this.getState().setProject(newProject);

			return newProject;
		} catch (SQLException e) {
			throw new SQLException(
					"An error occured while creating the project. Contact your Administrator.");
		}
	}

	/**
	 * Sets the current user's active project to p
	 * 
	 * @param p
	 */
	public void setActiveProject(Project p) {
		getState().setProject(p);
	}

	/**
	 * Adds the current user to a project
	 * 
	 * @param evt
	 */
	protected void joinProject(Project p) {
		User u = getState().getUser();

		if (u != null) {
			ProjectMember association;

			try {
				ProjectMember[] assocs = Database.manager().find(
						ProjectMember.class, "memberID = ? and projectID = ?",
						u, p);
				if (assocs.length == 0) {
					association = Database.manager()
							.create(ProjectMember.class);
					association.setProject(p);
					association.setMember(u);
					association.save();
				}
			} catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			view.updateAllComponents();
		}
	}

	/**
	 * Adds the current user to projects. Right now we only allow 1 project to
	 * be added at a time as a matter of policy, but this is here to enable the
	 * possibility.
	 * 
	 * @param projects
	 */
	protected void joinProjects(Collection<Project> projects) {
		for (Project p : projects)
			joinProject(p); // TODO this will call updateAllComponents each
		// time.. kind of silly
	}

	/**
	 * Remove a user from a project
	 * 
	 * @param evt
	 */
	protected void leaveProject(Project p) {

		User u = getState().getUser();

		if (u != null) {
			ProjectMember[] associations;
			UserStory[] userStories;
			try {
				associations = Database.manager().find(ProjectMember.class,
						"projectID = ? AND memberID = ?", p, u);
				userStories = Database.manager().find(UserStory.class,
						"ownerID = ? AND projectID = ?", u, p);

				for (int i = 0; i < userStories.length; i++) {
					userStories[i].setOwner(null);
					userStories[i].save();
				}

				Database.manager().delete(associations);

			} catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			// If they left their active project, set active project to null
			if (p == getState().getProject()) {
				getState().setProject(null);
				u.removePropertyChangeListener(getState());
				u.setLastOpenProject(null);
				u.save();
				u.addPropertyChangeListener(getState());
			}
			view.updateAllComponents();
		}
	}

	/**
	 * Removes the current user from projects. Right now we only allow 1 project
	 * to be left at a time as a matter of policy, but this is here to enable
	 * the possibility.
	 * 
	 * @param projects
	 */
	protected void leaveProjects(Collection<Project> projects) {
		for (Project p : projects)
			leaveProject(p); // TODO this will call updateAllComponents each
		// time.. kind of silly
	}
}
