/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Mihai Tiberiu Luca
 *    Mihajlo Zeljkovic
 *    Bryan Crabtree
 *    Dan Bjorge
 *    Ian Williams
 *    Lucas Scotta
 *******************************************************************************/

package edu.wpi.wpisuite.module.dashboard;

import java.awt.Color;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.DropMode;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.swing.ListSelectionModel;
import javax.swing.TransferHandler;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;



import edu.wpi.wpisuite.components.EntityList;
import edu.wpi.wpisuite.components.EntityListCellRenderer;
import edu.wpi.wpisuite.components.EntityListModel;
import edu.wpi.wpisuite.components.transfer.EntityListTransferHandler;
import edu.wpi.wpisuite.data.Project;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.data.util.UserComparator;
import edu.wpi.wpisuite.module.ActiveStateEvent;
import edu.wpi.wpisuite.module.ActiveStateListener;
import edu.wpi.wpisuite.module.ModuleView;

/**
 * A ModuleView for the Dashboard Module
 * 
 * @see Dashboard
 */
@SuppressWarnings("serial")
public class DashboardView extends ModuleView {

	private Dashboard dashboard;
	private Project selectedProject;

	/**
	 * Constructor
	 * 
	 * @param dashboard
	 */
	public DashboardView(Dashboard dashboard) {
		super(dashboard, new ImageIcon("resources/OtherIcons/Dashboard.PNG"));
		this.dashboard = dashboard;
		this.initComponents();
		this.initOthers();
		this.initProjectList();
		this.initListeners();
	}

	/**
	 * Sets the project list to be modeled by our custom handler object
	 * 
	 * @see ProjectListModel
	 */
	private void initProjectList() {
		this.projectsList.setDragEnabled(true);
		this.projectsList.setDropMode(DropMode.INSERT);
		this.projectsList
				.setTransferHandler(new EntityListTransferHandler<Project>(
						Project.class) {

					@SuppressWarnings("unchecked")
					@Override
					public boolean canImport(
							TransferHandler.TransferSupport support) {

						if (!super.canImport(support))
							return false;

						// no duplicate values
						List<Project> transferEntities;
						try {
							Object data = support.getTransferable()
									.getTransferData(this.supportedDataFlavor);
							transferEntities = (List<Project>) data;
						} catch (UnsupportedFlavorException e) {
							return false;
						} catch (IOException e) {
							return false;
						}

						EntityListModel<Project> model = ((EntityList<Project>) support
								.getComponent()).getModel();
						List<Project> listEntities = model.getAllEntities();
						if (listEntities.containsAll(transferEntities))
							return false;

						return true;
					}

					@Override
					public void postImport(
							TransferHandler.TransferSupport support,
							List<Project> data) {
						dashboard.joinProjects(data);
					}

				});

		this.allProjectsList.setDragEnabled(true);
		this.allProjectsList.setDropMode(DropMode.INSERT);
		this.allProjectsList.setCellRenderer(new EntityListCellRenderer());
		this.allProjectsList
				.setTransferHandler(new EntityListTransferHandler<Project>(
						Project.class) {

					@SuppressWarnings("unchecked")
					@Override
					public boolean canImport(
							TransferHandler.TransferSupport support) {

						if (!super.canImport(support))
							return false;

						// no duplicate values
						List<Project> transferEntities;
						try {
							transferEntities = (List<Project>) support
									.getTransferable().getTransferData(
											this.supportedDataFlavor);
						} catch (UnsupportedFlavorException e) {
							return false;
						} catch (IOException e) {
							return false;
						}
						EntityListModel<Project> model = ((EntityList<Project>) support
								.getComponent()).getModel();
						List<Project> listEntities = model.getAllEntities();
						if (listEntities.containsAll(transferEntities))
							return false;

						return true;
					}

					@Override
					public void postImport(
							TransferHandler.TransferSupport support,
							List<Project> data) {
						dashboard.leaveProjects(data);
					}

				});

	}

	private void initOthers() {

		projectsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		allProjectsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		descriptionTextArea.setEditable(false);
		descriptionTextArea.setLineWrap(true);
		descriptionTextArea.setWrapStyleWord(true);
		titleTextField.setEditable(false);
		titleTextField.setBackground(Color.white);
		selectedProject = null;
		projectUsersList.setCellRenderer(new EntityListCellRenderer());
	}

	/**
	 * Initializes the listeners for objects in the control panel
	 */
	private void initListeners() {

		// Sets selected project and clears other project list with each
		// selection event for my project list
		projectsList.addListSelectionListener(new ListSelectionListener() {

			@Override
			public void valueChanged(ListSelectionEvent arg0) {
				setSelectedProject(projectsList.getSelectedEntity());
				allProjectsList.clearSelection();
			}
		});

		// Sets selected project and clears other project list with each
		// selection event for all project list
		allProjectsList.addListSelectionListener(new ListSelectionListener() {

			@Override
			public void valueChanged(ListSelectionEvent arg0) {
				setSelectedProject(allProjectsList.getSelectedEntity());
				projectsList.clearSelection();
			}
		});

		// Double click sets the project to be active
		projectsList.addMouseListener(new MouseAdapter() {

			@Override
			public void mouseClicked(MouseEvent arg0) {
				if (arg0.getClickCount() == 2) {
					setActiveProject();
				}
			}
		});

		// register listener on the active state
		this.getModule().getState().registerListener(new ActiveStateListener() {
			@Override
			public void stateChange(ActiveStateEvent event) {
				if (dashboard == dashboard.getState().getModule())
					updateAllComponents();
			}
		});

		// Button listener for joining a project
		this.joinProjectButton.addActionListener(new ActionListener() {
			public void actionPerformed(final ActionEvent e) {

				// If they don't have a project selected, yell
				if (allProjectsList.getSelectedIndex() < 0) {
					JOptionPane
							.showMessageDialog(
									DashboardView.this.dashboard.getView(),
									"Please select a project from \"All Projects\" list",
									"Error", JOptionPane.ERROR_MESSAGE);
					return;
				}

				dashboard.joinProject(allProjectsList.getSelectedEntity());
			}
		});
	}

	/**
	 * Updates the list of projects to reflect the current User's projects
	 */
	private void updateProjectsList() {
		Project oldSelection = projectsList.getSelectedEntity();
		int oldSelectionID = -1;
		if (oldSelection != null)
			oldSelectionID = oldSelection.getID();
		int newSelectionIndex = -1;

		Project[] newProjects = this.dashboard.getProjectList();

		this.projectsList.getModel().clear();

		if (newProjects != null) {
			int index = 0;
			for (Project p : newProjects) {
				if (oldSelectionID == p.getID())
					newSelectionIndex = index;
				projectsList.getModel().addElement(p);
				index++;
			}

		}

		this.projectsList.clearFontStyles();
		this.projectsList.setEntityBold(dashboard.getState().getProject());
		this.projectsList.setSelectedIndex(newSelectionIndex);

	}

	/**
	 * Updates the list of projects to reflect the current User's projects
	 */
	private void updateAllProjectList() {
		Project oldSelection = this.allProjectsList.getSelectedEntity();
		int oldSelectionID = -1;
		if (oldSelection != null)
			oldSelectionID = oldSelection.getID();
		int newSelectionIndex = -1;

		ArrayList<Project> newProjects = this.dashboard.getAllOtherProjects();

		allProjectsList.getModel().clear();

		if (newProjects != null) {
			int index = 0;
			for (Project p : newProjects) {
				if (oldSelectionID == p.getID())
					newSelectionIndex = index;
				this.allProjectsList.getModel().addElement(p);
				index++;
			}

		}
		this.allProjectsList.setSelectedIndex(newSelectionIndex);
	}

	/**
	 * Updates the title and description fields to reflect the selected project
	 */
	private void updateProjectDetailsPanel() {
		this.projectUsersList.getModel().clear();
		if (selectedProject != null) {
			this.titleTextField.setText(selectedProject.getName());
			this.descriptionTextArea.setText(selectedProject.getDescription());

			User[] listOfMembers = selectedProject.getMembers();
			Arrays.sort(listOfMembers, new UserComparator());
			for (User u : listOfMembers)
				this.projectUsersList.getModel().addElement(u);
		} else {
			this.titleTextField.setText("");
			this.descriptionTextArea.setText("");
		}
	}

	private void currentProjectButtonActionPerformed(
			java.awt.event.ActionEvent evt) {
		// If no project is selected in the myprojects list, yell at the user
		if (this.projectsList.getSelectedIndex() < 0) {
			JOptionPane.showMessageDialog(this.dashboard.getView(),
					"Please select a project from \"My Projects\" list",
					"Error", JOptionPane.ERROR_MESSAGE);
			return;
		}

		// Otherwise focus on the selected project from the my project list and
		// set the current project
		this.projectsList.requestFocusInWindow();
		setSelectedProject(projectsList.getSelectedEntity());
		setActiveProject();
	}

	private void setActiveProject() {
		this.projectsList.clearFontStyles();
		this.projectsList.setEntityBold(dashboard.getState().getProject());
		dashboard.setActiveProject(projectsList.getSelectedEntity());
	}

	private void setSelectedProject(Project p) {
		selectedProject = p;
		updateProjectDetailsPanel();
	}

	private void leaveProjectButtonActionPerformed(
			java.awt.event.ActionEvent evt) {

		// If they try to leave a project without selecting one, yell
		if (this.projectsList.getSelectedIndex() < 0) {
			JOptionPane.showMessageDialog(this.dashboard.getView(),
					"Please select a project from \"My Projects\" list",
					"Error", JOptionPane.ERROR_MESSAGE);
			return;
		}
		dashboard.leaveProject(projectsList.getSelectedEntity());

	}

	@Override
	public void updateAllComponents() {
		this.updateAllProjectList();
		this.updateProjectsList();
		this.updateProjectDetailsPanel();
	}

	@Override
	public void initAddMenuItems() {
		// Blank by design
	}

	private void initComponents() {

		primarySplitPane = new javax.swing.JSplitPane();
		projectsPanel = new javax.swing.JPanel();
		projectsScrollPane = new javax.swing.JScrollPane();
		projectsList = new EntityList<Project>();
		currentProjectButton = new javax.swing.JButton();
		leaveProjectButton = new javax.swing.JButton();
		verticalSplitPane = new javax.swing.JSplitPane();
		horizontalSplitPane = new javax.swing.JSplitPane();
		projectDetailsPanel = new javax.swing.JPanel();
		titleTextField = new javax.swing.JFormattedTextField();
		titleLabel = new javax.swing.JLabel();
		descriptionLabel = new javax.swing.JLabel();
		descriptionScrollPane = new javax.swing.JScrollPane();
		descriptionTextArea = new javax.swing.JTextArea();
		projectUsersPanel = new javax.swing.JPanel();
		projectUsersScrollPane = new javax.swing.JScrollPane();
		projectUsersList = new EntityList<User>();
		allProjectsPanel = new javax.swing.JPanel();
		allProjectsScrollPane = new javax.swing.JScrollPane();
		allProjectsList = new EntityList<Project>();
		joinProjectButton = new javax.swing.JButton();

		setPreferredSize(new java.awt.Dimension(900, 600));

		primarySplitPane.setDividerLocation(250);
		primarySplitPane.setOneTouchExpandable(true);

		projectsPanel.setBorder(javax.swing.BorderFactory
				.createTitledBorder("My Projects"));

		projectsList.setVisibleRowCount(25);
		projectsScrollPane.setViewportView(projectsList);

		currentProjectButton.setText("Set Current Project");
		currentProjectButton
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						currentProjectButtonActionPerformed(evt);
					}
				});

		leaveProjectButton.setText("Leave Project");
		leaveProjectButton
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						leaveProjectButtonActionPerformed(evt);
					}
				});

		javax.swing.GroupLayout projectsPanelLayout = new javax.swing.GroupLayout(
				projectsPanel);
		projectsPanel.setLayout(projectsPanelLayout);
		projectsPanelLayout
				.setHorizontalGroup(projectsPanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								projectsPanelLayout
										.createSequentialGroup()
										.addComponent(currentProjectButton)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE).addComponent(
												leaveProjectButton)
										.addContainerGap()).addComponent(
								projectsScrollPane,
								javax.swing.GroupLayout.DEFAULT_SIZE, 240,
								Short.MAX_VALUE));
		projectsPanelLayout
				.setVerticalGroup(projectsPanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								javax.swing.GroupLayout.Alignment.TRAILING,
								projectsPanelLayout
										.createSequentialGroup()
										.addComponent(
												projectsScrollPane,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												542, Short.MAX_VALUE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												projectsPanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																currentProjectButton)
														.addComponent(
																leaveProjectButton))));

		primarySplitPane.setLeftComponent(projectsPanel);

		verticalSplitPane.setDividerLocation(250);
		verticalSplitPane.setOneTouchExpandable(true);

		horizontalSplitPane.setDividerLocation(300);
		horizontalSplitPane
				.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
		horizontalSplitPane.setResizeWeight(0.5);
		horizontalSplitPane.setOneTouchExpandable(true);

		projectDetailsPanel.setBorder(javax.swing.BorderFactory
				.createTitledBorder("Project Details"));

		titleLabel.setText("Title");

		descriptionLabel.setText("Description");

		descriptionTextArea.setColumns(20);
		descriptionTextArea.setRows(5);
		descriptionScrollPane.setViewportView(descriptionTextArea);

		javax.swing.GroupLayout projectDetailsPanelLayout = new javax.swing.GroupLayout(
				projectDetailsPanel);
		projectDetailsPanel.setLayout(projectDetailsPanelLayout);
		projectDetailsPanelLayout.setHorizontalGroup(projectDetailsPanelLayout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(
						projectDetailsPanelLayout.createSequentialGroup()
								.addComponent(titleLabel).addContainerGap(354,
										Short.MAX_VALUE)).addComponent(
						titleTextField, javax.swing.GroupLayout.DEFAULT_SIZE,
						374, Short.MAX_VALUE).addGroup(
						projectDetailsPanelLayout.createSequentialGroup()
								.addComponent(descriptionLabel)
								.addContainerGap()).addComponent(
						descriptionScrollPane,
						javax.swing.GroupLayout.Alignment.TRAILING,
						javax.swing.GroupLayout.DEFAULT_SIZE, 374,
						Short.MAX_VALUE));
		projectDetailsPanelLayout
				.setVerticalGroup(projectDetailsPanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								projectDetailsPanelLayout
										.createSequentialGroup()
										.addComponent(titleLabel)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(
												titleTextField,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												javax.swing.GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(descriptionLabel)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(
												descriptionScrollPane,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												206, Short.MAX_VALUE)));

		horizontalSplitPane.setLeftComponent(projectDetailsPanel);

		projectUsersPanel.setBorder(javax.swing.BorderFactory
				.createTitledBorder("Project Users"));

		projectUsersScrollPane.setViewportView(projectUsersList);

		javax.swing.GroupLayout projectUsersPanelLayout = new javax.swing.GroupLayout(
				projectUsersPanel);
		projectUsersPanel.setLayout(projectUsersPanelLayout);
		projectUsersPanelLayout.setHorizontalGroup(projectUsersPanelLayout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addComponent(projectUsersScrollPane,
						javax.swing.GroupLayout.DEFAULT_SIZE, 374,
						Short.MAX_VALUE));
		projectUsersPanelLayout.setVerticalGroup(projectUsersPanelLayout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addComponent(projectUsersScrollPane,
						javax.swing.GroupLayout.DEFAULT_SIZE, 263,
						Short.MAX_VALUE));

		horizontalSplitPane.setRightComponent(projectUsersPanel);

		verticalSplitPane.setRightComponent(horizontalSplitPane);

		allProjectsPanel.setBorder(javax.swing.BorderFactory
				.createTitledBorder("Other Projects"));

		allProjectsScrollPane.setViewportView(allProjectsList);

		joinProjectButton.setText("Join Project");

		javax.swing.GroupLayout allProjectsPanelLayout = new javax.swing.GroupLayout(
				allProjectsPanel);
		allProjectsPanel.setLayout(allProjectsPanelLayout);
		allProjectsPanelLayout.setHorizontalGroup(allProjectsPanelLayout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(
						allProjectsPanelLayout.createSequentialGroup()
								.addComponent(joinProjectButton)
								.addContainerGap()).addComponent(
						allProjectsScrollPane,
						javax.swing.GroupLayout.DEFAULT_SIZE, 237,
						Short.MAX_VALUE));
		allProjectsPanelLayout
				.setVerticalGroup(allProjectsPanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								javax.swing.GroupLayout.Alignment.TRAILING,
								allProjectsPanelLayout
										.createSequentialGroup()
										.addComponent(
												allProjectsScrollPane,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												540, Short.MAX_VALUE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(joinProjectButton)));

		verticalSplitPane.setLeftComponent(allProjectsPanel);

		primarySplitPane.setRightComponent(verticalSplitPane);

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		this.setLayout(layout);
		layout.setHorizontalGroup(layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				primarySplitPane, javax.swing.GroupLayout.DEFAULT_SIZE, 900,
				Short.MAX_VALUE));
		layout.setVerticalGroup(layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				primarySplitPane, javax.swing.GroupLayout.Alignment.TRAILING,
				javax.swing.GroupLayout.DEFAULT_SIZE, 600, Short.MAX_VALUE));
	}// </editor-fold>

	// Variables declaration - do not modify
	private EntityList<Project> allProjectsList;
	private javax.swing.JPanel allProjectsPanel;
	private javax.swing.JScrollPane allProjectsScrollPane;
	private javax.swing.JButton currentProjectButton;
	private javax.swing.JLabel descriptionLabel;
	private javax.swing.JScrollPane descriptionScrollPane;
	private javax.swing.JTextArea descriptionTextArea;
	private javax.swing.JSplitPane horizontalSplitPane;
	private javax.swing.JButton joinProjectButton;
	private javax.swing.JButton leaveProjectButton;
	private javax.swing.JSplitPane primarySplitPane;
	private javax.swing.JPanel projectDetailsPanel;
	private EntityList<User> projectUsersList;
	private javax.swing.JPanel projectUsersPanel;
	private javax.swing.JScrollPane projectUsersScrollPane;
	private EntityList<Project> projectsList;
	private javax.swing.JPanel projectsPanel;
	private javax.swing.JScrollPane projectsScrollPane;
	private javax.swing.JLabel titleLabel;
	private javax.swing.JFormattedTextField titleTextField;
	private javax.swing.JSplitPane verticalSplitPane;
	// End of variables declaration
}
