/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Ian Williams
 *    Bryan Crabtree
 *******************************************************************************/

package edu.wpi.wpisuite.module.projectadmin;

import java.awt.event.KeyEvent;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;

import net.java.ao.Query;

import edu.wpi.wpisuite.components.editor.EditorContainer;
import edu.wpi.wpisuite.components.editor.entity.ProjectEditor;
import edu.wpi.wpisuite.data.Project;
import edu.wpi.wpisuite.data.ProjectMember;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.data.UserStory;
import edu.wpi.wpisuite.database.Database;
import edu.wpi.wpisuite.module.ActiveState;
import edu.wpi.wpisuite.module.ActiveStateEvent;
import edu.wpi.wpisuite.module.ActiveStateListener;
import edu.wpi.wpisuite.module.Module;
import edu.wpi.wpisuite.module.ModuleView;


/**
 * Module for administrating a project - adding/removing users, setting
 * permissions, setting project details, etc.
 */
public class ProjectAdministration extends Module implements EditorContainer {

	ProjectAdministrationView view;
	private ProjectEditor projectEditor;

	public final static String moduleName = "Project Administration";

	public ProjectAdministration(ActiveState state) {
		super(state);

		this.projectEditor = new ProjectEditor(this);
		this.view = new ProjectAdministrationView(this);
		this.view.setEditorView(this.projectEditor.getView());
		this.updateEditor();
	}

	@Override
	public boolean shouldBeEnabled() {
		return true;
	}

	@Override
	public int getModuleKeyShortcut() {
		return KeyEvent.VK_P;
	}

	@Override
	public String getModuleName() {
		return moduleName;
	}

	@Override
	public ModuleView getView() {
		return this.view;
	}

	/**
	 * Triggered when the remove users button is pressed, it removes all
	 * selected users from the project
	 * 
	 * @param evt
	 */
	protected void removeUsersButtonActionPerformed(
			java.awt.event.ActionEvent evt) {
		for (User u : view.getUsersList().getSelectedEntities())
			if (u != null) {
				this.removeUserFromProject(u, this.getState().getProject());
				this.removeUserFromUserStory(u, this.getState().getProject());
			}
		this.view.updateAllComponents();
	}

	/**
	 * Adds the selected users from the all user list to the projects users
	 * 
	 * @param evt
	 */
	protected void addUsersButtonActionPerformed(java.awt.event.ActionEvent evt) {
		for (User u : view.getOtherUsersList().getSelectedEntities()) {
			boolean alreadyInProject = false;
			if (u != null) {
				User[] usersInProject = this.getState().getProject()
						.getMembers();
				for (User user : usersInProject)
					if (u.equals(user)) {
						alreadyInProject = true;
						break;
					}
				if (!alreadyInProject)
					this.addUserToProject(u, this.getState().getProject());
			}
		}
		this.view.updateAllComponents();
	}

	/**
	 * Adds a user to a project
	 * 
	 * @param evt
	 */
	protected void addUserToProject(User u, Project p) {
		if (u != null) {
			ProjectMember association;
			try {
				association = Database.manager().create(ProjectMember.class);
				association.setProject(p);
				association.setMember(u);
				association.save();
			} catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	/**
	 * Adds an array of users to a project
	 * 
	 * @param evt
	 */
	protected void addUsersToProject(User[] users, Project p) {
		if (users != null)
			for (User u : users)
				addUserToProject(u, p);
	}

	/**
	 * Remove a user from a project
	 * 
	 * @param evt
	 */
	protected void removeUserFromProject(User u, Project p) {

		if (u != null) {
			ProjectMember[] associations;
			try {
				associations = Database.manager().find(ProjectMember.class,
						"projectID = ? AND memberID = ?",
						this.getState().getProject(), u);
				Database.manager().delete(associations);
			} catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	/**
	 * Remove a user's ownership from a User Story
	 * 
	 * @param evt
	 */
	protected void removeUserFromUserStory(User u, Project p) {

		if (u != null) {
			UserStory[] userStories = null;
			try {
				userStories = Database.manager().find(UserStory.class,
						"ownerID = ? AND projectID = ?", u, p);
			} catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			for (int i = 0; i < userStories.length; i++) {
				userStories[i].setOwner(null);
				userStories[i].save();
			}
		}
	}

	/**
	 * Removes an array of users from a project
	 * 
	 * @param evt
	 */
	protected void removeUsersFromProjectAndUserStories(User[] users, Project p) {
		if (users != null)
			for (User u : users) {
				removeUserFromProject(u, p);
				removeUserFromUserStory(u, p);
			}
	}

	/**
	 * Returns all users not in the given project
	 * 
	 * @param p
	 *            the project. must not be null
	 * @author Lucas Scotta
	 * @return
	 */
	public ArrayList<User> getAllNonProjectMembers(Project p) {
		User[] projectMembers = p.getMembers();
		// Project[] allNonProjectMembers = null;

		ArrayList<User> nonMembers = new ArrayList<User>();
		try {
			ArrayList<Object> queryArgs = new ArrayList<Object>();
			queryArgs.addAll(Arrays.asList(projectMembers));
			
			User[] result = null;
			
//			Query q = Query.select().order("username").where("id IN (" + Database.questionMarks(projectMembers.length) + ")", queryArgs.toArray());
//			if(queryArgs.size() > 0){
//				Query q2 = Query.select().order("username").where("id NOT IN", q);
//				result = Database.manager().find(User.class, q2);
//			}
//			else {
//				result = Database.manager().find(User.class, q);
//			}
			

			Query q = Query.select().order("username");
			if(queryArgs.size() > 0){
				q = q.order("username").where("id NOT IN (" + Database.questionMarks(projectMembers.length) + ")", queryArgs.toArray());
			}
			result = Database.manager().find(User.class, q);			
			nonMembers.addAll(Arrays.asList(result));
		} catch (SQLException e) {
			e.printStackTrace();
		}

		return nonMembers;
	}

	/**
	 * Registers the fields that need to listen on outside components.
	 */
	protected void registerListeners() {
		this.getState().registerListener(new ActiveStateListener() {
			@Override
			public void stateChange(ActiveStateEvent event) {
				if (ProjectAdministration.this == getState().getModule())
					view.updateAllComponents();
			}
		});
	}

	public void updateCurrentProject(String name, String description) {
		Project p = this.getState().getProject();
		p.setName(name);
		p.setDescription(description);
		p.save();
	}

	@Override
	public void closeEditor() {
		this.projectEditor.setNewProject();
	}

	public void updateEditor() {
		this.projectEditor.setProject(this.getState().getProject());
	}

	public void createProject() {
		this.projectEditor.setNewProject();
	}

	public void onEditorSave() {
		this.getState().setProject(this.projectEditor.getProject());
	}
}
