/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Lucas Scotta - Initial design, API, and implementation
 *    Dan Bjorge - Initial design, API, and implementation
 *    
 *******************************************************************************/
package edu.wpi.wpisuite.module.rm;

import java.awt.event.KeyEvent;
import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;

import net.java.ao.Entity;


import edu.wpi.wpisuite.components.editor.Editor;
import edu.wpi.wpisuite.components.editor.EditorContainer;
import edu.wpi.wpisuite.components.editor.EditorFactory;
import edu.wpi.wpisuite.components.editor.Editor.ConfirmResult;
import edu.wpi.wpisuite.data.Feature;
import edu.wpi.wpisuite.data.Iteration;
import edu.wpi.wpisuite.data.Project;
import edu.wpi.wpisuite.data.Release;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.data.UserStory;
import edu.wpi.wpisuite.data.funcs.ProjectManager;
import edu.wpi.wpisuite.database.Database;
import edu.wpi.wpisuite.module.ActiveState;
import edu.wpi.wpisuite.module.Module;
import edu.wpi.wpisuite.module.ActiveStateEvent.ActiveStateEventType;

/**
 * A module for the WPI Suite Requirements Manager. Manages requirements -
 * provides methods to deal with all relationships of the iterations,
 * requirements, and all links between them and users
 */
public class RequirementsManager extends Module implements
		EntitySelectionListener, EditorContainer {

	RequirementsManagerView rmView;
	EditorFactory editorFactory;
	Editor entityEditor;

	final public static String moduleName = "Requirements Manager";

	/**
	 * Sets up this module to the main application
	 * 
	 * @param wpiSuite
	 *            The main application
	 */
	public RequirementsManager(ActiveState state) {
		super(state);
		this.rmView = new RequirementsManagerView(this);
		this.editorFactory = new EditorFactory(this);
	}

	@Override
	public boolean shouldBeEnabled() {
		return getState().getProject() != null;
	}

	public void setEntityEditor(Editor editor) {
		this.entityEditor = editor;
		if (editor != null)
			this.rmView.changeEntityEditorView(editor.getView());
		else
			this.rmView.changeEntityEditorView(null);
	}

	/**
	 * For permissions - incomplete
	 */
	public void setEntityEditor(Editor editor, boolean canRead,
			boolean canCreate, boolean canEdit) {
		this.entityEditor = editor;

		if (canEdit) {
			// Leave the editor in the default state
		} else if (canCreate) {
			// editor.setCreateOnly();
		} else
			// only read
			editor.setReadOnly();

		if (editor != null)
			this.rmView.changeEntityEditorView(editor.getView());
		else
			this.rmView.changeEntityEditorView(null);
	}

	/**
	 * @param newSelection
	 *            This can be either an Entity or a class for a new Entity
	 */
	@Override
	public void entitySelectionChange(Entity newSelection) {
		if (newSelection == null) {

		} else
			this.setEntityEditor(this.editorFactory.getEditor(newSelection));
	}

	/**
	 * For permissions
	 */
	public void entitySelectionChange(Entity newSelection, boolean canRead,
			boolean canCreate, boolean canEdit) {
		if (newSelection == null) {

		} else
			this.setEntityEditor(this.editorFactory.getEditor(newSelection),
					canRead, canCreate, canEdit);
	}

	public void newEntityCreator(Class<? extends Entity> newEntityType) {
		this.setEntityEditor(this.editorFactory.getEditor(newEntityType));
	}

	@Override
	public RequirementsManagerView getView() {
		return this.rmView;
	}

	/* See base doc */
	@Override
	public int getModuleKeyShortcut() {
		return KeyEvent.VK_R;
	}

	/* See base doc */
	@Override
	public String getModuleName() {
		return moduleName;
	}

	/**
	 * Gets the backlog for the current project. If there is no backlog, returns
	 * an empty array.
	 * 
	 * @return The backlog for the current project
	 */
	public UserStory[] getBacklog() {
		UserStory[] backlog = ProjectManager.getBacklog(this.getState()
				.getProject());
		if (backlog == null)
			return new UserStory[0];
		else
			return backlog;
	}

	/**
	 * * Generates a blank, empty new iteration for the current project. * Fills
	 * in some default fields and generates a reasonable name for it * @return
	 * The newly generated iteration * @throws SQLException If any database
	 * error occurs
	 */
	@Deprecated
	public Iteration createIteration() throws SQLException {
		Iteration newIteration = Database.manager().create(Iteration.class);
		newIteration.setProject(this.getState().getProject());
		newIteration.setName(this.generateNewIterationName());
		newIteration.save();
		this.getState().fireStateChange(ActiveStateEventType.PROJECT);
		return newIteration;
	}

	/**
	 * Generates a new iteration name for the current user. Format is
	 * "username's New Iteration", with " (n)" appended if that already existed
	 * 
	 * @return A new unique name for a new iteration
	 * @throws SQLException
	 *             If any database error occurs
	 */
	@Deprecated
	public String generateNewIterationName() throws SQLException {
		String username = this.getState().getUser().getUsername();
		String baseName = username + "'s New Iteration";

		String curName = baseName;
		int nameIndex = 1;
		while (Database.manager().find(Iteration.class, "name = ?", curName).length > 0) {
			nameIndex++;
			curName = baseName + " (" + nameIndex + ")";
		}

		return curName;
	}

	/**
	 * Generates a blank new user story for the current project and user. Fills
	 * in some default fields and generates a reasonable name for it
	 * 
	 * @return The newly generated user story
	 * @throws SQLException
	 *             If any database error occurs
	 * @deprecated
	 */
	@Deprecated
	public UserStory createUserstory() throws SQLException {
		UserStory newUserstory = Database.manager().create(UserStory.class);
		newUserstory.setProject(this.getState().getProject());
		newUserstory.setAuthor(this.getState().getUser());
		newUserstory.setTitle(this.generateNewUserStoryName());
		newUserstory.save();
		this.getState().fireStateChange(ActiveStateEventType.PROJECT);
		return newUserstory;
	}

	/**
	 * Generates a new user story for the current project and user and assigns
	 * it to the iteration passed.
	 * 
	 * @param iteration
	 * @return
	 * @throws SQLException
	 * @deprecated
	 */
	@Deprecated
	public UserStory createUserStoryInIteration(Iteration iteration)
			throws SQLException {
		UserStory newUserStory = Database.manager().create(UserStory.class);
		newUserStory.setProject(this.getState().getProject());
		newUserStory.setAuthor(this.getState().getUser());
		newUserStory.setTitle(this.generateNewUserStoryName());
		newUserStory.setIteration(iteration);
		newUserStory.save();
		this.getState().fireStateChange(ActiveStateEventType.PROJECT);
		return newUserStory;
	}

	/**
	 * Generates a new user story name for the current user. Format is
	 * "username's New User Story", with " (n)" appended if that already existed
	 * 
	 * @return A new unique name for a new user story
	 * @throws SQLException
	 *             If any database error occurs
	 * @deprecated
	 */
	@Deprecated
	public String generateNewUserStoryName() throws SQLException {
		String username = this.getState().getUser().getUsername();
		String baseName = username + "'s New User Story";

		String curName = baseName;
		int nameIndex = 1;
		while (Database.manager().find(UserStory.class, "title = ?", curName).length > 0) {
			nameIndex++;
			curName = baseName + " (" + nameIndex + ")";
		}

		return curName;
	}

	/**
	 * Generates a blank new feature for the current project and user. Fills in
	 * some default fields and generates a reasonable name for it
	 * 
	 * @return The newly generated feature
	 * @throws SQLException
	 *             If any database error occurs
	 * @deprecated
	 */
	@Deprecated
	public Feature createFeature() throws SQLException {
		Feature newFeature = Database.manager().create(Feature.class);

		newFeature.setProject(this.getState().getProject());
		newFeature.setAuthor(this.getState().getUser());
		newFeature.setTitle(this.generateNewFeatureName());
		newFeature.save();
		this.getState().fireStateChange(ActiveStateEventType.PROJECT);
		return newFeature;
	}

	/**
	 * Generates a new release name for the current user. Format is
	 * "username's New Release", with " (n)" appended if that already existed
	 * 
	 * @return A new unique name for a new release
	 * @throws SQLException
	 *             If any database error occurs
	 * @deprecated
	 */
	@Deprecated
	public String generateNewFeatureName() throws SQLException {
		String username = this.getState().getUser().getUsername();
		String baseName = username + "'s New Feature";
		String curName = baseName;

		int nameIndex = 1;
		while (Database.manager().find(Feature.class, "title = ?", curName).length > 0) {
			nameIndex++;
			curName = baseName + " (" + nameIndex + ")";
		}

		return curName;
	}

	/**
	 * Gets the usernames for the current members of the project.
	 * 
	 * @return An array of User objects.
	 */
	public ArrayList<String> getUsersForProject() {
		User[] members;
		ArrayList<String> userNames = new ArrayList<String>();

		if (this.getState().getProject() == null)
			return null;

		members = this.getState().getProject().getMembers();

		for (User usr : members)
			userNames.add(usr.getUsername());

		return userNames;
	}

	/**
	 * Gets the list of iterations in the current project. If there is no
	 * current project, returns an empty array.
	 * 
	 * @return The list of iterations for the current project
	 */
	public Iteration[] getIterationList() {
		Project curProject = this.getState().getProject();

		if (curProject == null)
			return new Iteration[0];
		else
			return curProject.getIterations();
	}

	/**
	 * Gets the list of releases in the current project. If there is no current
	 * project, returns an empty array.
	 * 
	 * @return The list of releases for the current project
	 */
	public Release[] getReleaseList() {

		Project curProject = this.getState().getProject();

		if (curProject == null)
			return new Release[0];
		else
			return curProject.getReleases();
	}

	public ArrayList<String> getIterationNamesForProject() {
		Project curProject = this.getState().getProject();
		ArrayList<String> iterationNames = new ArrayList<String>();

		if (curProject == null)
			return null;
		else {
			Iteration[] iterations = curProject.getIterations();
			for (Iteration i : iterations)
				iterationNames.add(i.getName());
		}

		iterationNames.add("Backlog");

		return iterationNames;
	}

	public void deleteIteration(Iteration iteration) throws IOException {
		ConfirmResult choice = this.rmView
				.askYesNoCancelQuestion(
						"Delete Iteration",
						"Are you sure you want to delete this iteration, and all associated requirements?\n\"No\" will send all requirements in this iteration to the backlog.");
		if (choice == ConfirmResult.CANCEL)
			return; // User didn't confirm - don't delete
		else if (choice == ConfirmResult.YES) { // Delete all user stories
			UserStory[] stories = iteration.getUserStories();
			for (UserStory u : stories)
				try {
					Database.manager().delete(u);
				} catch (SQLException e) {
					throw new IOException(
							"A database error occured while deleting a user story associated with this iteration.");
				}
		} else if (choice == ConfirmResult.NO) { // Move all user stories
			UserStory[] uStories = iteration.getUserStories();
			for (UserStory u : uStories) {
				u.setIteration(null);
				u.save();
			}
		}
		try { // Delete iteration
			Database.manager().delete(iteration);
		} catch (SQLException e) {
			throw new IOException(
					"A database error occured while deleteing this iteration.");
		}

		this.rmView.updateAllComponents();
	}

	public void closeEditor() {
		this.entityEditor = null;
		this.rmView.closeEditorView();
	}

	@Override
	public void onEditorSave() {
	}

}
