/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Unknown
 *    Lucas Scotta - users list sorted alphabetically
 *    
 *******************************************************************************/

package edu.wpi.wpisuite.module.systemadmin;

import java.awt.event.KeyEvent;
import java.sql.SQLException;

import net.java.ao.DBParam;
import net.java.ao.Query;


import edu.wpi.wpisuite.components.editor.Editor;
import edu.wpi.wpisuite.components.editor.EditorContainer;
import edu.wpi.wpisuite.components.editor.EditorFactory;
import edu.wpi.wpisuite.components.editor.EditorPanelView;
import edu.wpi.wpisuite.data.Config;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.database.Database;
import edu.wpi.wpisuite.module.ActiveState;
import edu.wpi.wpisuite.module.Module;
import edu.wpi.wpisuite.module.ModuleView;

/**
 * A module for system-wide administration - database initialization and
 * management, user management, etc.
 */
public class SystemAdmin extends Module implements EditorContainer {

	SystemAdminView systemAdminView;
	EditorFactory editorFactory;

	final public static String moduleName = "System Administration";

	public SystemAdmin(ActiveState state) {
		super(state);
		this.systemAdminView = new SystemAdminView(this);
		this.resetEmailFields();
		this.editorFactory = new EditorFactory(this);
		this.setEditor(null);
	}

	@Override
	public int getModuleKeyShortcut() {
		return KeyEvent.VK_A;
	}

	@Override
	public String getModuleName() {
		return moduleName;
	}

	@Override
	public ModuleView getView() {
		return this.systemAdminView;
	}

	@Override
	public boolean shouldBeVisible() {
		return getState().getUser() == null || getState().getUser().isAdmin();
	}

	@Override
	public void closeEditor() {
		this.systemAdminView.setEditorView(null);
	}

	@Override
	public void onEditorSave() {
		systemAdminView.updateUsersList();
	}

	/**
	 * 
	 * @return All users in the database, sorted by username
	 */
	public User[] getUsers() {
		try {
			return Database.manager().find(User.class,
					Query.select().order("username"));
		} catch (SQLException e) {
			systemAdminView.popErrorMessage(e);
			return new User[0];
		}
	}

	public void selectUser(User user) {
		setEditor(editorFactory.getEditor(user));
	}

	public void setEditor(Editor e) {
		EditorPanelView view = null;
		if (e != null)
			view = e.getView();
		this.systemAdminView.setEditorView(view);
	}

	public void addNewUser() {
		setEditor(editorFactory.getEditor(User.class));
	}

	public void resetEmailFields() {
		try {
			systemAdminView.setEmailFields(getConfigValue("mail_protocol"),
					getConfigValue("mail_server"), getConfigValue("mail_from"),
					getConfigValue("mail_user"),
					getConfigValue("mail_password"));
		} catch (Exception e) {
			systemAdminView.popErrorMessage(e);
		}
	}

	private String getConfigValue(String configName) throws Exception {
		Config conf = getConfig(configName);
		if (conf == null)
			return "";
		else
			return conf.getValue();
	}

	private Config getConfig(String configName) throws Exception {
		Config[] confs = Database.manager().find(Config.class, "name = ?",
				configName);
		if (confs.length == 0)
			return null;
		if (confs.length > 1)
			throw new Exception("Multiple conflicting configs found!");
		return confs[0];
	}

	private void createConfig(String configName, String configValue)
			throws SQLException {
		Database.manager().create(Config.class,
				new DBParam("name", configName),
				new DBParam("value", configValue));
	}

	private void setConfigValue(String configName, String configValue)
			throws Exception {
		Config conf = getConfig(configName);
		if (conf == null)
			createConfig(configName, configValue);
		else
			conf.setValue(configValue);
	}

	public void saveEmailSettings(String protocol, String server, String from,
			String username, String password) {
		try {
			setConfigValue("mail_protocol", protocol);
			setConfigValue("mail_server", server);
			setConfigValue("mail_from", from);
			setConfigValue("mail_user", username);
			setConfigValue("mail_password", password);
		} catch (Exception e) {
			systemAdminView.popErrorMessage(e);
		}
	}

}
