/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Lucas Scotta - Initial design, API, and implementation
 *    
 *******************************************************************************/

package edu.wpi.wpisuite.module.traceability;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import net.java.ao.Entity;

import org.apache.log4j.Logger;


import edu.wpi.wpisuite.components.editor.Editor;
import edu.wpi.wpisuite.components.editor.EditorContainer;
import edu.wpi.wpisuite.components.editor.EditorFactory;
import edu.wpi.wpisuite.data.Requirement;
import edu.wpi.wpisuite.data.TraceabilityAssociation;
import edu.wpi.wpisuite.data.TraceabilityMatrix;
import edu.wpi.wpisuite.database.Database;
import edu.wpi.wpisuite.module.ActiveState;
import edu.wpi.wpisuite.module.rm.EntitySelectionListener;

/**
 * Controller for the display of a traceability matrix
 */
public class MatrixDisplayController implements TraceabilityStepController {

	private Logger logger = Logger.getLogger(this.getClass());

	private MatrixDisplayView view;
	private TraceabilityController traceability;
	private RowEditorController rowEditorController;
	private ColEditorController colEditorController;
	private List<Requirement> rows;
	private List<Requirement> cols;

	public List<Requirement> getRows() {
		return rows;
	}

	public void setRows(List<Requirement> rows) {
		this.rows = rows;
	}

	public List<Requirement> getCols() {
		return cols;
	}

	public void setCols(List<Requirement> cols) {
		this.cols = cols;
	}

	public ArrayList<TraceabilityAssociation> getTraceabilityAssociations() {
		ArrayList<TraceabilityAssociation> associations = new ArrayList<TraceabilityAssociation>();

		TraceabilityMatrix tm = traceability.getActiveTraceabilityMatrix();

		Requirement[] rows = traceability.getMatrixRows();
		Requirement[] cols = traceability.getMatrixColumns();

		if (tm == null || rows.length == 0 || cols.length == 0)
			return associations;

		try {
			String fromMarks = Database.questionMarks(cols.length);
			String toMarks = Database.questionMarks(rows.length);

			String queryStr = "fromType = ? AND toType = ? AND fromID IN ("
					+ fromMarks + ") AND toID IN (" + toMarks + ")";

			Object[] queryArgs = new Object[2 + cols.length + rows.length];
			queryArgs[0] = Database.manager().getTableNameConverter().getName(
					cols[0].getEntityType());
			queryArgs[1] = Database.manager().getTableNameConverter().getName(
					rows[0].getEntityType());
			for (int i = 0; i < cols.length; i++)
				queryArgs[i + 2] = cols[i];
			for (int i = 0; i < rows.length; i++)
				queryArgs[i + cols.length + 2] = rows[i];

			TraceabilityAssociation[] queryResults = Database.manager().find(
					TraceabilityAssociation.class, queryStr, queryArgs);
			for (TraceabilityAssociation association : queryResults)
				associations.add(association);
		} catch (SQLException e) {
			e.printStackTrace();
		}

		return associations;
	}

	/**
	 * Sets up this panel to the traceability matrix module
	 * 
	 * @param wpiSuite
	 *            The main application
	 */
	public MatrixDisplayController(TraceabilityController traceability) {
		this.view = new MatrixDisplayView(this, traceability);
		this.traceability = traceability;

		rowEditorController = new RowEditorController();
		colEditorController = new ColEditorController();
	}

	@Override
	public MatrixDisplayView getView() {
		return this.view;
	}

	public ColEditorController getColEditorController() {
		return colEditorController;
	}

	public RowEditorController getRowEditorController() {
		return rowEditorController;
	}

	/**
	 * 
	 * @param tm
	 * @param to
	 * @param from
	 * @return
	 */
	public TraceabilityAssociation createTraceabilityAssociation(
			TraceabilityMatrix tm, Requirement to, Requirement from) {

		TraceabilityAssociation newAssociation = null;
		try {
			newAssociation = Database.manager().create(
					TraceabilityAssociation.class);
			newAssociation.setFrom(from);
			newAssociation.setTo(to);
			newAssociation.save();
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return newAssociation;
	}

	/**
	 * 
	 * @param tm
	 * @param to
	 * @param from
	 */
	public void deleteTraceabilityAssociation(TraceabilityMatrix tm,
			Requirement to, Requirement from) {
		try {
			String queryStr = "fromType = ? AND toType = ? AND fromID = ? AND toID = ?";

			Object[] queryArgs = new Object[4];
			queryArgs[0] = Database.manager().getTableNameConverter().getName(
					from.getEntityType());
			queryArgs[1] = Database.manager().getTableNameConverter().getName(
					to.getEntityType());
			queryArgs[2] = from.getID();
			queryArgs[3] = to.getID();

			TraceabilityAssociation[] queryResults = Database.manager().find(
					TraceabilityAssociation.class, queryStr, queryArgs);

			if (queryResults.length > 0)
				Database.manager().delete(queryResults);
		} catch (SQLException e) {
			logger.error(e.getMessage());
		}
	}

	public class RowEditorController implements EntitySelectionListener,
			EditorContainer {

		private EditorFactory editorFactory;
		private Editor entityEditor;

		public RowEditorController() {
			this.editorFactory = new EditorFactory(this);
		}

		public void setEntityEditor(Editor editor) {
			this.entityEditor = editor;
			if (editor != null) {
				this.entityEditor.setReadOnly();
				view.changeRowEntityEditorView(editor.getView());
			} else
				view.changeRowEntityEditorView(null);
		}

		/**
		 * @param newSelection
		 *            This can be either an Entity or a class for a new Entity
		 */
		@Override
		public void entitySelectionChange(Entity newSelection) {
			if (newSelection == null) {

			} else
				this
						.setEntityEditor(this.editorFactory
								.getEditor(newSelection));
		}

		public void newEntityCreator(Class<? extends Entity> newEntityType) {
			this.setEntityEditor(this.editorFactory.getEditor(newEntityType));
		}

		@Override
		public void closeEditor() {
			this.entityEditor = null;
			view.closeEditorView();
		}

		@Override
		public ActiveState getState() {
			return traceability.getState();
		}

		@Override
		public void onEditorSave() {
			// TODO Auto-generated method stub

		}

	}

	public class ColEditorController implements EntitySelectionListener,
			EditorContainer {

		private EditorFactory editorFactory;
		private Editor entityEditor;

		public ColEditorController() {
			this.editorFactory = new EditorFactory(this);
		}

		public void setEntityEditor(Editor editor) {
			this.entityEditor = editor;
			if (editor != null) {
				this.entityEditor.setReadOnly();
				view.changeColEntityEditorView(editor.getView());
			} else
				view.changeColEntityEditorView(null);
		}

		/**
		 * @param newSelection
		 *            This can be either an Entity or a class for a new Entity
		 */
		@Override
		public void entitySelectionChange(Entity newSelection) {
			if (newSelection == null) {

			} else
				this
						.setEntityEditor(this.editorFactory
								.getEditor(newSelection));
		}

		public void newEntityCreator(Class<? extends Entity> newEntityType) {
			this.setEntityEditor(this.editorFactory.getEditor(newEntityType));
		}

		@Override
		public void closeEditor() {
			this.entityEditor = null;
			view.closeEditorView();
		}

		@Override
		public ActiveState getState() {
			return traceability.getState();
		}

		@Override
		public void onEditorSave() {
			// TODO Auto-generated method stub

		}
	}

	@Override
	public void update() {
		this.setRows(Arrays.asList(traceability.getMatrixRows()));
		this.setCols(Arrays.asList(traceability.getMatrixColumns()));

		this.getView().updateAllComponents();
	}

	public void closeEditors() {
		view.closeEditorView();
	}
}