/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Sabrina Varanelli - Initial design
 *    Lucas Scotta - Additional design, initial API and implementation
 *    Dan Bjorge - comments and cleanup of controller-view separation
 *******************************************************************************/

package edu.wpi.wpisuite.module.traceability;

import java.awt.event.KeyEvent;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JComponent;
import javax.swing.JOptionPane;


import org.apache.log4j.Logger;

import edu.wpi.wpisuite.data.ActiveMatrixRelation;
import edu.wpi.wpisuite.data.Feature;
import edu.wpi.wpisuite.data.Iteration;
import edu.wpi.wpisuite.data.Project;
import edu.wpi.wpisuite.data.Requirement;
import edu.wpi.wpisuite.data.TraceabilityMatrix;
import edu.wpi.wpisuite.data.TraceabilityMatrixCriteria;
import edu.wpi.wpisuite.data.TraceabilityMatrixCriterion;
import edu.wpi.wpisuite.data.User;
import edu.wpi.wpisuite.data.UserStory;
import edu.wpi.wpisuite.database.Database;
import edu.wpi.wpisuite.module.ActiveState;
import edu.wpi.wpisuite.module.ActiveStateEvent;
import edu.wpi.wpisuite.module.ActiveStateListener;
import edu.wpi.wpisuite.module.Module;
import edu.wpi.wpisuite.module.ActiveStateEvent.ActiveStateEventType;
import edu.wpi.wpisuite.module.traceability.criteria.CriteriaController;
import edu.wpi.wpisuite.module.traceability.criteria.CriteriaController.AxisType;
import edu.wpi.wpisuite.module.velocity.Velocity;

/**
 * Defines a module which allows the user to create a traceability matrix.
 * Mostly orchestrates the relationships between the sub-controllers in this
 * package that do most of the real work.
 */
public class TraceabilityController extends Module implements
		ActiveStateListener {
	Logger logger = Logger.getLogger(this.getClass());

	/**
	 * The main view that contains all the sub-views
	 */
	private TraceabilityView view;

	private MatrixDisplayController matrixDisplayController;

	/**
	 * Defines the various subcomponents of this module
	 */
	public enum TraceabilityStep {
		EDIT, CHOOSE
	}

	private Map<TraceabilityStep, TraceabilityStepController> stepMap;

	private MatrixChooserController matrixChooserController;

	private MatrixEditController editController;

	private CriteriaController rowController;

	private CriteriaController colController;

	private NoMatrixView noMatrixView = new NoMatrixView();

	/** The displayed name of the module as a whole */
	final public static String moduleName = "Traceability Matrix";

	/**
	 * Calls generic module constructor and sets up the sub-controllers
	 * 
	 * @param wpiSuite
	 *            The main application object
	 */
	public TraceabilityController(ActiveState state) {
		super(state);

		initStepControllers();

		this.view = new TraceabilityView(this);

		fireStepChange(getInitialStep());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see module.Module#getView()
	 */
	@Override
	public TraceabilityView getView() {
		return this.view;
	}

	@Override
	public boolean shouldBeEnabled() {
		return getState().getProject() != null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see module.Module#getModuleName()
	 */
	@Override
	public String getModuleName() {
		return moduleName;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see module.Module#getModuleKeyShortcut()
	 */
	@Override
	public int getModuleKeyShortcut() {
		return KeyEvent.VK_T;
	}

	/**
	 * Initializes the sub-controllers defining each step of matrix dealings
	 */
	private void initStepControllers() {
		stepMap = new HashMap<TraceabilityStep, TraceabilityStepController>();

		this.rowController = new CriteriaController(this, AxisType.ROW);
		this.colController = new CriteriaController(this, AxisType.COLUMN);

		this.editController = new MatrixEditController(this, rowController,
				colController);
		stepMap.put(TraceabilityStep.EDIT, editController);

		this.matrixDisplayController = new MatrixDisplayController(this);
		this.matrixChooserController = new MatrixChooserController(this,
				matrixDisplayController);
		stepMap.put(TraceabilityStep.CHOOSE, matrixChooserController);
	}

	/**
	 * @return The last active traceability matrix for the current user and
	 *         project
	 */
	public TraceabilityMatrix getActiveTraceabilityMatrix() {
		Project curProj = this.getState().getProject();
		User curUser = this.getState().getUser();
		TraceabilityMatrix curMatrix = null;

		if (curProj == null || curUser == null)
			return null;

		try {
			ActiveMatrixRelation[] queryResults = Database.manager().find(
					ActiveMatrixRelation.class, "projectID = ? AND userID = ?",
					curProj.getID(), curUser.getID());
			if (queryResults.length == 1)
				curMatrix = queryResults[0].getTraceabilityMatrix();
			else if (queryResults.length > 1)
				logger
						.error("duplicate ActiveMatrixRelations found in database");
			else {
				TraceabilityMatrix[] projectMatrices = getState().getProject()
						.getTraceabilityMatrices();
				if (projectMatrices.length > 0)
					curMatrix = projectMatrices[0];
				else {
					newActiveMatrix();
					return getActiveTraceabilityMatrix();
				}
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}

		return curMatrix;

	}

	public TraceabilityMatrix createActiveMatrix() {
		Project curProject = getState().getProject();

		if (curProject == null) {
			logger.error("curProject is null");
			return null;
		}

		TraceabilityMatrix tm = null;
		try {
			tm = Database.manager().create(TraceabilityMatrix.class);
			tm.setName("New Matrix Name");
			tm.setProject(curProject);
			tm.save();
		} catch (SQLException e) {
			logger.error("SQLException");
		}

		setActiveTraceabilityMatrix(tm);

		return tm;
	}

	/**
	 * Sets the active traceability matrix for the current user and project
	 * 
	 * @return
	 */
	public void setActiveTraceabilityMatrix(TraceabilityMatrix t) {
		Project curProj = this.getState().getProject();
		User curUser = this.getState().getUser();

		if (curProj == null || curUser == null)
			return;

		try {
			ActiveMatrixRelation[] queryResults = Database.manager().find(
					ActiveMatrixRelation.class, "projectID = ? AND userID = ?",
					curProj.getID(), curUser.getID());
			if (queryResults.length == 0) {
				ActiveMatrixRelation activeMatrixRelation = Database.manager()
						.create(ActiveMatrixRelation.class);
				activeMatrixRelation.setProject(curProj);
				activeMatrixRelation.setUser(curUser);
				activeMatrixRelation.setTraceabilityMatrix(t);
				activeMatrixRelation.save();
			} else if (queryResults.length == 1) {
				queryResults[0].setTraceabilityMatrix(t);
				queryResults[0].save();
			} else
				logger
						.error("duplicate ActiveMatrixRelations found in database");
		} catch (SQLException e) {
			e.printStackTrace();
		}

		updateAllSteps();
	}

	/**
	 * Gets the list of iterations in the current project. If there is no
	 * current project, returns an empty array.
	 * 
	 * @return The list of iterations for the current project
	 */
	public Iteration[] getIterationList() {
		Project curProject = this.getState().getProject();

		if (curProject == null)
			return new Iteration[0];
		else
			return curProject.getIterations();
	}

	/**
	 * Gets the feature requirements for the current project. If there are no
	 * feature requirements, returns an empty array.
	 */
	public Feature[] getFeatureRequirements() {
		Project curProj = this.getState().getProject();

		if (curProj == null)
			return new Feature[0];
		else
			return curProj.getFeatures();
	}

	/**
	 * Gets the backlog for the current project. If there is no backlog, returns
	 * an empty array.
	 * 
	 * @return The backlog for the current project
	 */
	public UserStory[] getBacklog() {
		Project curProj = this.getState().getProject();

		if (curProj == null) {
			logger.error("Current project is null");
			return new UserStory[0];
		} else {
			UserStory[] backlog = null;
			try {
				backlog = Database.manager().find(UserStory.class,
						"iterationID IS NULL AND projectID = ?",
						curProj.getID());
			} catch (SQLException e) {
				JOptionPane.showMessageDialog(this.view, e.getMessage());
				return new UserStory[0];
			}
			return backlog;
		}
	}

	/**
	 * Gets the user stories belonging to a given iteration If there are no user
	 * stories, returns an empty array.
	 * 
	 * @param it
	 *            the iteration you want the user stories for. must not be null
	 */
	public UserStory[] getIterationUserStories(Iteration it) {
		return it.getUserStories();
	}

	/**
	 * Forces an update to the currently selected subcomponent
	 */
	public void updateSelected() {
		if (getState().getProject() == null || getState().getUser() == null)
			return;

		updateAllSteps();

	}

	public void updateAllSteps() {
		if (getState().getProject() == null || getState().getUser() == null)
			return;

		// order is important!
		rowController.update();
		colController.update();

		matrixChooserController.update();
		matrixDisplayController.update();

		boolean hasRows = getMatrixRows().length > 0;
		boolean hasCols = getMatrixColumns().length > 0;
		if (hasRows && hasCols)
			matrixChooserController.getView().setContentPane(
					matrixDisplayController.getView());
		else {
			if (hasRows)
				noMatrixView.setDisplayString("No columns selected");
			else if (hasCols)
				noMatrixView.setDisplayString("No rows selected");
			else
				noMatrixView.setDisplayString("No data selected");
			matrixChooserController.getView().setContentPane(noMatrixView);
		}

		editController.update();
	}

	public Requirement[] getMatrixRows() {
		return rowController.getSelectedRequirements();
	}

	public Requirement[] getMatrixColumns() {
		return colController.getSelectedRequirements();
	}

	/**
	 * @return The step to be selected on module startup
	 */
	public TraceabilityStep getInitialStep() {
		return TraceabilityStep.CHOOSE;
	}

	/**
	 * @return The view of the given step type
	 */
	public JComponent getStepView(TraceabilityStep step) {
		return stepMap.get(step).getView();
	}

	/**
	 * Sets the selected step and updates it for viewing
	 * 
	 * @param selectedStep
	 *            The newly selected step
	 */
	private void fireStepChange(TraceabilityStep selectedStep) {
		updateSelected();
		view.selectStep(selectedStep);
	}

	public void editActiveMatrix() {
		fireStepChange(TraceabilityStep.EDIT);
	}

	public void deleteActiveMatrix() {
		try {
			TraceabilityMatrix matrix = getActiveTraceabilityMatrix();

			ActiveMatrixRelation[] activeAssocs = Database.manager().find(
					ActiveMatrixRelation.class, "traceabilityMatrixID = ?",
					matrix);
			Database.manager().delete(activeAssocs);

			if (matrix != null) {
				TraceabilityMatrixCriteria rowc = matrix.getRowCriteria();
				TraceabilityMatrixCriteria colc = matrix.getColumnCriteria();

				Database.manager().delete(matrix);

				if (rowc != null) {
					TraceabilityMatrixCriterion rown = rowc.getRootCriterion();

					Database.manager().delete(rowc);

					rowController.deleteCriterionTree(rown);
				}

				if (colc != null) {
					TraceabilityMatrixCriterion coln = colc.getRootCriterion();

					Database.manager().delete(colc);

					colController.deleteCriterionTree(coln);
				}

			}

			updateSelected();
		} catch (SQLException e) {
			view.error("An error occured while deleting the matrix: "
					+ e.getMessage());
		}

	}

	public void newActiveMatrix() {
		setActiveTraceabilityMatrix(createActiveMatrix());
		editActiveMatrix();
	}

	public TraceabilityMatrix[] getMatrixOptions() {
		if (getState().getProject() == null)
			return new TraceabilityMatrix[0];
		return getState().getProject().getTraceabilityMatrices();
	}

	public void changeActiveMatrixName(String matrixName) {
		TraceabilityMatrix matrix = getActiveTraceabilityMatrix();
		if (matrix == null)
			return;
		matrix.setName(matrixName);
		matrix.save();
	}

	public void returnToActiveMatrix() {
		changeActiveMatrixName(editController.getMatrixName());
		fireStepChange(TraceabilityStep.CHOOSE);
	}

	@Override
	public void stateChange(ActiveStateEvent event) {
		if (TraceabilityController.this == super.getState().getModule() && (event.getType().equals(ActiveStateEventType.PROJECT)
				|| event.getType().equals(ActiveStateEventType.USER))) {
			matrixDisplayController.closeEditors();
			fireStepChange(this.getInitialStep());

		}
	}
}
