/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Lucas Scotta - initial API and implementation
 *    Dan Bjorge - comments and cleanup
 *******************************************************************************/

package edu.wpi.wpisuite.module.traceability.components;

import java.util.ArrayList;
import java.util.List;

import javax.swing.table.DefaultTableModel;

import edu.wpi.wpisuite.data.Requirement;
import edu.wpi.wpisuite.data.funcs.Artifact;

/**
 * A model for the table displaying the traceability matrix
 */
@SuppressWarnings("serial")
public class TraceMatrixTableModel extends DefaultTableModel {

	/** The string to display in the upper-left table cell */
	public String leftColumnTitle = "";

	/** The requirements representing rows */
	public List<Requirement> rows;

	/** The requirements representing columns */
	public List<Requirement> cols;

	/** The strings to display in the cells */
	public String[][] cellValues;

	/**
	 * Initializes the underlying table and the row, column, and cell data sets.
	 */
	public TraceMatrixTableModel() {
		super();
		this.rows = new ArrayList<Requirement>();
		this.cols = new ArrayList<Requirement>();
		this.cellValues = new String[0][0];
	}

	/**
	 * Initializes the underlying table, the cell data strings to blank, and the
	 * row and column data sets to the given ones
	 * 
	 * @param rows
	 *            The requirements to represent table rows
	 * @param cols
	 *            The requirements to represent table columns
	 */
	public TraceMatrixTableModel(ArrayList<Requirement> rows,
			ArrayList<Requirement> cols) {
		super();
		this.rows = rows;
		this.cols = cols;
		this.cellValues = new String[rows.size()][cols.size()];
	}

	/**
	 * Overriden to disable editing of cell values
	 */
	@Override
	public boolean isCellEditable(int rowIndex, int mColIndex) {
		return false;
	}

	/**
	 * Overriden to use the appropriate names of the heading requirements
	 */
	@Override
	public String getColumnName(int column) {
		if (column == 0)
			return this.leftColumnTitle;
		else
			return new Artifact(this.cols.get(column - 1)).toString();
	}

	/**
	 * Overriden to use the appropriate names of the heading requirements
	 */
	@Override
	public int getRowCount() {
		if (this.rows != null)
			return this.rows.size();
		else
			return 0;
	}

	/**
	 * Overriden to use the internal requirement column list, and to take into
	 * account that there is a non-counted first column for row headers
	 */
	@Override
	public int getColumnCount() {
		if (this.cols != null)
			return this.cols.size() + 1;
		else
			return 1;
	}

	/**
	 * Updates the rows list, and updates the data for the underlying JTable to
	 * display accordingly
	 * 
	 * @param rows
	 *            The new requirements to use a rows
	 */
	public void setRows(List<Requirement> rows) {
		this.rows = rows;
		this.cellValues = new String[rows.size()][this.cols.size()];
		this.fireTableDataChanged();
	}

	/**
	 * Updates the columns list, and updates the data for the underlying JTable
	 * to display accordingly
	 * 
	 * @param cols
	 *            The new requirements to use as columns
	 */
	public void setCols(List<Requirement> cols) {
		this.cols = cols;
		this.cellValues = new String[this.rows.size()][cols.size()];
		this.fireTableStructureChanged();
	}

	/**
	 * Empties the table of requirements and updates the underlying JTable
	 * structure to display the newly emptied table appropriately
	 */
	public void clear() {
		this.rows.clear();
		this.cols.clear();
		this.fireTableStructureChanged();
	}

	/**
	 * @param row
	 *            The index of row to get the associated Requirement for. This
	 *            is consistent with both this class's and the underlying
	 *            JTable's representations of a row
	 * @return The requirement associated with that row
	 */
	public Requirement getRow(int row) {
		return this.rows.get(row);
	}

	/**
	 * @param col
	 *            The index of column to get the associated Requirement for.
	 *            This is consistent with the underlying JTable's representation
	 *            of columns, NOT with this class's. This means that there is NO
	 *            requirement associated with column 0.
	 * @return The requirement at the given column, or null of the column index
	 *         is 0
	 */
	public Requirement getColumn(int col) {
		if (col > 0)
			return this.cols.get(col - 1);
		else
			return null;
	}

	/**
	 * @param r
	 *            The requirement to find the row of
	 * @return The index of the row for r, as defined by both the underlying
	 *         JTable AND this class's representation
	 */
	public int getRowIndexOf(Requirement r) {
		return this.rows.indexOf(r);
	}

	/**
	 * @param r
	 *            The requirement to find the column of
	 * @return The index of the column for r, as defined by the underlying
	 *         JTable, NOT this class's representation
	 */
	public int getColIndexOf(Requirement r) {
		return this.cols.indexOf(r) + 1;
	}

	/**
	 * Gets the string value at a particular location. Used by the underlying
	 * JTable implementation in drawing. Handles the row headers appropriately,
	 * since they are actually just a normal column as far as the underlying
	 * JTable is aware.
	 * 
	 * @param row
	 *            The row to get data from. The column headers are NOT a row.
	 * @param col
	 *            The column to get data from. The row headers ARE a column.
	 */
	@Override
	public String getValueAt(int row, int col) {
		if (col == 0)
			return new Artifact(this.rows.get(row)).toString();
		else
			return this.cellValues[row][col - 1];
	}

	/**
	 * Sets a particular data cell to display a given string value.
	 * 
	 * @param s
	 *            The string to place at the given location.
	 * @param row
	 *            The row to place it at. Headers do NOT count as rows.
	 * @param col
	 *            The column to place it at. Headers DO count as a column.
	 */
	public void setValueAt(String s, int row, int col) {
		if (col == 0)
			return;
		else
			this.cellValues[row][col - 1] = s;
		this.fireTableDataChanged();
	}
}
