/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.module.traceability.criteria;

import java.sql.SQLException;
import java.util.Arrays;
import java.util.Set;

import javax.swing.JPanel;

import edu.wpi.wpisuite.data.Project;
import edu.wpi.wpisuite.data.Requirement;
import edu.wpi.wpisuite.data.TraceabilityMatrix;
import edu.wpi.wpisuite.data.TraceabilityMatrixCriteria;
import edu.wpi.wpisuite.data.TraceabilityMatrixCriterion;
import edu.wpi.wpisuite.data.TraceabilityMatrixCriteria.RequirementType;
import edu.wpi.wpisuite.data.TraceabilityMatrixCriterion.CriterionType;
import edu.wpi.wpisuite.data.util.RequirementComparator;
import edu.wpi.wpisuite.database.Database;
import edu.wpi.wpisuite.module.ActiveState;
import edu.wpi.wpisuite.module.traceability.TraceabilityController;
import edu.wpi.wpisuite.module.traceability.TraceabilityStepController;

/**
 * This class handles the interfacing between a matrix and a set of axis
 * criteria, as well as between that criteria and its stored set of criterions.
 * 
 * Also provides the means by which criteria can be used to generate a set of
 * requirements satisfying the criteria
 */
public class CriteriaController implements CriterionControllerContainer,
		TraceabilityStepController {

	public enum AxisType {
		ROW, COLUMN
	}

	private TraceabilityController matrixController;
	private TraceabilityMatrixCriteria criteria;
	private CriterionController<?> rootCriterionController;

	private CriterionControllerFactory factory;

	private CriteriaPanelView view;

	private AxisType axisType;

	public CriteriaController(TraceabilityController matrixController,
			AxisType axis) {
		this.matrixController = matrixController;
		this.axisType = axis;
		this.factory = new CriterionControllerFactory(this);

		initializeView();

		initializeCriteria();
	}

	private void initializeView() {
		this.view = new CriteriaPanelView(this);

	}

	private void setRootCriterion(TraceabilityMatrixCriterion root) {
		criteria.setRootCriterion(root);
		criteria.save();

		updateRootCriterionController();
	}

	private void updateRootCriterionController() {
		rootCriterionController = factory.constructFor(criteria
				.getRootCriterion());

		if (rootCriterionController == null)
			view.setRootCriterionView(null);
		else
			view.setRootCriterionView(rootCriterionController.getView());

	}

	public Requirement[] getSelectedRequirements() {
		if (rootCriterionController == null)
			return new Requirement[0];
		Set<Requirement> selectedSet = rootCriterionController
				.getSelectedRequirements();
		Requirement[] selected = selectedSet.toArray(new Requirement[0]);
		Arrays.sort(selected, new RequirementComparator());
		return selected;
	}

	public Project getProject() {
		return matrixController.getState().getProject();
	}

	/**
	 * Initializes this controller with the appropriate axis of criteria for the
	 * traceability controller's current matrix, or a new empty set of criteria
	 * if the matrix or its current criteria is null
	 * 
	 * @throws SQLException
	 */
	protected void initializeCriteria() {
		TraceabilityMatrix matrix = matrixController
				.getActiveTraceabilityMatrix();

		if (matrix == null)
			return;

		TraceabilityMatrixCriteria mCriteria = getMatrixCriteria(matrix);
		criteria = null;

		try {
			if (mCriteria == null)
				initializeNewCriteria();
			else {
				criteria = mCriteria;
				updateRootCriterionController();
			}
		} catch (SQLException e) {
			view
					.error("A database error occured while initializing the critera");
		}

		view.updateRequirementType();
	}

	/**
	 * Gets the criteria of the axis appropriate to this object for the given
	 * matrix
	 * 
	 * @param matrix
	 *            The matrix to get criteria from
	 * @return The criteria for the appropriate axis of the matrix
	 */
	public TraceabilityMatrixCriteria getMatrixCriteria(
			TraceabilityMatrix matrix) {
		switch (axisType) {
		case ROW:
			return matrix.getRowCriteria();
		case COLUMN:
			return matrix.getColumnCriteria();
		}
		return null;
	}

	/**
	 * Initializes this controller with a new, empty set of criteria
	 * 
	 * @throws SQLException
	 */
	public void initializeNewCriteria() throws SQLException {
		criteria = Database.manager().create(TraceabilityMatrixCriteria.class);
		setMatrixCriteria(matrixController.getActiveTraceabilityMatrix());
		criteria.setRequirementType(view.getSelectedRequirementType());
		setDefaultCriterion();
		criteria.save();
	}

	public void setDefaultCriterion() {
		setRootCriterion(getDefaultCriterion());
	}

	public TraceabilityMatrixCriterion getDefaultCriterion() {
		TraceabilityMatrixCriterion c = null;

		try {
			c = Database.manager().create(TraceabilityMatrixCriterion.class);

			c.setType(CriterionType.REQUIREMENT);
			c.setTypeData(new int[0]);
			c.save();
		} catch (Exception e) {
			view.error(e.getMessage());
		}

		return c;

	}

	/**
	 * Updates the given matrix to use this set of criteria (and also ensures
	 * that this set of criteria is set to the given matrix). This should NOT
	 * need to be called for criteria updates - only for new criteria creation
	 */
	public void setMatrixCriteria(TraceabilityMatrix matrix) {
		switch (axisType) {
		case ROW:
			matrix.setRowCriteria(criteria);
			break;
		case COLUMN:
			matrix.setColumnCriteria(criteria);
			break;
		}
		matrix.save();
	}

	/**
	 * @return The axis type this controller is dealing with selection over
	 */
	public AxisType getAxisType() {
		return this.axisType;
	}

	@Override
	public RequirementType getRequirementType() {
		if (criteria == null || criteria.getRequirementType() == null)
			return this.getDefaultRequirementType();
		else
			return this.criteria.getRequirementType();
	}

	private RequirementType getDefaultRequirementType() {
		if (axisType.equals(AxisType.COLUMN))
			return RequirementType.FEATURE;
		else
			return RequirementType.USER_STORY;
	}

	@Override
	public ActiveState getState() {
		return this.matrixController.getState();
	}

	public void requirementTypeSelected(RequirementType newType) {
		if (criteria == null)
			initializeCriteria();

		if (criteria.getRequirementType() == null
				|| !criteria.getRequirementType().equals(newType)) {
			TraceabilityMatrixCriterion root = criteria.getRootCriterion();

			criteria.setRootCriterion(null);
			criteria.setRequirementType(newType);
			setDefaultCriterion();
			criteria.save();

			deleteCriterionTree(root);
		}
	}

	public void deleteCriterionTree(TraceabilityMatrixCriterion c) {
		if (c == null)
			return;

		if (c.getType() != null && c.getType().isTree())
			for (int id : c.getTypeData())
				deleteCriterionTree(Database.manager().get(
						TraceabilityMatrixCriterion.class, id));

		try {
			Database.manager().delete(c);
		} catch (Exception e) {
			view.error("An error occured while deleting a criterion");
		}
	}

	@Override
	public JPanel getView() {
		return view;
	}

	@Override
	public void update() {
		initializeCriteria();
	}

	public String getAxisDescription() {
		if (axisType.equals(AxisType.COLUMN))
			return "From... ";
		else
			return "To... ";
	}
}
