/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Dan Bjorge - Initial design, API, and implementation
 *******************************************************************************/

package edu.wpi.wpisuite.module.traceability.criteria;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import net.java.ao.Entity;

import org.apache.log4j.Logger;


import edu.wpi.wpisuite.components.EntityIconSet;
import edu.wpi.wpisuite.data.Requirement;
import edu.wpi.wpisuite.data.TraceabilityMatrixCriterion;
import edu.wpi.wpisuite.database.Database;

/**
 * @param <V>
 *            The type of the view for this particular controller
 */
public abstract class CriterionController<V extends CriterionView> {
	/** The containing controller */
	protected CriterionControllerContainer container;

	/** The view for this editor controller */
	protected V view;

	/** Set of entity icons to be used in the editor panel */
	protected EntityIconSet entityIcons;

	/** The criterion we're editing */
	protected TraceabilityMatrixCriterion criterion;

	protected Logger logger = Logger.getLogger(this.getClass());

	/**
	 * Constructs a new controller for the given supercontroller and the given
	 * class of view, including initializing both this controller and a view of
	 * the given class.
	 * 
	 * @param container
	 *            The containing controller
	 * @param viewClass
	 *            The class of view to initialize. Uses reflection to construct
	 *            a new view based on this instance. This view is required to
	 *            have a constructor which can take this object as its only
	 *            parameter.
	 */
	public CriterionController(CriterionControllerContainer container,
			TraceabilityMatrixCriterion criterion, Class<V> viewClass) {
		this.container = container;
		try {
			this.view = viewClass.getConstructor(this.getClass()).newInstance(
					this);
		} catch (Exception e) {
			logger.error(e);
			throw new RuntimeException("Instantiation of "
					+ this.getClass().getCanonicalName() + " failed.");
		}
		this.entityIcons = new EntityIconSet();

		this.initNullableCriterion(criterion);
	}

	/**
	 * @return The view for this controller
	 */
	public V getView() {
		return this.view;
	}

	/**
	 * Initializes this controller with the given criterion, which may be null.
	 * Just delegates to {@link #initCriterion(Criterion)} or
	 * {@link #initNewCriterion()}, as appropriate
	 * 
	 * @param criterion
	 *            The possibly null criterion to initialize with.
	 */
	public void initNullableCriterion(TraceabilityMatrixCriterion criterion) {
		this.criterion = null;
		if (criterion == null) {
			try {
				this.criterion = Database.manager().create(
						TraceabilityMatrixCriterion.class);
			} catch (SQLException e) {
				logger.error(e);
				view
						.error("An error occured while initializing a new criterion");
			}
			initNewCriterion();
		} else {
			this.criterion = criterion;
			initCriterion();
		}
	}

	/**
	 * Gets an array of ids from a collection of entities. Useful for storing a
	 * collection of entities as criterion typedata.
	 * 
	 * @param ents
	 *            The entities whose IDs are desired
	 * @return An array of the ids of ents
	 */
	protected int[] getIDArray(Collection<? extends Entity> ents) {
		int[] ids = new int[ents.size()];

		int index = 0;
		for (Entity ent : ents)
			ids[index++] = ent.getID();

		return ids;
	}

	/**
	 * Gets a list of ids from a collection of entities. Useful for storing a
	 * collection of entities as criterion typedata.
	 * 
	 * @param ents
	 *            The entities whose IDs are desired
	 * @return A list of the ids of ents
	 */
	protected List<Integer> getIDList(Collection<? extends Entity> ents) {
		List<Integer> ids = new ArrayList<Integer>();

		for (Entity ent : ents)
			ids.add(ent.getID());

		return ids;
	}

	/**
	 * Assumes that {@link #criterion}'s typeData references an array of ids for
	 * the given Entity class, and retrieves a list of the entities referenced
	 * by those ID numbers.
	 * 
	 * @param <E>
	 *            The type of entity to fetch
	 * @param clazz
	 *            The class of entity to fetch
	 * @return A list of entities fetched by criterion typedata IDs.
	 */
	protected <E extends Entity> List<E> getCriterionEntities(Class<E> clazz) {
		int[] ids = criterion.getTypeData();
		List<E> ents = new ArrayList<E>();
		for (int id : ids) {
			E ent = Database.manager().get(clazz, id);
			if (ent != null)
				ents.add(ent);
		}
		return ents;
	}

	/**
	 * Sets {@link #criterion} typedata to the ids of the given entities
	 * 
	 * @param ents
	 *            The entities to set criterion typedata to
	 */
	protected void setCriterionEntities(Collection<? extends Entity> ents) {
		criterion.setTypeData(getIDArray(ents));
		criterion.save();
	}

	/**
	 * Assuming that criterion typedata references entity ids, removes all ids
	 * which match the id of any entity in ents from the typedata of the
	 * criterion
	 * 
	 * @param ents
	 *            The entities whose IDs should be removed from the criterion
	 *            data
	 */
	protected void removeEntitiesFromCriterion(Collection<? extends Entity> ents) {
		Set<Integer> ids = new HashSet<Integer>();
		for (int id : criterion.getTypeData())
			ids.add(id);

		ids.removeAll(this.getIDList(ents));

		int[] idArray = new int[ids.size()];
		int index = 0;
		for (int id : ids)
			idArray[index++] = id;

		criterion.setTypeData(idArray);
		criterion.save();
	}

	/**
	 * Assuming that criterion typedata references entity ids, adds all ids
	 * which match the id of any entity in ents and aren't already in the
	 * typedata of the criterion to the typedata of the criterion
	 * 
	 * @param ents
	 *            The entities whose IDs should be added to the criterion data
	 */
	protected void addEntitiesToCriterion(Collection<? extends Entity> ents) {
		Set<Integer> ids = new HashSet<Integer>();
		for (int id : criterion.getTypeData())
			ids.add(id);

		ids.addAll(this.getIDList(ents));

		int[] idArray = new int[ids.size()];
		int index = 0;
		for (int id : ids)
			idArray[index++] = id;

		criterion.setTypeData(idArray);
		criterion.save();
	}

	/**
	 * Initializes this controller based on the set criterion. Generally this
	 * method will deal with telling the view how to initialize itself based on
	 * the data in the criterion.
	 */
	public abstract void initCriterion();

	/**
	 * As {@link #initCriterion()}, but with the preknowledge that
	 * this.criterion has just been newly created. Usually the only thing this
	 * will do differently is setting the criterion type and initializing its
	 * internal id array
	 */
	public abstract void initNewCriterion();

	/**
	 * Generates the set of accepted requirements dynamically based on current
	 * criterion selection.
	 * 
	 * @return The requirements which the criterion specifies
	 */
	public abstract Set<Requirement> getSelectedRequirements();

	public TraceabilityMatrixCriterion getCriterion() {
		return criterion;
	}
}