/*******************************************************************************
 * This file is a part of WPI Suite.
 *
 * Copyright (c) 2009 Worcester Polytechnic Institute, CS3733 D09 - Team 5.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Sabrina Varanelli - Initial API
 *    Dan Bjorge - Implementation
 *******************************************************************************/

package edu.wpi.wpisuite.module.velocity;

import java.awt.Color;
import java.util.Collection;
import java.util.List;

import javax.swing.ImageIcon;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.table.DefaultTableModel;


import org.apache.log4j.Logger;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.AreaRendererEndType;
import org.jfree.chart.renderer.category.AreaRenderer;
import org.jfree.data.category.DefaultCategoryDataset;


import edu.wpi.wpisuite.components.editor.InvalidInputException;
import edu.wpi.wpisuite.data.Iteration;
import edu.wpi.wpisuite.data.errors.InvalidIterationInputException;
import edu.wpi.wpisuite.data.funcs.IterationManager;
import edu.wpi.wpisuite.module.ModuleView;

/**
 * A ModuleView for the Velocity Report Module. Specifies textual and graphical
 * representations of the report.
 */
@SuppressWarnings("serial")
public class VelocityView extends ModuleView {
	private Logger logger = Logger.getLogger(this.getClass());

	// Non-generative variables - may modify
	private Velocity velocity;
	private ChartPanel chartPanel;
	private DefaultCategoryDataset chartData;
	private DefaultTableModel tableData;

	// End non-generative variables

	/*
	 * (non-Javadoc)
	 * 
	 * @see module.ModuleView#updateAllComponents()
	 */
	@Override
	public void updateAllComponents() {
		updateDataset();
		updateTable();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see module.ModuleView#initAddMenuItems()
	 */
	@Override
	public void initAddMenuItems() {
		// Blank by design
	}

	/**
	 * Initializes this view and its components.
	 * 
	 * @param dashboard
	 *            The Module this is a view of.
	 */
	public VelocityView(Velocity velocity) {
		super(velocity, new ImageIcon("resources/OtherIcons/Velocity.PNG"));
		this.velocity = velocity;

		this.initComponents();
		this.initChart();
		this.initTable();
		this.initFilterTree();
	}

	/**
	 * Initializes the filter tree, a listener on its changes, and updates it
	 */
	public void initFilterTree() {
		settingsTree.getCheckingModel().addTreeSelectionListener(
				new TreeSelectionListener() {
					@Override
					public void valueChanged(TreeSelectionEvent e) {
						velocity.updateSelectedIterations(settingsTree
								.getIterations());
					}
				});
		updateFilterTree();
	}

	/**
	 * Updates the filter tree to display the list of iterations to filter over
	 */
	public void updateFilterTree() {
		settingsTree.setIterations(velocity.getSelectableIterations());
	}

	/**
	 * Initializes the textual, table view of the velocity report with any
	 * content which is constant for all of them. Then runs the updater.
	 */
	private void initTable() {
		tableData = new DefaultTableModel(new Object[][] {}, new String[] {
				"Iteration", "Estimated Units", "Actual Units", "Difference" }) {
			Class<?>[] types = new Class[] { java.lang.String.class,
					java.lang.Integer.class, java.lang.Integer.class,
					java.lang.Integer.class };

			@Override
			public Class<?> getColumnClass(int columnIndex) {
				return types[columnIndex];
			}
		};
		textTable.setModel(tableData);
		updateTable();
	}

	/**
	 * Updates the textual, table view of the velocity report with the current
	 * iteration data from the controller
	 */
	private void updateTable() {
		while (tableData.getRowCount() > 0)
			tableData.removeRow(0);

		for (Iteration it : velocity.getOrderedIterations())
			tableData.addRow(getRowData(it));
	}

	/**
	 * Generates a row of values from an iteration
	 * 
	 * @param it
	 *            The iteration to get data from
	 * @return The array of data expected of the velocity table
	 */
	private Object[] getRowData(Iteration it) {
		try {
			int estimated = IterationManager.countEstimateUnit(it);
			int actual = IterationManager.countActualUnit(it);

			Object[] data = new Object[] { it.getName(), estimated, actual,
					actual - estimated };

			return data;
		} catch (InvalidInputException e) {
			logger.error("Cannot get row data for a null iteration");
			return null;
		}

	}

	/**
	 * Initializes the velocity chart with the display values which are constant
	 * for all velocity charts. Also then runs the updater function to display
	 * the specific chart specified at call time
	 */
	private void initChart() {
		initDataset();
		JFreeChart chart = ChartFactory.createAreaChart("Velocity Report",
				"Iteration", "Units", chartData, PlotOrientation.VERTICAL,
				true, false, false);

		// NOW DO SOME OPTIONAL CUSTOMISATION OF THE CHART...
		chart.setBackgroundPaint(Color.white);

		// get a reference to the plot for further customisation...
		final CategoryPlot plot = chart.getCategoryPlot();
		plot.setBackgroundPaint(Color.lightGray);
		plot.setDomainGridlinePaint(Color.white);
		plot.setRangeGridlinePaint(Color.white);

		final AreaRenderer areaRenderer = new AreaRenderer();
		areaRenderer.setSeriesPaint(0, new Color(200, 20, 20, 200));
		areaRenderer.setSeriesPaint(1, new Color(100, 100, 100, 200));
		areaRenderer.setEndType(AreaRendererEndType.TRUNCATE);
		plot.setRenderer(areaRenderer);

		// change the auto tick unit selection to integer units only...
		final NumberAxis rangeAxis = (NumberAxis) plot.getRangeAxis();
		rangeAxis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());
		// OPTIONAL CUSTOMISATION COMPLETED.

		// Now set it to be viewable
		chartPanel = new ChartPanel(chart);
		rightSplitPane.setTopComponent(chartPanel);

	}

	/**
	 * Initializes the data for the chart as a new dataset, then updates the
	 * dataset to the current data as per {@link #updateDataset()}
	 */
	private void initDataset() {
		chartData = new DefaultCategoryDataset();
		updateDataset();
	}

	/**
	 * Updates the dataset defining the velocity chart to match the current
	 * values as returned by the module and selected by the checktree.
	 */
	private void updateDataset() {
		chartData.clear();

		// row keys...
		final String estSeries = "Estimated";
		final String actSeries = "Actual";

		// column keys are iteration names

		List<Iteration> iterations = velocity.getOrderedIterations();

		try {
			for (Iteration iteration : iterations) {
				chartData.addValue(IterationManager
						.countEstimateUnit(iteration), // value
						estSeries, // row
						iteration.getName() // column
						);
				chartData.addValue(IterationManager.countActualUnit(iteration), // value
						actSeries, // row
						iteration.getName() // column
						);
			}
		} catch (InvalidIterationInputException e) {
			this.popErrorMessage(e);
		}
	}

	public void updateSelectableIterations() {
		settingsTree.setIterations(velocity.getSelectableIterations());
	}

	public Collection<Iteration> getSelectedIterations() {
		return settingsTree.getIterations();
	}

	public void updateSelectedIterations() {
		this.updateDataset();
		this.updateTable();
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	// <editor-fold defaultstate="collapsed"
	// desc="Generated Code">//GEN-BEGIN:initComponents
	private void initComponents() {

		primarySplitPane = new javax.swing.JSplitPane();
		rightSplitPane = new javax.swing.JSplitPane();
		graphicalViewPanel = new javax.swing.JPanel();
		textualViewPanel = new javax.swing.JPanel();
		textScrollPane = new javax.swing.JScrollPane();
		textTable = new javax.swing.JTable();
		settingsPanel = new javax.swing.JPanel();
		settingsScrollPane = new javax.swing.JScrollPane();
		settingsTree = new VelocityFilterTree();

		setPreferredSize(new java.awt.Dimension(900, 600));

		primarySplitPane.setDividerLocation(250);
		primarySplitPane.setOneTouchExpandable(true);

		rightSplitPane.setDividerLocation(300);
		rightSplitPane.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
		rightSplitPane.setResizeWeight(0.5);
		rightSplitPane.setOneTouchExpandable(true);

		graphicalViewPanel.setBorder(javax.swing.BorderFactory
				.createTitledBorder("Graphical View"));

		javax.swing.GroupLayout graphicalViewPanelLayout = new javax.swing.GroupLayout(
				graphicalViewPanel);
		graphicalViewPanel.setLayout(graphicalViewPanelLayout);
		graphicalViewPanelLayout.setHorizontalGroup(graphicalViewPanelLayout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGap(0, 630, Short.MAX_VALUE));
		graphicalViewPanelLayout.setVerticalGroup(graphicalViewPanelLayout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGap(0, 272, Short.MAX_VALUE));

		rightSplitPane.setTopComponent(graphicalViewPanel);

		textualViewPanel.setBorder(javax.swing.BorderFactory
				.createTitledBorder("Textual View"));

		textTable.setAutoCreateRowSorter(true);
		textScrollPane.setViewportView(textTable);

		javax.swing.GroupLayout textualViewPanelLayout = new javax.swing.GroupLayout(
				textualViewPanel);
		textualViewPanel.setLayout(textualViewPanelLayout);
		textualViewPanelLayout.setHorizontalGroup(textualViewPanelLayout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addComponent(textScrollPane,
						javax.swing.GroupLayout.DEFAULT_SIZE, 630,
						Short.MAX_VALUE));
		textualViewPanelLayout.setVerticalGroup(textualViewPanelLayout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addComponent(textScrollPane,
						javax.swing.GroupLayout.DEFAULT_SIZE, 265,
						Short.MAX_VALUE));

		rightSplitPane.setRightComponent(textualViewPanel);

		primarySplitPane.setRightComponent(rightSplitPane);

		settingsPanel.setBorder(javax.swing.BorderFactory
				.createTitledBorder("Settings"));

		settingsScrollPane.setViewportView(settingsTree);

		javax.swing.GroupLayout settingsPanelLayout = new javax.swing.GroupLayout(
				settingsPanel);
		settingsPanel.setLayout(settingsPanelLayout);
		settingsPanelLayout.setHorizontalGroup(settingsPanelLayout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addComponent(settingsScrollPane,
						javax.swing.GroupLayout.DEFAULT_SIZE, 237,
						Short.MAX_VALUE));

		settingsPanelLayout
				.setVerticalGroup(settingsPanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								javax.swing.GroupLayout.Alignment.TRAILING,
								settingsPanelLayout
										.createSequentialGroup()
										.addComponent(
												settingsScrollPane,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												529, Short.MAX_VALUE)
										.addContainerGap()));

		primarySplitPane.setLeftComponent(settingsPanel);

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		this.setLayout(layout);
		layout.setHorizontalGroup(layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				primarySplitPane, javax.swing.GroupLayout.DEFAULT_SIZE, 900,
				Short.MAX_VALUE));
		layout.setVerticalGroup(layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				primarySplitPane, javax.swing.GroupLayout.DEFAULT_SIZE, 600,
				Short.MAX_VALUE));
	}// </editor-fold>//GEN-END:initComponents

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JPanel graphicalViewPanel;
	private javax.swing.JSplitPane primarySplitPane;
	private javax.swing.JSplitPane rightSplitPane;
	private javax.swing.JPanel settingsPanel;
	private javax.swing.JScrollPane settingsScrollPane;
	private VelocityFilterTree settingsTree;
	private javax.swing.JScrollPane textScrollPane;
	private javax.swing.JTable textTable;
	private javax.swing.JPanel textualViewPanel;

	// End of variables declaration//GEN-END:variables
}